/*
 $Id: AREA.CPP,v 1.5 2002/07/17 21:29:31 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// Area.cpp
// Implementation file for CArea

#include "stdafx.h"
#include "Area.h"

CSObject * FindArea(CPtrList * pList, CString csName, bool bCreate)
{
	if ( !pList->IsEmpty() )
	{
		POSITION pos = pList->GetHeadPosition();
		while (pos != NULL)
		{
			CArea * pTest = (CArea *) pList->GetNext(pos);
			if ( pTest->m_csName == csName )
				return pTest;
		}
	}
	// Obviously, we didn't find it.
	if ( bCreate )
	{
		CArea * pArea = new (CArea);
		pArea->m_csName = csName;
		if ( pList->IsEmpty() )
			pList->AddHead(pArea);
		else
			pList->AddTail(pArea);
		return pArea;
	}
	return NULL;
}

CArea::CArea()
{
	// Fill in the default values
	this->m_pParentArea = NULL;
	this->m_colAreas.RemoveAll();
	this->m_colRooms.RemoveAll();
	this->m_csName = "";
	this->m_iRainChance = -1;
	this->m_iSnowChance = -1;
	this->m_dwTriggerID = 0xFFFFFFFF;
	this->m_wFlags = 0;
	this->m_dwTrigPeriodic = 0xFFFFFFFF;
	this->m_bCustom = false;
	this->m_pOriginal = NULL;
}

CArea::~CArea()
{
	POSITION pos = this->m_colAreas.GetHeadPosition();
	while ( pos != NULL )
	{
		CArea * pArea = (CArea *) this->m_colAreas.GetNext( pos );
		if ( pArea )
			delete pArea;
	}
	pos = this->m_colRooms.GetHeadPosition();
	while ( pos != NULL )
	{
		CRoom * pRoom = (CRoom *) this->m_colRooms.GetNext( pos );
		if ( pRoom )
			delete pRoom;
	}
	this->m_caRects.RemoveAll();
}

void CArea::ReadFromFile(CStdioFile &csfInput)
{
	BOOL bStatus = true;
	while (bStatus)
	{
		DWORD dwCur = csfInput.GetPosition();
		CString csLine;
		bStatus = csfInput.ReadString(csLine);
		if (bStatus)
		{
			if (csLine.Find("[") == 0)
			{
				// Ooops!!  We're in a new section.  Rewind the file 1 line
				bStatus = false;
				csfInput.Seek((LONG) (dwCur - csfInput.GetPosition()), CFile::current);
			}
			else
			{
				// This is a normal guy
				if (csLine.Find("//") != 0)
				{
					// Not a comment
					this->ReadTag(csLine);
				}
			}
		}
	}
}

void CArea::ReadTag(CString csLine)
{
	CString csTag;
	if (csLine == "")
		return;
	if (csLine.Find("=") != -1)
		csTag = csLine.SpanExcluding("=");
	else
		csTag = csLine.SpanExcluding(" ");
	CString csValue;
	if ( csLine.GetLength() == csTag.GetLength() )
		csValue = "";
	else
		csValue = csLine.Mid(csTag.GetLength() + 1);
	switch(FindTable(csTag, knownMapTags, 30))
	{
	case 0: // GROUP
		this->m_csGroup = csValue;	break;
	case 1: // TITLE
		this->m_csTitle = csValue;	break;
	case 2: // P
		this->m_csPoint = csValue;	break;
	case 3: // RECT
		this->m_caRects.Add(csValue); break;
	case 4: // MIDILIST
		this->m_csMidiList = csValue;	break;
	case 5: // MAGICDAMAGE
		if ( atoi(csValue) ) this->m_wFlags |= REGION_ANTIMAGIC_DAMAGE; break;
	case 6:	// MAGIC
		if ( atoi(csValue) ) this->m_wFlags |= REGION_ANTIMAGIC_ALL; break;
	case 7:	// MARK
	case 8:	// RECALLIN
		if ( atoi(csValue) ) this->m_wFlags |= REGION_ANTIMAGIC_RECALL_IN; break;
	case 9:	// RECALLOUT
		if ( atoi(csValue) ) this->m_wFlags |= REGION_ANTIMAGIC_RECALL_OUT; break;
	case 10: // GATE
	case 11: // GATEIN
		if ( atoi(csValue) ) this->m_wFlags |= REGION_ANTIMAGIC_GATE; break;
	case 12: // GATEOUT
		if ( atoi(csValue) ) this->m_wFlags |= REGION_ANTIMAGIC_GATE; break;
	case 13: // BUILDABLE
		if ( ! atoi(csValue) ) this->m_wFlags |= REGION_FLAG_NOBUILDING; break;
	case 14: // NOBUILD
		if ( atoi(csValue) ) this->m_wFlags |= REGION_FLAG_NOBUILDING; break;
	case 15: // GUARDED
		if ( atoi(csValue) ) this->m_wFlags |= REGION_FLAG_GUARDED; break;
	case 16: // GUARDOWNER
		this->m_csGuardOwner = csValue;	break;
	case 17: // RAINCHANCE
		this->m_iRainChance = atoi(csValue);	break;
	case 18: // COLDCHANCE
		this->m_iSnowChance = atoi(csValue);	break;
	case 19: // LOCAL
		this->m_csLocal = csValue;	break;
	case 20: // GLOBAL
		if ( atoi(csValue) ) this->m_wFlags |= REGION_FLAG_GLOBALNAME; break;
	case 21: // SOUNDS
		this->m_csSounds = csValue;	break;
	case 22: // NODECAY
		if ( atoi(csValue) ) this->m_wFlags |= REGION_FLAG_NODECAY; break;
	case 23: // SAFE
		if ( atoi(csValue) ) this->m_wFlags |= REGION_FLAG_SAFE; break;
	case 24: // UNDERGROUND
		if ( atoi(csValue) ) this->m_wFlags |= REGION_FLAG_UNDERGROUND; break;
	case 25: // ANNOUNCEMENT
		this->m_csAnnouncement = csValue;	break;
	case 26: // ANNOUNCE
		if ( atoi(csValue) ) this->m_wFlags |= REGION_FLAG_ANNOUNCE; break;
	case 27: // TRIGGER
		this->m_dwTriggerID = ahextoi(csValue); break;
	case 28: // FLAGS
		this->m_wFlags = (WORD) ahextoi(csValue); break;
	case 29: // TRIGPERIODIC
		this->m_dwTrigPeriodic = ahextoi(csValue); break;
	default:
		break;
	}
}

void CArea::Copy(CArea *pCopy)
{
	this->m_csGuardOwner = pCopy->m_csGuardOwner;
	this->m_csLocal = pCopy->m_csLocal;
	this->m_csName = pCopy->m_csName;
	this->m_csTitle = pCopy->m_csTitle;
	this->m_iRainChance = pCopy->m_iRainChance;
	this->m_iSnowChance = pCopy->m_iSnowChance;
	this->m_csAnnouncement = pCopy->m_csAnnouncement;
	this->m_dwTriggerID = pCopy->m_dwTriggerID;
	this->m_csGroup = pCopy->m_csGroup;
	this->m_wFlags = pCopy->m_wFlags;
	this->m_pParentArea = pCopy->m_pParentArea;
	for ( int i = 0; i < pCopy->m_caRects.GetSize(); i++ )
		this->m_caRects.Add(pCopy->m_caRects.GetAt(i) );
	this->m_csPoint = pCopy->m_csPoint;
	this->m_dwTrigPeriodic = pCopy->m_dwTrigPeriodic;
}

bool CArea::Write(CStdioFile &csfOutput)
{
	// We assume that the output file is already open.
	CString csTemp;
	csTemp.Format("[AREA %s]\n", this->m_csName);
	csfOutput.WriteString(csTemp);
	WriteTag("TITLE", csfOutput);
	WriteTag("GROUP", csfOutput);
	WriteTag("P", csfOutput);
	WriteTag("RECT", csfOutput);
	WriteTag("GUARDOWNER", csfOutput);
	WriteTag("RAINCHANCE", csfOutput);
	WriteTag("COLDCHANCE", csfOutput);
	WriteTag("LOCAL", csfOutput);
	WriteTag("SOUNDS", csfOutput);
	WriteTag("ANNOUNCEMENT", csfOutput);
	WriteTag("FLAGS", csfOutput);
	WriteTag("TRIGGER", csfOutput);
	WriteTag("TRIGPERIODIC", csfOutput);
	csfOutput.WriteString("\n");
	// Now write out our children
	POSITION pos = this->m_colAreas.GetHeadPosition();
	while ( pos != NULL )
	{
		CArea * pArea = (CArea *) this->m_colAreas.GetNext( pos );
		if ( pArea )
			pArea->Write( csfOutput );
	}
	pos = this->m_colRooms.GetHeadPosition();
	while ( pos != NULL )
	{
		CRoom * pRoom = (CRoom *) this->m_colRooms.GetNext( pos );
		if ( pRoom )
			pRoom->Write( csfOutput );
	}
	return true;
}

bool CArea::WriteBasic(CStdioFile &csfOutput)
{
	CString csTemp;
	csTemp.Format("[AREA %s]\n", this->m_csName);
	csfOutput.WriteString(csTemp);
	WriteTag("GROUP", csfOutput);
	WriteTag("P", csfOutput);
	return true;
}

void CArea::WriteTag(CString csTag, CStdioFile &csfOutput)
{
	CString csValue = "";
	switch(FindTable(csTag, knownMapTags, 30))
	{
	case 0:	// GROUP
		if (this->m_csGroup != "")
			csValue.Format("GROUP=%s\n", this->m_csGroup);
		break;
	case 1: // TITLE
		if (this->m_csTitle != "")
			csValue.Format("TITLE=%s\n", this->m_csTitle);
		break;
	case 2: // P
		if (this->m_csPoint != "")
			csValue.Format("P=%s\n", this->m_csPoint);
		break;
	case 3: // RECT
		if (this->m_caRects.GetUpperBound() != -1)
		{
			for (int i = 0; i < (this->m_caRects.GetUpperBound() + 1); i++)
			{
				CString csTemp = csValue;
				csValue.Format("%sRECT=%s\n", csTemp, this->m_caRects.GetAt(i));
			}
		}
		break;
	case 4: // MIDILIST
		if (this->m_csMidiList != "")
			csValue.Format("MIDILIST=%s\n", this->m_csMidiList);
		break;
	case 5: // MAGICDAMAGE
	case 6: // MAGIC
	case 7: //MARK
	case 8: // RECALLIN
	case 9: // RECALLOUT
	case 10: // GATE
	case 11: // GATEIN
	case 12: // GATEOUT
	case 13: // BUILDABLE
	case 14: // NOBUILD
	case 15: // GUARDED
	case 20: // GLOBAL
	case 22: // NODECAY
	case 23: // SAFE
	case 24: // UNDERGROUND
	case 26: // ANNOUNCE
		break;
	case 16: // GUARDOWNER
		if (this->m_csGuardOwner != "")
			csValue.Format("GUARDOWNER=%s\n", this->m_csGuardOwner);
		break;
	case 17: // RAINCHANCE
		if (this->m_iRainChance != -1)
			csValue.Format("RAINCHANCE %ld\n", this->m_iRainChance);
		break;
	case 18: // COLDCHANCE
		if (this->m_iSnowChance != -1)
			csValue.Format("COLDCHANCE %ld\n", this->m_iSnowChance);
		break;
	case 19: // LOCAL
		if (this->m_csLocal != "")
			csValue.Format("LOCAL=%s\n", this->m_csLocal);
		break;
	case 21: // SOUNDS
		if (this->m_csSounds != "")
			csValue.Format("SOUNDS=%s\n", this->m_csSounds);
		break;
	case 25: // ANNOUNCEMENT
		if (this->m_csAnnouncement != "")
			csValue.Format("ANNOUNCEMENT=%s\n", this->m_csAnnouncement);
		break;
	case 27: // TRIGGER
		if (this->m_dwTriggerID !=  0xFFFFFFFF)
			csValue.Format("TRIGGER=%05x\n", this->m_dwTriggerID);
		break;
	case 28: // FLAGS
		csValue.Format("FLAGS=%05x\n", this->m_wFlags );
		break;
	case 29: // TRIGPERIODIC
		if (this->m_dwTrigPeriodic != 0xFFFFFFFF)
			csValue.Format("TRIGPERIODIC=%05x\n", this->m_dwTrigPeriodic);
		break;
	default:
		break;
	}
	if (csValue != "")
		csfOutput.WriteString(csValue);
	return;
}

bool CArea::IsInside(CArea *pTestArea)
{
	if ( this->m_caRects.GetSize() > 0 )
	{
		for ( int i = 0; i <= this->m_caRects.GetUpperBound(); i++ )
		{
			bool p1Inside = false;
			bool p2Inside = false;
			int x1, y1, x2, y2;
			CString csRect = this->m_caRects.GetAt(i);
			CString csTemp;
			csTemp = csRect.SpanExcluding(",");
			csRect = csRect.Mid( csRect.Find(",") + 1 );
			x1 = atoi(csTemp);
			csTemp = csRect.SpanExcluding(",");
			csRect = csRect.Mid( csRect.Find(",") + 1 );
			y1 = atoi(csTemp);
			csTemp = csRect.SpanExcluding(",");
			csRect = csRect.Mid( csRect.Find(",") + 1 );
			x2 = atoi(csTemp);
			y2 = atoi(csRect);
			for ( int j = 0; j <= pTestArea->m_caRects.GetUpperBound(); j++ )
			{
				int tx1, ty1, tx2, ty2;
				csRect = pTestArea->m_caRects.GetAt(j);
				csTemp = csRect.SpanExcluding(",");
				csRect = csRect.Mid( csRect.Find(",") + 1 );
				tx1 = atoi(csTemp);
				csTemp = csRect.SpanExcluding(",");
				csRect = csRect.Mid( csRect.Find(",") + 1 );
				ty1 = atoi(csTemp);
				csTemp = csRect.SpanExcluding(",");
				csRect = csRect.Mid( csRect.Find(",") + 1 );
				tx2 = atoi(csTemp);
				ty2 = atoi(csRect);
				if ( (x1 >= tx1) && (x1 <= tx2) && (y1 >= ty1) && (y1 <=ty2) )
					p1Inside = true;
				if ( (x2 >= tx1) && (x2 <= tx2) && (y2 >= ty1) && (y2 <=ty2) )
					p2Inside = true;
			}
			if ( ! ( p1Inside && p2Inside ) )
				return false;
		}
		return true;
	}
	else
	{
		// No RECTs defined for the area...Check the teleport point.
		if ( this->m_csPoint != "" )
		{
			int x, y;
			CString csTemp = this->m_csPoint.SpanExcluding(",");
			x = atoi(csTemp);
			csTemp = this->m_csPoint.Mid(m_csPoint.Find(",") + 1);
			csTemp = csTemp.SpanExcluding(",");
			y = atoi(csTemp);
			for ( int i = 0; i <= pTestArea->m_caRects.GetUpperBound(); i++ )
			{
				int tx1, ty1, tx2, ty2;
				CString csRect = pTestArea->m_caRects.GetAt(i);
				csTemp = csRect.SpanExcluding(",");
				csRect = csRect.Mid( csRect.Find(",") + 1 );
				tx1 = atoi(csTemp);
				csTemp = csRect.SpanExcluding(",");
				csRect = csRect.Mid( csRect.Find(",") + 1 );
				ty1 = atoi(csTemp);
				csTemp = csRect.SpanExcluding(",");
				csRect = csRect.Mid( csRect.Find(",") + 1 );
				tx2 = atoi(csTemp);
				ty2 = atoi(csRect);
				if ( (x >= tx1) && (x <= tx2) && (y >=ty1) && (y <=ty2) )
					return true;
			}
		}
	}
	return false;
}

void CArea::Insert(CArea *pParentArea)
{
	POSITION pos = pParentArea->m_colAreas.GetHeadPosition();
	while ( pos != NULL )
	{
		CArea * pArea = (CArea *) pParentArea->m_colAreas.GetNext( pos );
		if ( pArea )
		{
			if ( this->IsInside( pArea ) )
			{
				this->Insert( pArea );
				break;
			}
		}
	}
	if ( this->m_pParentArea == NULL )
	{
		// Is this an area or a room?
		CRoom * pRoom = dynamic_cast <CRoom *> (this);
		if ( pRoom != NULL )
		{
			// this area is a child of the parent area
			if ( pParentArea->m_colRooms.IsEmpty() )
				pParentArea->m_colRooms.AddHead( pRoom );
			else
				pParentArea->m_colRooms.AddTail( pRoom );
			this->m_pParentArea = pParentArea;
		}
		else
		{
			// this area is a child of the parent area
			if ( pParentArea->m_colAreas.IsEmpty() )
				pParentArea->m_colAreas.AddHead( this );
			else
				pParentArea->m_colAreas.AddTail( this );
			this->m_pParentArea = pParentArea;
		}
	}
}

CRoom::CRoom()
{
}

CRoom::~CRoom()
{
}


bool CRoom::Write(CStdioFile &csfOutput)
{
	// We assume that the output file is already open.
	CString csTemp;
	csTemp.Format("[ROOM %s]\n", this->m_csName);
	csfOutput.WriteString(csTemp);
	WriteTag("GROUP", csfOutput);
	WriteTag("TITLE", csfOutput);
	WriteTag("P", csfOutput);
	WriteTag("RECT", csfOutput);
	WriteTag("MIDILIST", csfOutput);
	WriteTag("GUARDOWNER", csfOutput);
	WriteTag("RAINCHANCE", csfOutput);
	WriteTag("COLDCHANCE", csfOutput);
	WriteTag("LOCAL", csfOutput);
	WriteTag("SOUNDS", csfOutput);
	WriteTag("ANNOUNCEMENT", csfOutput);
	WriteTag("FLAGS", csfOutput);
	csfOutput.WriteString("\n");
	return true;
}

void CRoom::WriteTag(CString csTag, CStdioFile &csfOutput)
{
	CString csValue = "";
	switch(FindTable(csTag, knownMapTags, 27))
	{
	case 0:	// GROUP
		if (this->m_csGroup != "")
			csValue.Format("GROUP=%s\n", this->m_csGroup);
		break;
	case 1: // TITLE
		if (this->m_csTitle != "")
			csValue.Format("TITLE=%s\n", this->m_csTitle);
		break;
	case 2: // P
		if (this->m_csPoint != "")
			csValue.Format("P=%s\n", this->m_csPoint);
		break;
	case 3: // RECT
		if (this->m_caRects.GetUpperBound() != -1)
		{
			for (int i = 0; i < (this->m_caRects.GetUpperBound() + 1); i++)
			{
				CString csTemp = csValue;
				csValue.Format("%sRECT=%s\n", csTemp, this->m_caRects.GetAt(i));
			}
		}
		break;
	case 4: // MIDILIST
		if (this->m_csMidiList != "")
			csValue.Format("MIDILIST=%s\n", this->m_csMidiList);
		break;
	case 5: // MAGICDAMAGE
	case 6: // MAGIC
	case 7: //MARK
	case 8: // RECALLIN
	case 9: // RECALLOUT
	case 10: // GATE
	case 11: // GATEIN
	case 12: // GATEOUT
	case 13: // BUILDABLE
	case 14: // NOBUILD
	case 15: // GUARDED
	case 20: // GLOBAL
	case 22: // NODECAY
	case 23: // SAFE
	case 24: // UNDERGROUND
	case 26: // ANNOUNCE
		break;
	case 16: // GUARDOWNER
		if (this->m_csGuardOwner != "")
			csValue.Format("GUARDOWNER=%s\n", this->m_csGuardOwner);
		break;
	case 17: // RAINCHANCE
		if (this->m_iRainChance != -1)
			csValue.Format("RAINCHANCE %ld\n", this->m_iRainChance);
		break;
	case 18: // COLDCHANCE
		if (this->m_iSnowChance != -1)
			csValue.Format("COLDCHANCE %ld\n", this->m_iSnowChance);
		break;
	case 19: // LOCAL
		if (this->m_csLocal != "")
			csValue.Format("LOCAL=%s\n", this->m_csLocal);
		break;
		break;
	case 21: // SOUNDS
		if (this->m_csSounds != "")
			csValue.Format("SOUNDS=%s\n", this->m_csSounds);
		break;
	case 25: // ANNOUNCEMENT
		if (this->m_csAnnouncement != "")
			csValue.Format("ANNOUNCEMENT=%s\n", this->m_csAnnouncement);
		break;
	case 28: // FLAGS
		csValue.Format("FLAGS=%05x", this->m_wFlags );
		break;
	default:
		break;
	}
	if (csValue != "")
		csfOutput.WriteString(csValue);
	return;
}

bool CArea::IsInside(CPoint p)
{
	for ( int i = 0; i < this->m_caRects.GetSize(); i++ )
	{
		int x1, x2, y1, y2;
		CString csRect = this->m_caRects.GetAt(i);
		CString csTemp;
		csTemp = csRect.SpanExcluding(",");
		csRect = csRect.Mid( csRect.Find(",") + 1 );
		x1 = atoi(csTemp);
		csTemp = csRect.SpanExcluding(",");
		csRect = csRect.Mid( csRect.Find(",") + 1 );
		y1 = atoi(csTemp);
		csTemp = csRect.SpanExcluding(",");
		csRect = csRect.Mid( csRect.Find(",") + 1 );
		x2 = atoi(csTemp);
		y2 = atoi(csRect);
		if ( ( p.x >= x1 ) && ( p.x <= x2 ) && ( p.y >= y1 ) && ( p.y <= y2 ) )
			return true;
	}
	return false;
}

bool CArea::Overlaps(CArea *pTest)
{
	for ( int i = 0; i <= this->m_caRects.GetUpperBound(); i++ )
	{
		int x1, y1, x2, y2;
		CString csRect = this->m_caRects.GetAt(i);
		CString csTemp;
		csTemp = csRect.SpanExcluding(",");
		csRect = csRect.Mid( csRect.Find(",") + 1 );
		x1 = atoi(csTemp);
		csTemp = csRect.SpanExcluding(",");
		csRect = csRect.Mid( csRect.Find(",") + 1 );
		y1 = atoi(csTemp);
		csTemp = csRect.SpanExcluding(",");
		csRect = csRect.Mid( csRect.Find(",") + 1 );
		x2 = atoi(csTemp);
		y2 = atoi(csRect);
		for ( int j = 0; j <= pTest->m_caRects.GetUpperBound(); j++ )
		{
			int tx1, ty1, tx2, ty2;
			csRect = pTest->m_caRects.GetAt(j);
			csTemp = csRect.SpanExcluding(",");
			csRect = csRect.Mid( csRect.Find(",") + 1 );
			tx1 = atoi(csTemp);
			csTemp = csRect.SpanExcluding(",");
			csRect = csRect.Mid( csRect.Find(",") + 1 );
			ty1 = atoi(csTemp);
			csTemp = csRect.SpanExcluding(",");
			csRect = csRect.Mid( csRect.Find(",") + 1 );
			tx2 = atoi(csTemp);
			ty2 = atoi(csRect);
			if ( (x1 >= tx1) && (x1 <= tx2) && (y1 >= ty1) && (y1 <=ty2) )
				return true;
			if ( (x2 >= tx1) && (x2 <= tx2) && (y2 >= ty1) && (y2 <=ty2) )
				return true;
			if ( (tx1 >= x1) && (tx2 <= x1) && (ty1 >= y1) && (ty2 <= y1) )
				return true;
			if ( (tx1 >= x2) && (tx2 <= x2) && (ty1 >= y2) && (ty2 <= y2) )
				return true;
		}
	}
	return false;
}

const char * pLocTags [] = 
{
	"CATEGORY",
	"SUBSECTION",
	"DESCRIPTION",
	"POINT",
};

bool CLocation::Read(CStdioFile &csfInput)
{
	BOOL bStatus = TRUE;
	while (bStatus)
	{
		DWORD dwCurrent = csfInput.GetPosition();
		CString csLine;
		bStatus = csfInput.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( csLine.Find("[") != -1 )
		{
			csfInput.Seek((LONG) (dwCurrent - csfInput.GetPosition()), CFile::current);
			break;
		}
		csLine = csLine.SpanExcluding("//");
		CString csKey, csValue;
		if ( csLine.Find("=") != -1 && (csLine.Find("=") != csLine.GetLength()) )
		{
			csKey = csLine.SpanExcluding("=");
			csValue = csLine.Mid(csLine.Find("=") + 1);
			switch ( FindTable(csKey, pLocTags, 4) )
			{
			case 0:
				m_csCategory = csValue; break;
			case 1:
				m_csSubsection = csValue; break;
			case 2:
				m_csDescription = csValue; break;
			case 3:
				m_csPoint = csValue; break;
			}
		}
	}

	return bStatus ? true : false;
}

bool CLocation::Write(CStdioFile &csfOutput)
{
	CString csLine;
	csfOutput.WriteString("\n[LOCATION]\n");
	for ( int i = 0; i < 4; i++ )
	{
		CString csValue;
		switch (i)
		{
		case 0:
			csValue = m_csCategory; break;
		case 1:
			csValue = m_csSubsection; break;
		case 2:
			csValue = m_csDescription; break;
		case 3:
			csValue = m_csPoint; break;
		}
		csLine.Format("%s=%s\n", pLocTags[i], csValue);
		csfOutput.WriteString(csLine);
	}
	return true;
}

bool CRoom::WriteBasic(CStdioFile &csfOutput)
{
	CString csTemp;
	csTemp.Format("[ROOM %s]\n", this->m_csName);
	csfOutput.WriteString(csTemp);
	WriteTag("GROUP", csfOutput);
	WriteTag("P", csfOutput);
	return true;
}
