/*
 $Id: AREA.H,v 1.6 2002/07/17 21:29:31 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// Area.h
// Definition file for CArea

#ifndef __AREA_H__
#define __AREA_H__

#include "stdafx.h"
#include "common.h"

#define REGION_ANTIMAGIC_ALL		0x0001	// All magic banned here.
#define REGION_ANTIMAGIC_RECALL_IN	0x0002	// Teleport,recall in to this, and mark
#define REGION_ANTIMAGIC_RECALL_OUT	0x0004	// can't recall out of here.
#define REGION_ANTIMAGIC_GATE		0x0008
#define REGION_ANTIMAGIC_TELEPORT	0x0010	// Can't teleport into here.
#define REGION_ANTIMAGIC_DAMAGE		0x0020	// just no bad magic here
#define REGION_FLAG_SHIP			0x0040	// This is a ship region. ship commands
#define REGION_FLAG_NOBUILDING		0x0080	// No building in this area
#define REGION_FLAG_GLOBALNAME		0x0100	// Make sure the name is avail globally.
#define REGION_FLAG_ANNOUNCE		0x0200	// Announce to all who enter.
#define REGION_FLAG_INSTA_LOGOUT	0x0400	// Instant Log out is allowed here. (hotel)
#define REGION_FLAG_UNDERGROUND		0x0800	// dungeon type area. (no weather)
#define REGION_FLAG_NODECAY			0x1000	// Things on the ground don't decay here.
#define REGION_FLAG_SAFE			0x2000	// This region is safe from all harm.
#define REGION_FLAG_GUARDED			0x4000
#define REGION_DYNAMIC				0x8000	// Must be saved in the WORLD.SCP file (not multi)

class CArea : public CSObject
{
public:
	bool Overlaps(CArea *pTest);
	bool IsInside(CPoint p);
	bool m_bCustom;
	CArea * m_pOriginal;
	DWORD m_dwTriggerID;
	DWORD m_dwTrigPeriodic;
	bool m_bAnnounce;
	CString m_csAnnouncement;
	CString m_csSounds;
	CString m_csLocal;
	CString m_csGuardOwner;
	CString m_csMidiList;
	CString m_csPoint;
	CString m_csTitle;
	int m_iSnowChance;
	int m_iRainChance;
	CString m_csName;
	CString m_csGroup;
	WORD m_wFlags;
	CStringArray m_caRects;
	CPtrList m_colAreas;
	CPtrList m_colRooms;
	CArea * m_pParentArea;
	CArea();
	virtual ~CArea();
	void ReadFromFile(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);
	bool WriteBasic(CStdioFile &csfOutput);
	void Copy(CArea *pCopy);
	bool IsInside(CArea *pTestArea);
	void Insert(CArea * pParentArea);
protected:
	void ReadTag(CString csLine);
	void WriteTag(CString csTag, CStdioFile &csfOutput);
};

class CRoom : public CArea
{
public:
	bool WriteBasic(CStdioFile &csfOutput);
	CRoom();
	virtual ~CRoom();
	bool Write(CStdioFile &csfOutput);
protected:
	void WriteTag(CString csTag, CStdioFile &csfOutput);
};

class CLocation : public CSObject
{
public:
	CLocation() {m_bCustom = false;};
	virtual ~CLocation() {};
	bool Read(CStdioFile &csfInput);
	bool Write(CStdioFile &csfOutput);

	bool m_bCustom;
	CString m_csPoint;
};

#endif