/*
 $Id: CNPC.CPP,v 1.11 2002/11/17 04:25:27 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

#include "stdafx.h"
#include "common.h"
#include "CNPC.h"
#include "RemoteConnection.h"
#include "CScriptObjects.h"
#include "RemoteConsole.h"

const char * knownNPCTags[] =
{
"DEFNAME",			// 0
"NAME",
"ICON",
"ANIM",
"SOUND",

"CAN",				// 5
"ATT",
"ARMOR",
"RESOURCES",
"FOODTYPE",

"STR",				// 10
"DEX",
"INT",

"Alchemy",
"Anatomy",
"Animallore",		// 15
"Archery",
"Armslore",
"Begging",
"Blacksmithing",
"Bowcraft",			// 20
"Camping",
"Carpentry",
"Cartography",
"Cooking",
"Detectinghidden",	// 25
"Enticement",
"Evaluatingintel",
"Fencing",
"Fishing",
"Forensics",		// 30
"Healing",
"Herding",
"Hiding",
"Itemid",
"Inscription",		// 35
"Lockpicking",
"Lumberjacking",
"Macefighting",
"Magery",
"Magicresistance",	// 40
"Meditation",
"Mining",
"Musicianship",
"Necromancy",
"Parrying",			// 45
"Peacemaking",
"Poisoning",
"Provocation",
"Removetrap",
"Snooping",			// 50
"Spiritspeak",
"Stealing",
"Stealth",
"Swordsmanship",
"Tactics",			// 55
"Tailoring",
"Taming",
"Tasteid",
"Tinkering",
"Tracking",			// 60
"Veterinary",
"Wrestling",

"NPC",
"FAME",
"KARMA",			// 65
"DESIRES",
"NEED",
"HIREDAYWAGE",

"TSPEECH",
"TEVENTS",			// 70
"SPEECH",
"EVENTS",

"ID",
"COLOR",
"ITEM",				// 75
"BUY",
"SELL",
"ITEMNEWBIE",

"CATEGORY",	
"SUBSECTION",		// 80
"DESCRIPTION",
"NOINDEX",
"MORE1",
"MORE2",
"SPELLS",			// 85

};


CNPC::CNPC()
{
	m_bType = TYPE_CHAR;
	m_csAnim = "";
	m_csCan = "";
	m_csColor = "";
	m_csFoodType = "";
	m_csIcon = "";
	m_wID = 0;
	m_csNPC = "";
	m_csSound = "";
	m_wSpeech = 0;
	m_csCategory = "<none>";
	m_csSubsection = "<none>";
	m_csDescription = "<unnamed>";
	m_csValue = "";
	m_csPreviousTag = "";
	m_pLastLootItem = NULL;
	m_bNoIndex = false;
	m_iDailyWage = 0;
	m_bHasSpellbook = false;
	m_dwSpellsMore1 = 0;
	m_dwSpellsMore2 = 0;
	m_bCustom = false;
	m_csDesires = "";
	m_csDefName = "";
	m_csNeed = "";
	for ( int i = 0; i < SKILL_QTY; i++ )
	{
		m_irSkill[i].min = 0;
		m_irSkill[i].max = 0;
	}
}

CNPC::~CNPC()
{
	// Free the objects in the object lists
	while (!m_olBuyItems.IsEmpty())
	{
		CVendorItem * pOb = (CVendorItem *) m_olBuyItems.RemoveHead();
		delete pOb;
	}
	while (!m_olSellItems.IsEmpty())
	{
		CVendorItem * pOb = (CVendorItem *) m_olSellItems.RemoveHead();
		delete pOb;
	}
	while (!m_olItems.IsEmpty())
	{
		CNPCLootItem * pOb = (CNPCLootItem *) m_olItems.RemoveHead();
		delete pOb;
	}
	this->m_saSpells.RemoveAll();
	this->m_csTEvents.RemoveAll();
	this->m_csTSpeech.RemoveAll();
}

bool CNPC::Read(CStdioFile &csfInput)
{
	BOOL bStatus = true;
	CString csLine;
	BOOL bNextItemReached = false;

	while (bStatus)
	{
		DWORD dwCur = csfInput.GetPosition();
		bStatus = csfInput.ReadString(csLine);
		if (!bStatus)
			break;
		csLine.TrimLeft();
		if (csLine != "")
		{
			if (csLine[0] == '[')
			{
				csfInput.Seek((LONG) (dwCur - csfInput.GetPosition()), CFile::current);
				break;
			}
			CString csTmp = csLine;
			csTmp.MakeUpper();
			this->ReadTag(csLine);
		}
	}
	return ( bStatus ? true : false );
}

void CNPC::ReadTag(CString csLine)
{
	CString csTag;
	csTag = csLine.Left(csLine.Find("="));
	if ( csTag == "" )
		csTag = csLine;
	csTag.MakeUpper();
	CString csValue = csLine.Right(csLine.GetLength() - (csTag.GetLength() + 1));
	csValue.TrimLeft();
	csValue.TrimRight();
	csValue.Remove(0x0d);
	int iTagIndex = FindTable(csTag, knownNPCTags, 86);
	switch(iTagIndex)
	{
	case 0:
		{
			/*
			csValue.MakeUpper();
			this->m_csDefName = csValue;
			CDef * pDef = new CDef;
			pDef->m_csValue = csValue;
			pDef->m_expression.Read(this->m_csValue);
			CDrewskyApp * pApp = (CDrewskyApp *) AfxGetApp();
			int iOld = pApp->m_pScripts->m_aDefs.Find(csValue);
			if ( iOld != -1 )
			{
				CDef * pOld = (CDef *) pApp->m_pScripts->m_aDefs.GetAt(iOld);
				pApp->m_pScripts->m_aDefs.RemoveAt(iOld);
				m_aDefs.FreeExtra();
				delete pOld;
			}
			pApp->m_pScripts->m_aDefs.Insert(pDef);
			*/
		}
		m_csDefName = csValue;
		break;
	case 1:
		this->m_csNPCName = csValue; break;
	case 2:
		csValue.MakeUpper(); this->m_csIcon = csValue; break;
	case 3:
		csValue.MakeUpper(); this->m_csAnim = csValue; break;
	case 4:
		csValue.MakeUpper(); this->m_csSound = csValue; break;
	case 5:
		csValue.MakeUpper(); this->m_csCan = csValue; break;
	case 6:
		this->m_irAtt.ReadRange(csValue); break;
	case 7:
		this->m_irDef.ReadRange(csValue); break;
	case 8:
		this->m_csResources = csValue; break;
	case 9:
		this->m_csFoodType = csValue; break;
	case 10:
		this->m_irStr.ReadRange(csValue); break;
	case 11:
		this->m_irDex.ReadRange(csValue); break;
	case 12:
		this->m_irInt.ReadRange(csValue); break;
	case 13: case 14: case 15: case 16: case 17: case 18: case 19: case 20: case 21: case 22:
	case 23: case 24: case 25: case 26: case 27: case 28: case 29: case 30: case 31: case 32:
	case 33: case 34: case 35: case 36: case 37: case 38: case 39: case 40: case 41: case 42:
	case 43: case 44: case 45: case 46: case 47: case 48: case 49: case 50: case 51: case 52:
	case 53: case 54: case 55: case 56: case 57: case 58: case 59: case 60: case 61: case 62:
		this->m_irSkill[iTagIndex - 13].ReadRange(csValue); break;
	case 63:
		csValue.MakeUpper(); this->m_csNPC = csValue; break;
	case 64:
		this->m_irFame.ReadRange(csValue); break;
	case 65:
		this->m_irKarma.ReadRange(csValue); break;
	case 66:
		this->m_csDesires = csValue; break;
	case 67:
		this->m_csNeed = csValue; break;
	case 68:
		this->m_iDailyWage = atoi(csValue); break;
	case 69:
		this->m_csTSpeech.Add(csValue); break;
	case 70:
		this->m_csTEvents.Add(csValue); break;
	case 71:
		this->m_wSpeech = (WORD) ahextoi(csValue); break;
	case 72:
		this->m_csEvents = csValue; break;
	case 73:
		// This could be a def
		if (isdigit(csValue[0]))
			this->m_wID = (WORD) ahextoi(csValue);
		else
			// Look this one up in the defs file
			this->m_wID = (WORD) DefLookup(csValue);
		break;
	case 74:
		if (m_csPreviousTag == "ITEM" || m_csPreviousTag == "ITEMNEWBIE" )
			this->m_pLastLootItem->m_csColor = csValue;
		else
		{
			csValue.MakeUpper();
			this->m_csColor = csValue;
		}
		break;
	case 75:
		{
			if (csValue.CompareNoCase("spell_book") == 0)
			{
				m_bHasSpellbook = true;
				break;
			}
			CNPCLootItem * pLoot = new (CNPCLootItem);
			csValue.TrimRight();
			if (csValue.Find("{") == -1)
			{
				if (csValue.Find(",") != -1)
					pLoot->m_csName = csValue.Left(csValue.Find(","));
				else
					pLoot->m_csName = csValue;
				if (csValue.Find(",") != -1)
				{
					CString csMore = csValue.Mid(pLoot->m_csName.GetLength() + 1);
					if (csMore != "")
					{
						if (csMore.Find(",") != -1)
						{
							CString csProb;
							CString csAmount = csMore.Left(csMore.Find(","));
							pLoot->m_irAmount.ReadRange(csAmount);
							csProb = csMore.Mid(csAmount.GetLength() + 1);
							if (csProb != "")
							{
								if (csProb.Find("R") != -1)
								{
									csProb = csProb.Mid(csProb.Find("R") + 1);
									pLoot->m_iProbability = atoi(csProb);
								}
							}
						}
						else
						{
							if (csMore.Find("R") == -1)
								pLoot->m_irAmount.ReadRange(csMore);
							else
							{
								pLoot->m_irAmount.ReadRange("1");
								CString csProb = csMore.Mid(csMore.Find("R") + 1);
								pLoot->m_iProbability = atoi(csProb);
							}
						}
					}
				}
			}
			else
				pLoot->m_csWeight = csValue;
			if (m_olItems.IsEmpty())
				m_olItems.AddHead(pLoot);
			else
				m_olItems.AddTail(pLoot);
			this->m_pLastLootItem = pLoot;
		}
		break;
	case 76:
		{
			CVendorItem * pBuy = new (CVendorItem);
			if (csValue.Find(",") != -1)
				pBuy->m_csName = csValue.Left(csValue.Find(","));
			else
				pBuy->m_csName = csValue;
			if (csValue.Find(",") != -1)
			{
				CString csMore = csValue.Mid(pBuy->m_csName.GetLength() + 1);
				if (csMore != "")
				{
					if (csMore.Find(",") != -1)
					{
						CString csAmount = csMore.Left(csMore.Find(","));
						pBuy->m_irAmount.ReadRange(csAmount);
						CString csProb = csMore.Mid(csAmount.GetLength() + 1);
						if (csProb != "")
						{
							if (csProb.Find("R") != -1)
							{
								csProb = csProb.Mid(csProb.Find("R") + 1);
								pBuy->m_iProbability = atoi(csProb);
							}
						}
					}
					else
						pBuy->m_irAmount.ReadRange(csMore);
				}
			}
			if (m_olBuyItems.IsEmpty())
				m_olBuyItems.AddHead(pBuy);
			else
				m_olBuyItems.AddTail(pBuy);
		}
		break;
	case 77:
		{
			CVendorItem * pSell = new (CVendorItem);
			if (csValue.Find(",") != -1)
				pSell->m_csName = csValue.Left(csValue.Find(","));
			else
				pSell->m_csName = csValue;
			if (csValue.Find(",") != -1)
			{
				CString csMore = csValue.Mid(pSell->m_csName.GetLength() + 1);
				if (csMore != "")
				{
					if (csMore.Find(",") != -1)
					{
						CString csAmount = csMore.Left(csMore.Find(","));
						pSell->m_irAmount.ReadRange(csAmount);
						CString csProb = csMore.Mid(csAmount.GetLength() + 1);
						if (csProb != "")
						{
							if (csProb.Find("R") != -1)
							{
								csProb = csProb.Mid(csProb.Find("R") + 1);
								pSell->m_iProbability = atoi(csProb);
							}
						}
					}
					else
						pSell->m_irAmount.ReadRange(csMore);
				}
			}
			if (m_olSellItems.IsEmpty())
				m_olSellItems.AddHead(pSell);
			else
				m_olSellItems.AddTail(pSell);
		}
		break;
	case 78:
		{
			if (csValue.CompareNoCase("spell_book") == 0)
			{
				m_bHasSpellbook = true;
				break;
			}
			CNPCLootItem * pLoot = new (CNPCLootItem);
			pLoot->m_bNewbie = true;
			csValue.TrimRight();
			if (csValue.Find("{") == -1)
			{
				if (csValue.Find(",") != -1)
					pLoot->m_csName = csValue.Left(csValue.Find(","));
				else
					pLoot->m_csName = csValue;
				if (csValue.Find(",") != -1)
				{
					CString csMore = csValue.Mid(pLoot->m_csName.GetLength() + 1);
					if (csMore != "")
					{
						if (csMore.Find(",") != -1)
						{
							CString csProb;
							CString csAmount = csMore.Left(csMore.Find(","));
							pLoot->m_irAmount.ReadRange(csAmount);
							csProb = csMore.Mid(csAmount.GetLength() + 1);
							if (csProb != "")
							{
								if (csProb.Find("R") != -1)
								{
									csProb = csProb.Mid(csProb.Find("R") + 1);
									pLoot->m_iProbability = atoi(csProb);
								}
							}
						}
						else
						{
							if (csMore.Find("R") == -1)
								pLoot->m_irAmount.ReadRange(csMore);
							else
							{
								pLoot->m_irAmount.ReadRange("1");
								CString csProb = csMore.Mid(csMore.Find("R") + 1);
								pLoot->m_iProbability = atoi(csProb);
							}
						}
					}
				}
			}
			else
				pLoot->m_csWeight = csValue;
			if (m_olItems.IsEmpty())
				m_olItems.AddHead(pLoot);
			else
				m_olItems.AddTail(pLoot);
			this->m_pLastLootItem = pLoot;
		}
		break;
	case 79:
		this->m_csCategory = csValue; break;
	case 80:
		this->m_csSubsection = csValue; break;
	case 81:
		this->m_csDescription = csValue; break;
	case 82:
		this->m_bNoIndex = true;
	case 83:	// MORE1
		if (m_bHasSpellbook)
			this->m_dwSpellsMore1 |= ahextoi(csValue);
		break;
	case 84:	// MORE2
		if (m_bHasSpellbook)
			this->m_dwSpellsMore2 |= ahextoi(csValue);
		break;
	case 85:	// SPELLS
		if (m_bHasSpellbook)
		{
			int iSpell = 0;
			for ( int i = 0; i < Main->m_pScripts->m_tables.m_spells.GetSize(); i++)
			{
				CTSpellDef * pSpell = (CTSpellDef *) Main->m_pScripts->m_tables.m_spells.GetAt(i);
				if ( !pSpell )
					continue;
				if ( pSpell->m_csName.CompareNoCase(csValue) == 0 )
				{
					iSpell = i;
					break;
				}
			}
			if (iSpell != 0)
			{
				iSpell = iSpell - 1;
				if (iSpell < 32)
					m_dwSpellsMore1 |= 1<<iSpell;
				else if (iSpell < 64)
					m_dwSpellsMore2 |= 1<<(iSpell - 32);
				else
					m_saSpells.Add(csValue);
			}
			else
				m_saSpells.Add(csValue);
		}
		break;
	default:
		// Don't know about this tag...simply ignore it for now
		break;
	}
	m_csPreviousTag = csTag;
}


void CNPC::Copy(CNPC *ob)
{
	// Copy constructor
	m_csAnim = ob->m_csAnim;
	m_csCan = ob->m_csCan;
	m_csCategory = ob->m_csCategory;
	m_csColor = ob->m_csColor;
	m_csFoodType = ob->m_csFoodType;
	m_csDescription = ob->m_csDescription;
	m_csNPC = ob->m_csNPC;
	m_csNPCName = ob->m_csNPCName;
	m_csPreviousTag = ob->m_csPreviousTag;
	m_csResources = ob->m_csResources;
	m_csSound = ob->m_csSound;
	m_csSubsection = ob->m_csSubsection;
	m_csValue = ob->m_csSubsection;
	m_csValue = ob->m_csValue;
	m_irAtt = ob->m_irAtt;
	m_irDef = ob->m_irDef;
	m_irDex = ob->m_irDex;
	m_irFame = ob->m_irFame;
	m_irInt = ob->m_irInt;
	m_irKarma = ob->m_irKarma;
	m_irStr = ob->m_irStr;
	m_pLastLootItem = ob->m_pLastLootItem;
	m_csIcon = ob->m_csIcon;
	m_wID = ob->m_wID;
	m_wSpeech = ob->m_wSpeech;
	m_bNoIndex = ob->m_bNoIndex;
	m_iDailyWage = ob->m_iDailyWage;
	if (ob->m_bHasSpellbook)
	{
		m_bHasSpellbook = true;
		m_dwSpellsMore1 = ob->m_dwSpellsMore1;
		m_dwSpellsMore2 = ob->m_dwSpellsMore2;
	}
	for ( int i = 0; i < SKILL_QTY; i++ )
	{
		this->m_irSkill[i].max = ob->m_irSkill[i].max;
		this->m_irSkill[i].min= ob->m_irSkill[i].min;
	}
	
	// Make sure the object lists are clean
	while (!m_olBuyItems.IsEmpty())
	{
		CSObject * pOb = (CSObject *) m_olBuyItems.RemoveHead();
		delete pOb;
	}
	while (!m_olSellItems.IsEmpty())
	{
		CSObject * pOb = (CSObject *) m_olSellItems.RemoveHead();
		delete pOb;
	}
	while (!m_olItems.IsEmpty())
	{
		CSObject * pOb = (CSObject *) m_olItems.RemoveHead();
		delete pOb;
	}
	this->m_saSpells.RemoveAll();

	// Now for the object lists
	if (!ob->m_olBuyItems.IsEmpty())
	{
		POSITION pos = ob->m_olBuyItems.GetHeadPosition();
		while (pos != NULL)
		{
			CVendorItem * pItem = (CVendorItem *) ob->m_olBuyItems.GetNext(pos);
			if (pItem != NULL)
			{
				CVendorItem *pNew = new (CVendorItem);
				pNew->m_csName = pItem->m_csName;
				pNew->m_iProbability = pItem->m_iProbability;
				pNew->m_irAmount = pItem->m_irAmount;
				if (m_olBuyItems.IsEmpty())
					m_olBuyItems.AddHead(pNew);
				else
					m_olBuyItems.AddTail(pNew);
			}
		}
	}
	if (!ob->m_olSellItems.IsEmpty())
	{
		POSITION pos = ob->m_olSellItems.GetHeadPosition();
		while (pos != NULL)
		{
			CVendorItem * pItem = (CVendorItem *) ob->m_olSellItems.GetNext(pos);
			if (pItem != NULL)
			{
				CVendorItem *pNew = new (CVendorItem);
				pNew->m_csName = pItem->m_csName;
				pNew->m_iProbability = pItem->m_iProbability;
				pNew->m_irAmount = pItem->m_irAmount;
				if (m_olSellItems.IsEmpty())
					m_olSellItems.AddHead(pNew);
				else
					m_olSellItems.AddTail(pNew);
			}
		}
	}
	if (!ob->m_olItems.IsEmpty())
	{
		POSITION pos = ob->m_olItems.GetHeadPosition();
		while (pos != NULL)
		{
			CNPCLootItem * pItem = (CNPCLootItem *) ob->m_olItems.GetNext(pos);
			if (pItem != NULL)
			{
				CNPCLootItem *pNew = new (CNPCLootItem);
				pNew->Copy( pItem );
				if (m_olItems.IsEmpty())
					m_olItems.AddHead(pNew);
				else
					m_olItems.AddTail(pNew);
			}
		}
	}
	for (i = 0; i <= ob->m_saSpells.GetUpperBound(); i++)
		this->m_saSpells.SetAtGrow(i, ob->m_saSpells.GetAt(i));

}

bool CNPC::Save(CString csFileRoot)
{
#ifndef _AXIS
	CString csFileName, csTmpFileName;
	csFileName.Format(Main->GetProfileType() == PROFILE_REMOTE ? "%s.rmt" : "%s.scp", csFileRoot);
	if (Main->GetProfileType() == PROFILE_REMOTE)
		csTmpFileName.Format("remoteaxis.tmp");
	else
		csTmpFileName.Format("%s.tmp", csFileRoot);
	CStdioFile csfInput;
	CStdioFile csfOutput;
	if (!csfInput.Open(csFileName, CFile::modeRead | CFile::shareDenyNone))
		return false;
	if (!csfOutput.Open(csTmpFileName, CFile::modeWrite | CFile::modeCreate | CFile::shareDenyWrite))
	{
		csfInput.Close();
		return false;
	}
	// We need to read the input file and search for the correct place to put this item
	// If the item is already there, we need to overwrite the existing one (except for certain things)
	// If the item isn't already there, we need to insert it in numerical order.
	CString csIn, csOut, endl;
	endl.Format("\n");

	BOOL bStatus = true;
	BOOL bNPCWritten = false;

	while (bStatus)
	{
		bool bTagsHandled[86] = {false};
		// Read a line from the file...
		if (bStatus)
			bStatus = csfInput.ReadString(csIn);
		// If it starts with a '[', it is the beginning of an item description
		if (!bStatus && !bNPCWritten)
		{
			// We hit the end of the file and didn't find it...
			// Put this item at the end
			this->Write(csfOutput);
			bNPCWritten = true;
			break;
		}
		if (csIn != "")
		{
			if ((csIn[0] == '[') && !bNPCWritten)
			{
				// Get the item number
				CString csLine = csIn.Mid(1);
				csLine = csLine.SpanExcluding("]");
				DWORD dwThisItem = ahextoi(csLine);
				if (dwThisItem < ahextoi(this->m_csValue) && !Main->GetProfileType() == PROFILE_REMOTE)
				{
					csfOutput.WriteString(csIn);
					csfOutput.WriteString(endl);
					continue;
				}
				if (dwThisItem == ahextoi(this->m_csValue))
				{
					// this is the item...we need to handle this one carefully
					// Write the NPC
					Write(csfOutput);
					bNPCWritten = true;
					csfOutput.WriteString("\n");
					// Read the rest of the data up until the next item
					while (bStatus)
					{
						bStatus = csfInput.ReadString(csIn);
						if (!bStatus)
							break;
						if (csIn == "")
							continue;
						if (csIn[0] == '[')
						{
							// We are into the next item now
							break;
						}
					}
				}
				if (dwThisItem > ahextoi(this->m_csValue) && !bNPCWritten)
				{
					// Insert the new one
					this->Write(csfOutput);
					bNPCWritten = true;
				}
			}
		}
		// otherwise, at this point, it's either a comment, or a tag in an item preceding the one we want, or we've already written the new item
		// so write the line to the new file
		if (bStatus && !Main->GetProfileType() == PROFILE_REMOTE)
		{
			csfOutput.WriteString(csIn);
			csfOutput.WriteString(endl);
		}
	}
	csfInput.Close();
	csfOutput.Close();
	if (Main->GetProfileType() == PROFILE_REMOTE)
	{
		CRemoteConnection crRemote;
		if (crRemote.IsValid())
		{
			//crRemote.SendCustomFile("graychar2", csTmpFileName, this);
			//crRemote.GetCustomFile("graychar2");
		}
	}
	else
	{
		CRemoteConsole * pConsole = Main->m_pRConsole;
		bool bDeleteConsole = false;
		if (pConsole == NULL)
		{
			pConsole = new (CRemoteConsole);
			bDeleteConsole = true;
		}
		pConsole->Pause();
		int i = 0;
		while (pConsole->m_bCommandPending)
		{
			i++;
			Sleep(50);
			if (i > 100)
				break;
		}
		ReplaceFiles(csTmpFileName, csFileName);
		pConsole->Resync();
		i = 0;
		while (pConsole->m_bCommandPending)
		{
			i++;
			Sleep(50);
			if (i > 100)
				break;
		}
		if (bDeleteConsole)
			delete pConsole;
	}
#endif
	return true;

}

bool CNPC::WriteBasic(CStdioFile &csfOutput)
{
	if ( !CSObject::Write(csfOutput) )
		return false;
	CString csLine;
	if ( this->m_wID != 0 )
	{
		csLine.Format("ID=%05x\n", this->m_wID);
		csfOutput.WriteString(csLine);
	}
	if ( this->m_csColor != "" )
	{
		csLine.Format("COLOR=%s\n", this->m_csColor);
		csfOutput.WriteString(csLine);
	}
	return true;
}

bool CNPC::Write(CStdioFile & csfOutput)
{
	CString csVal;
	csVal.Format("[%s]\n", this->m_csValue);
	csfOutput.WriteString(csVal);
	bool bTagsHandled[85] = {false};
	// Write this stuff out in a particular order.
	WriteTag("DEFNAME", csfOutput, &bTagsHandled[0], "");	
	WriteTag("NAME", csfOutput, &bTagsHandled[0], "");	
	WriteTag("ANIM", csfOutput, &bTagsHandled[0], "");	
	WriteTag("ICON", csfOutput, &bTagsHandled[0], "");	
	WriteTag("SOUND", csfOutput, &bTagsHandled[0], "");	
	WriteTag("ID", csfOutput, &bTagsHandled[0], "");	
	WriteTag("CAN", csfOutput, &bTagsHandled[0], "");	
	WriteTag("COLOR", csfOutput, &bTagsHandled[0], "");	
	WriteTag("ATT", csfOutput, &bTagsHandled[0], "");	
	WriteTag("ARMOR", csfOutput, &bTagsHandled[0], "");	
	WriteTag("STR", csfOutput, &bTagsHandled[0], "");	
	WriteTag("DEX", csfOutput, &bTagsHandled[0], "");	
	WriteTag("INT", csfOutput, &bTagsHandled[0], "");	
	WriteTag("NPC", csfOutput, &bTagsHandled[0], "");	
	WriteTag("KARMA", csfOutput, &bTagsHandled[0], "");	
	WriteTag("FAME", csfOutput, &bTagsHandled[0], "");	
	WriteTag("SPEECH", csfOutput, &bTagsHandled[0], "");	
	WriteTag("RESOURCES", csfOutput, &bTagsHandled[0], "");	
	WriteTag("FOODTYPE", csfOutput, &bTagsHandled[0], "");	
	WriteTag("DESIRES", csfOutput, &bTagsHandled[0], "");
	WriteTag("NEED", csfOutput, &bTagsHandled[0], "");
	WriteTag("HIREDAYWAGE", csfOutput, &bTagsHandled[0], "");
	WriteTag("TSPEECH", csfOutput, &bTagsHandled[0], "");
	WriteTag("TEVENTS", csfOutput, &bTagsHandled[0], "");
	WriteTag("EVENTS", csfOutput, &bTagsHandled[0], "");
	for (int i = 0; i < SKILL_QTY; i++)
		WriteTag(knownNPCTags[i+13], csfOutput, &bTagsHandled[0], "");
	WriteTag("CATEGORY", csfOutput, &bTagsHandled[0], "");	
	WriteTag("SUBSECTION", csfOutput, &bTagsHandled[0], "");	
	WriteTag("DESCRIPTION", csfOutput, &bTagsHandled[0], "");	
	WriteTag("BUY", csfOutput, &bTagsHandled[0], "");	
	WriteTag("SELL", csfOutput, &bTagsHandled[0], "");	
	WriteTag("ITEM", csfOutput, &bTagsHandled[0], "");
	WriteTag("NOINDEX", csfOutput, &bTagsHandled[0], "");
	csfOutput.WriteString("\n");
	return true;
}

void CNPC::WriteTag(CString csTag, CStdioFile & csfOutput, bool * bTagsHandled, CString csIn)
{
	csTag.MakeUpper();
	CString csNewVal = "";
	CString endl = "\n";
	int iTag = FindTable(csTag, knownNPCTags, 85);
	if (iTag >= 85)
		return;
	if (bTagsHandled[iTag])
		return;
	int iTagIndex = FindTable(csTag, knownNPCTags, 86);
	switch(iTagIndex)
	{
	case 0:
		csNewVal = m_csDefName; break;
	case 1:
		csNewVal = m_csNPCName; break;
	case 2:
		csNewVal = m_csIcon; break;
	case 3:
		csNewVal = m_csAnim; break;
	case 4:
		csNewVal = m_csSound; break;
	case 5:
		csNewVal = m_csCan; break;
	case 6:
		if ((m_irAtt.min != 0) && (m_irAtt.max != 0))
			csNewVal = m_irAtt.WriteRange();
		break;
	case 7:
		if ((m_irDef.min != 0) && (m_irDef.max != 0))
			csNewVal = m_irDef.WriteRange();
		break;
	case 8:
		csNewVal = m_csResources; break;
	case 9:
		csNewVal = m_csFoodType; break;
	case 10:
		if ((m_irStr.min != 0) && (m_irStr.max != 0))
			csNewVal = m_irStr.WriteRange();
		break;
	case 11:
		if ((m_irDex.min != 0) && (m_irDex.max != 0))
			csNewVal = m_irDex.WriteRange();
		break;
	case 12:
		if ((m_irInt.min != 0) && (m_irInt.max != 0))
			csNewVal = m_irInt.WriteRange();
		break;
	case 13: case 14: case 15: case 16: case 17: case 18: case 19: case 20: case 21: case 22:
	case 23: case 24: case 25: case 26: case 27: case 28: case 29: case 30: case 31: case 32:
	case 33: case 34: case 35: case 36: case 37: case 38: case 39: case 40: case 41: case 42:
	case 43: case 44: case 45: case 46: case 47: case 48: case 49: case 50: case 51: case 52:
	case 53: case 54: case 55: case 56: case 57: case 58: case 59: case 60: case 61: case 62:
		if ( (m_irSkill[iTagIndex - 13].min != 0) && (m_irSkill[iTagIndex - 13].max != 0))
		{
			if ( m_irSkill[iTagIndex - 13].min != m_irSkill[iTagIndex - 13].max )
				csNewVal.Format("{%.1f %.1f}", (float) (m_irSkill[iTagIndex - 13].min / 10.0), (float) (m_irSkill[iTagIndex - 13].max / 10.0));
			else
				csNewVal.Format("%.1f", (float) (m_irSkill[iTagIndex - 13].min / 10.0));
		}
		break;
	case 63:
		csNewVal = m_csNPC; break;
	case 64:
		if ((m_irFame.min != 0) && (m_irFame.max != 0))
			csNewVal = m_irFame.WriteRange();
		break;
	case 65:
		if ((m_irKarma.min != 0) && (m_irKarma.max != 0))
			csNewVal = m_irKarma.WriteRange();
		break;
	case 66:
		csNewVal = m_csDesires; break;
	case 67:
		csNewVal = m_csNeed; break;
	case 68:
		if ( m_iDailyWage )
			csNewVal.Format("%ld", m_iDailyWage);
		break;
	case 69:
		{
			CString csLine;
			for ( int i = 0; i < m_csTSpeech.GetSize(); i++ )
			{
				csLine.Format("%s=%s\n", csTag, m_csTSpeech.GetAt(i));
				csfOutput.WriteString(csLine);
			}
		}
		break;
	case 70:
		{
			CString csLine;
			for ( int i = 0; i < m_csTEvents.GetSize(); i++ )
			{
				csLine.Format("%s=%s\n", csTag, m_csTEvents.GetAt(i));
				csfOutput.WriteString(csLine);
			}
		}
		break;
	case 71:
		if ( m_wSpeech )
			csNewVal.Format("%04x", m_wSpeech);
		break;
	case 72:
		csNewVal = m_csEvents; break;
	case 73:
		if ( m_wID )
			csNewVal.Format("%04x", m_wID);
		break;
	case 74:
		csNewVal = m_csColor; break;
	case 75:
		if (m_bHasSpellbook)
		{
			// Write out everything for the spellbook
			CString csExtras;
			for (int i = 0; i <= m_saSpells.GetUpperBound(); i++)
			{
				if (m_saSpells.GetAt(i) != "")
				{
					CString csTemp = csExtras;
					csExtras.Format("%sSPELLS=%s\n", csTemp, m_saSpells.GetAt(i));
				}
			}
			if (csExtras != "")
				csNewVal.Format("ITEMNEWBIE=spell_book\nMORE1=0%08x\nMORE2=0%08x\n%s\n", m_dwSpellsMore1, m_dwSpellsMore2, csExtras);
			else
				csNewVal.Format("ITEMNEWBIE=spell_book\nMORE1=0%08x\nMORE2=0%08x\n", m_dwSpellsMore1, m_dwSpellsMore2);
			csfOutput.WriteString(csNewVal);
			csNewVal = "";
		}
		if (!m_olItems.IsEmpty())
		{
			POSITION pos = m_olItems.GetHeadPosition();
			while (pos != NULL)
			{
				CNPCLootItem * pItem = (CNPCLootItem *) m_olItems.GetNext(pos);
				if (pItem != NULL)
				{
					if (pItem->m_csName != "")
					{
						CString csAmount = "";
						CString csProb = "";
						CString csItem = pItem->m_csName;
						if (pItem->m_irAmount.min != pItem->m_irAmount.max)
							csAmount.Format("%ld-%ld", pItem->m_irAmount.min, pItem->m_irAmount.max);
						else
							csAmount.Format("%ld", pItem->m_irAmount.min);
						if (csAmount == "1")
							csAmount = "";
						if (pItem->m_iProbability != 1)
							csProb.Format("R%ld", pItem->m_iProbability);
						if ( pItem->m_bNewbie )
							csNewVal.Format("ITEMNEWBIE=%s", csItem);
						else
							csNewVal.Format("ITEM=%s", csItem);
						CString csTmp = csNewVal;
						if (csAmount != "")
							csNewVal.Format("%s,%s", csTmp, csAmount);
						csTmp = csNewVal;
						if (csProb != "")
							csNewVal.Format("%s,%s", csTmp, csProb);
						csTmp = csNewVal;
						if (pItem->m_csColor != "")
							csNewVal.Format("%s\nCOLOR=%s\n", csTmp, pItem->m_csColor);
						else
							csNewVal.Format("%s\n", csTmp);
					}
					else
					{
						if ( pItem->m_bNewbie )
							csNewVal.Format("ITEMNEWBIE=%s\n", pItem->m_csWeight);
						else
							csNewVal.Format("ITEM=%s\n", pItem->m_csWeight);
					}
					csfOutput.WriteString(csNewVal);
					csNewVal = "";
				}
			}
		}
		break;
	case 76:
		if (!m_olBuyItems.IsEmpty())
		{
			POSITION pos = m_olBuyItems.GetHeadPosition();
			while (pos != NULL)
			{
				CVendorItem * pItem = (CVendorItem *) m_olBuyItems.GetNext(pos);
				if (pItem != NULL)
				{
					CString csItem = "";
					CString csAmount = "";
					CString csProb = "";
					csItem = pItem->m_csName;
					if (pItem->m_irAmount.min != pItem->m_irAmount.max)
						csAmount.Format("%ld-%ld", pItem->m_irAmount.min, pItem->m_irAmount.max);
					else
						csAmount.Format("%ld", pItem->m_irAmount.min);
					if (csAmount == "1")
						csAmount = "";
					if (pItem->m_iProbability != 1)
						csProb.Format("R%ld", pItem->m_iProbability);
					csNewVal.Format("BUY=%s", csItem);
					CString csTmp = csNewVal;
					if (csAmount != "")
						csNewVal.Format("%s,%s", csTmp, csAmount);
					csTmp = csNewVal;
					if (csProb != "")
						csNewVal.Format("%s,%s", csTmp, csProb);
					csTmp = csNewVal;
					csNewVal.Format("%s\n", csTmp);
					if (csNewVal != "\n")
						csfOutput.WriteString(csNewVal);
					csNewVal = "";
				}
			}
		}
		break;
	case 77:
		if (!m_olSellItems.IsEmpty())
		{
			POSITION pos = m_olSellItems.GetHeadPosition();
			while (pos != NULL)
			{
				CVendorItem * pItem = (CVendorItem *) m_olSellItems.GetNext(pos);
				if (pItem != NULL)
				{
					CString csItem = "";
					CString csAmount = "";
					CString csProb = "";
					csItem = pItem->m_csName;
					if (pItem->m_irAmount.min != pItem->m_irAmount.max)
						csAmount.Format("%ld-%ld", pItem->m_irAmount.min, pItem->m_irAmount.max);
					else
						csAmount.Format("%ld", pItem->m_irAmount.min);
					if (csAmount == "1")
						csAmount = "";
					if (pItem->m_iProbability != 1)
						csProb.Format("R%ld", pItem->m_iProbability);
					csNewVal.Format("SELL=%s", csItem);
					CString csTmp = csNewVal;
					if (csAmount != "")
						csNewVal.Format("%s,%s", csTmp, csAmount);
					csTmp = csNewVal;
					if (csProb != "")
						csNewVal.Format("%s,%s", csTmp, csProb);
					csTmp = csNewVal;
					csNewVal.Format("%s\n", csTmp);
					if (csNewVal != "\n")
						csfOutput.WriteString(csNewVal);
					csNewVal = "";
				}
			}
		}
		break;
	case 78: break;
	case 79:
		csNewVal = m_csCategory; break;
	case 80:
		csNewVal = m_csSubsection; break;
	case 81:
		csNewVal = m_csDescription; break;
	case 82:
		if ( m_bNoIndex )
			csfOutput.WriteString("NOINDEX\n");
		break;
	case 83: break;
	case 84: break;
	case 85: break;
	default:
		csfOutput.WriteString(csIn);
		csfOutput.WriteString(endl);
		break;
	}
	if ( csNewVal != "" )
	{
		CString csLine;
		csLine.Format("%s=%s\n", csTag, csNewVal);
		csfOutput.WriteString(csLine);
	}
	bTagsHandled[iTag] = true;
}
