/*
 $Id: Drewsky.cpp,v 1.20 2002/11/13 17:52:35 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// drewsky.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "area.h"
#include "drewsky.h"
#include "DrewskyPS.h"
#include "AdditionalSettingsDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CDrewskyApp * Main;
/////////////////////////////////////////////////////////////////////////////
// CDrewskyApp

BEGIN_MESSAGE_MAP(CDrewskyApp, CWinApp)
	//{{AFX_MSG_MAP(CDrewskyApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDrewskyApp construction

CDrewskyApp::CDrewskyApp()
{
	Main = this;
	m_bKeepGoing = true;
	m_bNoScripts = false;
	m_pcppAxisLogTab = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CDrewskyApp object

CDrewskyApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CDrewskyApp initialization

BOOL CDrewskyApp::InitInstance()
{
	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	AfxEnableControlContainer();
	AfxInitRichEdit();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	m_pcppAxisLogTab = new CAxisLogDlg;	// Create this object as soon as possible, so we get all the log info.

	m_log.Add("Starting %s, built %s", Main->GetVersionTitle(), Main->GetBuildTimestamp());

	// Check to see if we have an existing registry section for Axis
	HKEY hKey;
	if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\Menasoft\\GM Tools", 0, KEY_READ, &hKey) != ERROR_SUCCESS )
	{
		CAdditionalSettingsDlg dlg;
		if( dlg.DoModal() != IDOK)
			return FALSE;
	}
	RegCloseKey(hKey);
	// Find the path to the UO files
	m_pRConsole = NULL;
	m_log.Add("Looking for client installation");
	csUOPath = GetRegistryString("Client Path", "");
	csUOExe = GetRegistryString("Client EXE", "client.exe");
	if ( csUOPath == "" )
	{
		CString csClient = GetRegistryString("ExePath", "", HKEY_LOCAL_MACHINE, "SOFTWARE\\Origin Worlds Online\\Ultima Online\\1.0");
		csUOPath = csClient.Left(csClient.ReverseFind('\\') + 1);
		csUOExe = csClient.Mid(csClient.ReverseFind('\\') + 1);
	}
	m_log.Add("Client path is %s", csUOPath);

	// Load the saved settings
	m_dwSpawnMessage = 0;
	m_dwStartLauncher = 0;
	m_dwDrawItems = 1;
	m_dwDrawChars = 1;
	m_dw3DRoomView = 1;
	m_dwScaleItems = 1;
	m_dwAlwaysOnTop = 0;
	m_dwShowRadar = 1;
	m_dwUseRemoteScripts = 0;
	m_dwShowSpawnpoints = 0;
	m_dwSysClose = 0;
	m_dwResourcesOnly = 0;
	m_bSaveNormal = false;
	m_bIgnoreWarnings = false;
	m_bAdminMode = false;
	m_csUserID = "";
	m_csPassword = "";
	m_csServerIP = "";
	m_csAxissvrPort = "";
	m_csSpherePort = "";
	m_dwNPCSpawnColor = 0x00FF0000;
	m_dwItemSpawnColor = 0x0000FF00;
	m_dwNoNPCAnimations = 0;
	m_dwRemoteSpawns = 0;
	m_iReceiveTimeout = 60000;
	m_dwDialogColor = GetSysColor(COLOR_3DFACE);
	m_dwTextColor = GetSysColor(COLOR_BTNTEXT);
	m_dwGodClient = 0;
	if (strstr(m_lpCmdLine, "-torgo") != NULL)
	{
		m_log.Add("Starting in Admin mode");
		m_bAdminMode = true;
	}
	else
		m_bAdminMode = false;
	if ( strstr(m_lpCmdLine, "-seer") != NULL)
	{
		m_log.Add("Starting in OSI Seer mode");
		m_bSeerMode = true;
	}
	else
		m_bSeerMode = false;

	m_log.Add("Retrieving registry settings");
	// Check for multiple profiles
	LONG lStatus = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\Menasoft\\GM Tools\\Remote Profiles", 0, KEY_ALL_ACCESS, &hKey);
	int iProfiles = 0;
	while (lStatus == ERROR_SUCCESS)
	{
		char szBuffer[MAX_PATH];
		DWORD dwSize = sizeof(szBuffer);
		lStatus = RegEnumKeyEx(hKey, iProfiles, &szBuffer[0], &dwSize, 0, NULL, NULL, NULL);
		if (lStatus == ERROR_SUCCESS)
			iProfiles++;
	}
	CChooseProfileDlg profileDlg;
	if ( profileDlg.DoModal() != IDOK )
		return FALSE;
	RegCloseKey(hKey);

	m_dwSpawnMessage = GetRegistryDword("SpawnMessage");
	m_dwStartLauncher = GetRegistryDword("StartLauncher");
	m_dwDrawItems = GetRegistryDword("DrawItems");
	m_dw3DRoomView = GetRegistryDword("3DRoomView");
	m_dwScaleItems = GetRegistryDword("ScaleItems");
	m_dwAlwaysOnTop = GetRegistryDword("AlwaysOnTop");
	m_dwShowRadar = GetRegistryDword("ShowRadar");
	m_dwUseRemoteScripts = GetRegistryDword("UseRemoteScripts");
	m_dwShowSpawnpoints = GetRegistryDword("ShowSpawnpoints");
	m_dwSysClose = GetRegistryDword("SysClose");
	m_dwAllowMultiple = GetRegistryDword("AllowMultipleInstances");
	m_dwResourcesOnly = GetRegistryDword("ResourcesOnly");
	m_dwNPCSpawnColor = GetRegistryDword("NPCSpawnColor", 0x00FF0000);
	m_dwItemSpawnColor = GetRegistryDword("ItemSpawnColor", 0x0000FF00);
	m_dwDialogColor = GetRegistryDword("DialogColor", 0x00777777);
	m_dwTextColor = GetRegistryDword("TextColor");
	m_dwNoNPCAnimations = GetRegistryDword("NoNPCAnimations");
	m_dwOldCommands = GetRegistryDword("UseOldCommands");
	m_dwGodClient = GetRegistryDword("GodClient");
	m_dwCommandVersion = GetRegistryDword("CommandVersion");

	
	m_csPosition = GetRegistryString("Position");
	m_csProfile = GetRegistryString("Script Profile");

	if ( GetProfileType() == PROFILE_REMOTE )
	{
		CString csKey;
		m_log.Add("Retrieving remote profile");
		csKey.Format("SOFTWARE\\Menasoft\\GM Tools\\Remote Profiles\\%s", GetProfileString());
		m_csUserID = GetRegistryString("Username", "", HKEY_LOCAL_MACHINE, csKey);
		m_csPassword = GetRegistryString("Password", "", HKEY_LOCAL_MACHINE, csKey);
		m_csServerIP = GetRegistryString("ServerIP", "", HKEY_LOCAL_MACHINE, csKey);
		m_csAxissvrPort = GetRegistryString("HogPort", "4006", HKEY_LOCAL_MACHINE, csKey);
		m_csSpherePort = GetRegistryString("TusPort", "2593", HKEY_LOCAL_MACHINE, csKey);
		m_iReceiveTimeout = (int) GetRegistryDword("ReceiveTimeout", 60000, HKEY_LOCAL_MACHINE, csKey);
		if ( m_iReceiveTimeout < 1000 )
				m_iReceiveTimeout = 1000;

		m_csPassword = Encrypt(m_csPassword);
	}
	if ( GetProfileType() == PROFILE_LOCAL )
	{
		CString csKey;
		m_log.Add("Retrieving local profile");
		csKey.Format("SOFTWARE\\Menasoft\\GM Tools\\Local Profiles\\%s", GetProfileString());
		m_csRootDirectory = GetRegistryString("Root Directory", "", HKEY_LOCAL_MACHINE, csKey);
		m_csServerIP = GetRegistryString("ServerIP", "127.0.0.1", HKEY_LOCAL_MACHINE, csKey);
		m_csSpherePort = GetRegistryString("Port", "2593", HKEY_LOCAL_MACHINE, csKey);
		m_csUserID = GetRegistryString("Username", "", HKEY_LOCAL_MACHINE, csKey);
		m_csPassword = GetRegistryString("Password", "", HKEY_LOCAL_MACHINE, csKey);
		m_csAxissvrPort = GetRegistryString("HogPort", "4006", HKEY_LOCAL_MACHINE, csKey);
		m_dwRemoteSpawns = GetRegistryDword("Read Remote Spawnpoints", 0, HKEY_LOCAL_MACHINE, csKey);
		m_iReceiveTimeout = (int) GetRegistryDword("ReceiveTimeout", 60000, HKEY_LOCAL_MACHINE, csKey);
		if ( m_iReceiveTimeout < 1000 )
				m_iReceiveTimeout = 1000;

		m_csPassword = Encrypt(m_csPassword);
	}

	if (!m_dwAllowMultiple)
	{
		if (FindInstance())
			return FALSE;
	}

	SetDialogColors();

	// Read the hues.
	m_log.Add("Reading hues from hues.mul");
	CFile cfHues;
	CString csHueFile;
	csHueFile.Format("%s\\hues.mul", csUOPath);
	if ( cfHues.Open( csHueFile, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone ) )
	{
		int iBytesRead = 1;
		while ( iBytesRead )
		{
			CHueGroup * pHueGroup = new (CHueGroup);
			iBytesRead = cfHues.Read( pHueGroup, sizeof(CHueGroup) );
			if ( iBytesRead )
			{
				m_aHueGroups.Add( pHueGroup );
			}
			else
				delete pHueGroup;

		}
		cfHues.Close();
	}
	else
		m_log.Add("Unable to open hues file %s\\hues.mul", csUOPath);
	// Now read the hues from the verdata.mul file
	m_log.Add("Reading hues from verdata.mul");
	csHueFile.Format("%s\\verdata.mul", csUOPath);
	if ( cfHues.Open( csHueFile, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone ) )
	{
		DWORD dwCount;
		int iBytesRead = cfHues.Read(&dwCount, sizeof(dwCount));
		DWORD i = 0;
		while ( i++ < dwCount )
		{
			DWORD dwIndexData [5];
			memset(&dwIndexData[0], 0x00, sizeof(dwIndexData));
			cfHues.Read(&dwIndexData[0], sizeof(dwIndexData));
			// Is this hue data?
			if ( dwIndexData[0] == 0x20 )
			{
				// Hues in the verdata file are seriously messed up
				CHueGroup * pHueGroup = (CHueGroup *) m_aHueGroups.GetAt(dwIndexData[1]);
				BYTE bOldData[sizeof(DWORD) + 8 * (sizeof(CHueEntry) + 64)];
				DWORD dwPosition = cfHues.GetPosition();
				cfHues.Seek(dwIndexData[2], CFile::begin);
				cfHues.Read( &bOldData[0], sizeof(bOldData));
				cfHues.Seek(dwPosition, CFile::begin);
				for ( int i = 0; i < 8; i++ )
					memcpy(&(pHueGroup->Hues[i]), &bOldData[sizeof(DWORD) + i * (sizeof(CHueEntry) + 64)], sizeof(CHueEntry));
				memcpy(&pHueGroup->dwHeader, &bOldData[0], sizeof(DWORD));
			}
		}
		cfHues.Close();
	}
	else
		m_log.Add("Unable to open verdata file %s\\verdata.mul", csUOPath);

	CString csTitle;
	csTitle.Format( "%s - Press F1 for help", GetVersionTitle() );
	CDrewskyPS dlg(csTitle);
	dlg.m_psh.dwFlags = PSH_HASHELP | PSH_NOAPPLYNOW | PSH_PROPSHEETPAGE;
	dlg.EnableStackedTabs( false );
	
	m_log.Add("Creating tabs");
	
	m_pcppGeneralTab = new CGeneralTab;
	m_pcppTravelTab = new CTravelTab;
	m_pcppFlagsBrainsTab = new CFlagsBrainsTab;
	m_pcppSpawnTab = new CSpawnTab;
	m_pcppItemGenTab = new CItemGenTab;
	m_pcppItemTweakTab = new CItemTweakTab;
	m_pcppPlayerTweakTab = new CPlayerTweakTab;
	m_pcppLauncherTab = new CLauncherTab;
	m_pcppSettingsTab = new CSettingsTab;
	m_pcppMiscTab = new CMiscTab;
	m_pcppCommandTab = new CCommandTab;
	m_pcppMacroDlg = new CMacroDlg;

	dlg.AddPage(m_pcppGeneralTab);
	dlg.AddPage(m_pcppTravelTab);
	dlg.AddPage(m_pcppFlagsBrainsTab);
	dlg.AddPage(m_pcppSpawnTab);
	dlg.AddPage(m_pcppItemGenTab);
	dlg.AddPage(m_pcppItemTweakTab);
	dlg.AddPage(m_pcppPlayerTweakTab);
	dlg.AddPage(m_pcppLauncherTab);
	dlg.AddPage(m_pcppSettingsTab);
	dlg.AddPage(m_pcppMiscTab);
	dlg.AddPage(m_pcppCommandTab);
	dlg.AddPage(m_pcppAxisLogTab);
	if ( m_bAdminMode )
		dlg.AddPage(m_pcppMacroDlg);

	m_pMainWnd = &dlg;

	if (m_dwStartLauncher)
		dlg.SetActivePage(m_pcppLauncherTab);

	// Read the scripts
	m_pScripts = new CScriptObjects;
	if ( GetProfileType() != PROFILE_NONE )
	{
		m_log.Add("Loading scripts");
		this->m_pScripts->LoadAll();
	}
	m_log.Add("Ready to display main window");
	m_pcppAxisLogTab->SetVisible(true);
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with OK
	}
	else if (nResponse == IDCANCEL)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with Cancel
	}
	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	//return FALSE;
	return TRUE;
}

BOOL CDrewskyApp::PreTranslateMessage(MSG* pMsg) 
{
	if ((pMsg->message == 256) && (pMsg->wParam == 27))
	{
		// this is the escape key
		return (TRUE);
	}
	return CWinApp::PreTranslateMessage(pMsg);
}

bool CDrewskyApp::FindInstance()
{
	// try to determine if there is already an instance of this application running.
	EnumWindows(EnumInstanceProc, 0);
	if (hwndHoGInstance)
	{
		// Found the client window
		CWnd * pWnd = CWnd::FromHandle(hwndHoGInstance);
		pWnd->ShowWindow(SW_SHOWNORMAL);
		pWnd->SetForegroundWindow();
		return true;
	}
	else
		return false;

}

CDrewskyApp::~CDrewskyApp()
{
	if ( m_pScripts )
		delete m_pScripts;
	delete m_pcppGeneralTab;
	delete m_pcppTravelTab;
	delete m_pcppFlagsBrainsTab;
	delete m_pcppSpawnTab;
	delete m_pcppItemGenTab;
	delete m_pcppItemTweakTab;
	delete m_pcppPlayerTweakTab;
	delete m_pcppLauncherTab;
	delete m_pcppSettingsTab;
	delete m_pcppMiscTab;
	delete m_pcppMacroDlg;
	delete m_pcppCommandTab;
	delete m_pcppAxisLogTab;
	m_pcppAxisLogTab = NULL;	// Need to do this so the log doesn't try to send to a bogus window when it closes.

}

void CDrewskyApp::SetDialogColors()
{
	//SetDialogBkColor(m_dwDialogColor, m_dwTextColor);
}

DWORD CDrewskyApp::GetRegistryDword(CString csValue, DWORD dwDefault, HKEY hMainKey, CString csSubKey)
{
	HKEY hKey;
	if ( RegOpenKeyEx(hMainKey, csSubKey, 0, KEY_READ, &hKey) != ERROR_SUCCESS )
		return dwDefault;
	DWORD dwType, dwSize, dwValue;
	dwSize = sizeof(DWORD);
	LONG lStatus = RegQueryValueEx(hKey, LPCTSTR(csValue), 0, &dwType, (BYTE *) &dwValue, &dwSize);
	if ( lStatus != ERROR_SUCCESS )
	{
		if ( lStatus == ERROR_FILE_NOT_FOUND )
		{
			//m_log.Add("Unable to find value %s in the registry.  Using default values", csValue);
		}
		else
			m_log.Add("Error reading value %s from registry.  Last error was %s", csValue, strerror(GetLastError()) );
		RegCloseKey(hKey);
		return dwDefault;
	}
	if ( dwType != REG_DWORD )
	{
		m_log.Add("Registry value of wrong type for %s.  Type was %ld, expected %ld", csValue, dwType, REG_DWORD);
		RegCloseKey(hKey);
		return dwDefault;
	}
	RegCloseKey(hKey);
	return dwValue;
}

CString CDrewskyApp::GetRegistryString(CString csValue, CString csDefault, HKEY hMainKey, CString csSubKey)
{
	HKEY hKey;
	if ( RegOpenKeyEx(hMainKey, csSubKey, 0, KEY_READ, &hKey) != ERROR_SUCCESS )
		return csDefault;
	DWORD dwType, dwSize;
	char szBuffer[MAX_BUFFER];
	memset(&szBuffer[0], 0x00, sizeof(szBuffer));
	dwSize = sizeof(szBuffer);
	LONG lStatus = RegQueryValueEx(hKey, LPCTSTR(csValue), 0, &dwType, (BYTE *) &szBuffer[0], &dwSize);
	if ( lStatus != ERROR_SUCCESS )
	{
		if ( lStatus == ERROR_FILE_NOT_FOUND )
		{
			// m_log.Add("Unable to find value %s in the registry.  Using default values", csValue);
		}
		else
			m_log.Add("Error reading value %s from registry.  Last error was %s", csValue, strerror(GetLastError()) );
		RegCloseKey(hKey);
		return csDefault;
	}
	if ( dwType != REG_SZ )
	{
		m_log.Add("Registry value of wrong type for %s.  Type was %ld, expected %ld", csValue, dwType, REG_SZ);
		RegCloseKey(hKey);
		return csDefault;
	}
	RegCloseKey(hKey);
	return _T(szBuffer);
}

void CDrewskyApp::PutRegistryDword(CString csValue, DWORD dwValue, HKEY hMainKey, CString csSubKey)
{
	HKEY hKey;
	DWORD dwDisp;
	LONG lStatus = RegCreateKeyEx(hMainKey, csSubKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, &dwDisp);

	if ( lStatus != ERROR_SUCCESS )
	{
		m_log.Add("Unable to open registry key %s for writing.  Last error was %s", csSubKey, strerror(lStatus) );
		return;
	}
	lStatus = RegSetValueEx(hKey, csValue, 0, REG_DWORD, ((BYTE *) &dwValue), sizeof(DWORD) );
	if ( lStatus != ERROR_SUCCESS )
		m_log.Add("Error writing value %s to the registry.  Last error was %s", csValue, strerror(lStatus) );
	RegCloseKey(hKey);
}

void CDrewskyApp::PutRegistryString(CString csValue, CString csString, HKEY hMainKey, CString csSubKey)
{
	HKEY hKey;
	DWORD dwDisp;

	LONG lStatus = RegCreateKeyEx(hMainKey, csSubKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, &dwDisp);

	if ( lStatus != ERROR_SUCCESS )
	{
		m_log.Add("Unable to open registry key %s for writing.  Last error was %s", csSubKey, strerror(lStatus) );
		return;
	}
	lStatus = RegSetValueEx(hKey, csValue, 0, REG_SZ, ((BYTE *) csString.GetBuffer(csString.GetLength()) ), csString.GetLength() );
	if ( lStatus != ERROR_SUCCESS )
		m_log.Add("Error writing value %s to the registry.  Last error was %s", csValue, strerror(lStatus) );
	RegCloseKey(hKey);
}


void CDrewskyApp::DeleteRegistryKey(CString csKeyName, HKEY hRootKey)
{
	LONG lStatus = RegDeleteKey(hRootKey, csKeyName);
	if ( lStatus != ERROR_SUCCESS )
		m_log.Add("Error deleting registry key %s.  Last error was %s", csKeyName, strerror(lStatus));
	return;
}

void CDrewskyApp::DeleteRegistryValue(CString csValueName, CString csSubKey, HKEY hRootKey)
{
	HKEY hKey;
	LONG lStatus = RegOpenKey(hRootKey, csSubKey, &hKey);
	if ( lStatus == ERROR_SUCCESS )
	{
		lStatus = RegDeleteValue(hKey, csValueName);
		if ( lStatus != ERROR_SUCCESS )
			m_log.Add("Error deleting registry value %s.  Last error was %s", csValueName, strerror(lStatus));
		RegCloseKey(hKey);
	}
	else
		m_log.Add("Couldn't open registry key %s.  Last error was %s", csSubKey, strerror(lStatus));
	return;
}

CString CDrewskyApp::GetVersionTitle()
{
	// Get the version info from the exe itself
	CString csExe = AfxGetAppName();
	csExe += ".exe";
	char * pszExe = new char [csExe.GetLength() + 1];
	sprintf(pszExe, csExe);
	DWORD dwHandle = 0;
	DWORD dwSize = GetFileVersionInfoSize(pszExe, &dwHandle);
	char * pBuffer = new char [dwSize];
	GetFileVersionInfo(pszExe, dwHandle, dwSize, (LPVOID) pBuffer);
	CString csVersionInfo;
	unsigned int iDataSize = 80;
	LPVOID pData;
	VerQueryValue(pBuffer, _T("\\StringFileInfo\\040904B0\\ProductVersion"), &pData, &iDataSize);
	csVersionInfo.Format("%s", pData);

	delete [] pszExe;
	delete [] pBuffer;
	
	csVersionInfo.Replace(",", ".");
	csVersionInfo.Replace(" ", "");

	CString csVersionTitle;
	csVersionTitle.Format("Axis version %s", csVersionInfo);

	return csVersionTitle;
}

CString CDrewskyApp::GetBuildTimestamp()
{
	CString csTimestamp;
	csTimestamp.Format("%s %s", __DATE__, __TIME__);
	return csTimestamp;
}

CString CDrewskyApp::GetProfileString()
{
	if ( m_csProfile.Find(": ", 0) != -1 )
		return m_csProfile.Mid(m_csProfile.Find(": ") + 2);
	else
		return "";
}

int CDrewskyApp::GetProfileType()
{
	if ( m_csProfile.Find(": ", 0) != -1 )
	{
		CString csType;
		csType = m_csProfile.SpanExcluding(": ");
		if ( csType.CompareNoCase("Local") == 0 )
			return PROFILE_LOCAL;
		if ( csType.CompareNoCase("Remote") == 0 )
			return PROFILE_REMOTE;
	}
	else
	{
		if ( m_csProfile.Find("NONE") != -1 )
			return PROFILE_NONE;
		if ( m_csProfile.Find("DEFAULT") != -1 )
			return PROFILE_DEFAULT;
	}
	return PROFILE_UNDEFINED;
}

void CDrewskyApp::PutRegistryMultiSz(CString csValue, CStringArray * pStrings, HKEY hMainKey, CString csSubKey)
{
	HKEY hKey;
	DWORD dwDisp;
	LONG lStatus = RegCreateKeyEx(hMainKey, csSubKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, &dwDisp);

	// Find out how much space we need to allocate for the string buffer
	int i = 0;
	int iLength = 0;
	for ( i = 0; i <= pStrings->GetUpperBound(); i++ )
		iLength += ( pStrings->GetAt(i).GetLength() + 1);

	char * pszStrings = new char [iLength + 1];
	
	memset(pszStrings, 0x00, iLength + 1);
	char * ptr = pszStrings;
	for ( i = 0; i <= pStrings->GetUpperBound(); i++ )
	{
		memcpy(ptr, LPCTSTR(pStrings->GetAt(i)), pStrings->GetAt(i).GetLength());
		ptr += pStrings->GetAt(i).GetLength() + 1;
	}

	if ( lStatus != ERROR_SUCCESS )
	{
		m_log.Add("Unable to open registry key %s for writing.  Last error was %s", csSubKey, strerror(lStatus) );
		return;
	}
	lStatus = RegSetValueEx(hKey, csValue, 0, REG_MULTI_SZ, ((BYTE *) pszStrings), iLength );
	if ( lStatus != ERROR_SUCCESS )
		m_log.Add("Error writing value %s to the registry.  Last error was %s", csValue, strerror(lStatus) );
	RegCloseKey(hKey);

	delete [] pszStrings;
}

void CDrewskyApp::GetRegistryMultiSz(CString csValue, CStringArray * pStrings, HKEY hMainKey, CString csSubKey)
{
	// Make sure the input array is empty
	pStrings->RemoveAll();

	HKEY hKey;
	if ( RegOpenKeyEx(hMainKey, csSubKey, 0, KEY_READ, &hKey) != ERROR_SUCCESS )
		return;
	DWORD dwType, dwSize;
	// Need to find out how big the buffer is
	dwSize = 0;
	LONG lStatus = RegQueryValueEx(hKey, LPCTSTR(csValue), 0, &dwType, NULL, &dwSize);

	if ( dwSize == 0 )
	{
		RegCloseKey(hKey);
		return;
	}

	char * pszBuffer = new char [dwSize];
	memset(pszBuffer, 0x00, dwSize);
	lStatus = RegQueryValueEx(hKey, LPCTSTR(csValue), 0, &dwType, (BYTE*) pszBuffer, &dwSize);
	char * pDataPtr = pszBuffer;

	if ( lStatus != ERROR_SUCCESS )
	{
		if ( lStatus == ERROR_FILE_NOT_FOUND )
		{
			// m_log.Add("Unable to find value %s in the registry.", csValue);
		}
		else
			m_log.Add("Error reading value %s from registry.  Last error was %s", csValue, strerror(GetLastError()) );
	}
	else
	{
		// parse the registry value
		DWORD dwOffset = 0;
		while ( dwOffset < dwSize )
		{
			CString csValue = _T(pDataPtr);
			pStrings->Add(csValue);
			pDataPtr += csValue.GetLength() + 1;
			dwOffset += csValue.GetLength() + 1;
		}
		
	}
	if ( dwType != REG_MULTI_SZ )
		m_log.Add("Registry value of wrong type for %s.  Type was %ld, expected %ld", csValue, dwType, REG_MULTI_SZ);
	RegCloseKey(hKey);
	delete [] pszBuffer;
}