/*
 $Id: SpawnGroupEditDlg.cpp,v 1.3 2002/05/01 03:33:10 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// SpawnGroupEditDlg.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "SpawnGroupEditDlg.h"
#include "CNPC.h"
#include "AddValueDialog.h"
#include "RemoteConnection.h"
#include "FileReadProgress.h"
#include "RemoteConsole.h"
#include "EditValue.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSpawnGroupEditDlg dialog

int CSpawnGroupEditDlg::iIDSort = 1;
int CSpawnGroupEditDlg::iNameSort = 1;

CSpawnGroupEditDlg::CSpawnGroupEditDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSpawnGroupEditDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSpawnGroupEditDlg)
	//}}AFX_DATA_INIT
	m_pSpawn = NULL;
}


void CSpawnGroupEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSpawnGroupEditDlg)
	DDX_Control(pDX, IDC_SGNAME, m_ceName);
	DDX_Control(pDX, IDC_SGENTRY, m_ccbSpawnGroupEntry);
	DDX_Control(pDX, IDC_SGDESCRIPTION, m_ceDescription);
	DDX_Control(pDX, IDC_SGSUBSECTION, m_ccbSubsection);
	DDX_Control(pDX, IDC_SGCATEGORY, m_ccbCategory);
	DDX_Control(pDX, IDC_SGWEIGHT, m_ceWeight);
	DDX_Control(pDX, IDC_SPAWNGROUPLIST, m_clcSpawnGroupList);
	DDX_Control(pDX, IDC_SGNPCLIST, m_clcNPCList);
	DDX_Control(pDX, IDC_SGCATEGORYTREE, m_ctcCategoryTree);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSpawnGroupEditDlg, CDialog)
	//{{AFX_MSG_MAP(CSpawnGroupEditDlg)
	ON_CBN_SELCHANGE(IDC_SGCATEGORY, OnSelchangeSgcategory)
	ON_NOTIFY(TVN_SELCHANGED, IDC_SGCATEGORYTREE, OnSelchangedSgcategorytree)
	ON_BN_CLICKED(IDC_SGADDCATEGORY, OnSgaddcategory)
	ON_BN_CLICKED(IDC_SGADDSUBSECTION, OnSgaddsubsection)
	ON_BN_CLICKED(IDC_SGADD, OnSgadd)
	ON_BN_CLICKED(IDC_SGREMOVE, OnSgremove)
	ON_NOTIFY(NM_CLICK, IDC_SGNPCLIST, OnClickSgnpclist)
	ON_NOTIFY(NM_DBLCLK, IDC_SGNPCLIST, OnDblclkSgnpclist)
	ON_NOTIFY(LVN_KEYDOWN, IDC_SPAWNGROUPLIST, OnKeydownSpawngrouplist)
	ON_CBN_SELCHANGE(IDC_SGENTRY, OnSelchangeSgname)
	ON_BN_CLICKED(IDC_NEWSPAWNGROUP, OnNewspawngroup)
	ON_NOTIFY(NM_DBLCLK, IDC_SPAWNGROUPLIST, OnDblclkSpawngrouplist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSpawnGroupEditDlg message handlers

BOOL CSpawnGroupEditDlg::OnInitDialog() 
{
	CWaitCursor hourglass;
	CDialog::OnInitDialog();

	m_pApp = (CDrewskyApp *) AfxGetApp();
	m_pApp->m_pScripts->LoadDefs();
	m_pApp->m_pScripts->LoadNPCs();
	m_pApp->m_pScripts->LoadTables();
	CFileReadProgress dlg;
	dlg.Create(IDD_FILEREAD_PROGRESS);
	m_pApp->m_pScripts->CategorizeNPCs();

	CString csTitle;
	this->SetWindowText(csTitle);
	this->m_clcSpawnGroupList.InsertColumn(0, "Creature", LVCFMT_LEFT, 175);
	this->m_clcSpawnGroupList.InsertColumn(1, "Weight", LVCFMT_LEFT, 75);
	this->m_clcNPCList.InsertColumn(0, "Creature", LVCFMT_LEFT, 125);
	this->m_clcNPCList.InsertColumn(1, "ID", LVCFMT_LEFT, 50);
	FillCategoryList();
	FillCategoryTree();
	int iSel = -1;
	for ( int i = 0; i < m_pApp->m_pScripts->m_tables.m_spawns.GetSize(); i++ )
	{
		CSObject * pBase = (CSObject *) m_pApp->m_pScripts->m_tables.m_spawns.GetAt(i);
		if ( !pBase )
			continue;
		CString csId;
		csId.Format("%s [%s]", pBase->m_csDescription, pBase->m_csValue);
		int index = this->m_ccbSpawnGroupEntry.AddString(csId);
		this->m_ccbSpawnGroupEntry.SetItemDataPtr(index, (void *) pBase);
		if ( m_pSpawn && m_pSpawn->m_csValue == pBase->m_csValue )
			iSel = index;
	}
	if ( iSel != -1 )
	{
		this->m_ccbSpawnGroupEntry.SetCurSel(iSel);
		PopulateFields();
	}
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSpawnGroupEditDlg::FillCategoryTree()
{
	HTREEITEM CategoryParent;
	HTREEITEM SubsectionParent;
	TV_INSERTSTRUCT InsertItem;

	this->m_ctcCategoryTree.DeleteAllItems();
	if (!m_pApp->m_pScripts->m_olNPCs.IsEmpty())
	{
		POSITION pos = m_pApp->m_pScripts->m_olNPCs.GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pCategory = (CCategory *) m_pApp->m_pScripts->m_olNPCs.GetNext(pos);
			InsertItem.item.mask = TVIF_TEXT;
			InsertItem.item.pszText = (char *)LPCTSTR(pCategory->m_csName);
			InsertItem.item.cchTextMax = pCategory->m_csName.GetLength();
			InsertItem.hParent = NULL;
			InsertItem.hInsertAfter = TVI_SORT;
			CategoryParent = this->m_ctcCategoryTree.InsertItem(&InsertItem);
			if (!pCategory->m_SubsectionList.IsEmpty())
			{
				POSITION sPos = pCategory->m_SubsectionList.GetHeadPosition();
				while (sPos != NULL)
				{
					CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(sPos);
					InsertItem.item.mask = TVIF_TEXT;
					InsertItem.item.pszText = (char *)LPCTSTR(pSubsection->m_csName);
					InsertItem.item.cchTextMax = pSubsection->m_csName.GetLength();
					InsertItem.hParent = CategoryParent;
					InsertItem.hInsertAfter = TVI_SORT;
					SubsectionParent = this->m_ctcCategoryTree.InsertItem(&InsertItem);
				}
			}
		}
	}
}

void CSpawnGroupEditDlg::FillCategoryList()
{
	// Fill in the category box
	if (!m_pApp->m_pScripts->m_olNPCs.IsEmpty())
	{
		POSITION pos = m_pApp->m_pScripts->m_olNPCs.GetHeadPosition();
		while (pos != NULL)
		{
			CCategory * pCategory = (CCategory *) m_pApp->m_pScripts->m_olNPCs.GetNext(pos);
			int iIndex = m_ccbCategory.AddString(pCategory->m_csName);
			m_ccbCategory.SetItemData(iIndex, (DWORD) pCategory);
		}
	}	
}

void CSpawnGroupEditDlg::OnSelchangeSgcategory() 
{
	CString csCategory;
	m_ccbCategory.GetLBText(m_ccbCategory.GetCurSel(), csCategory);
	m_ccbSubsection.ResetContent();
	if ( (csCategory != ""))
	{
		int iSel = m_ccbCategory.GetCurSel();
		if (iSel == -1)
			return;
		CCategory * pCategory = (CCategory *) m_ccbCategory.GetItemData(iSel);
		if (pCategory == NULL)
			return;
		if (!pCategory->m_SubsectionList.IsEmpty())
		{
			POSITION pos = pCategory->m_SubsectionList.GetHeadPosition();
			while (pos != NULL)
			{
				CSubsection * pSubsection = (CSubsection *) pCategory->m_SubsectionList.GetNext(pos);
				int iIndex = m_ccbSubsection.AddString(pSubsection->m_csName);
				m_ccbSubsection.SetItemData(iIndex, DWORD (pSubsection));
			}
		}
		m_ccbSubsection.SetCurSel(0);
	}
	else
		m_ccbSubsection.SetCurSel(-1);
}

void CSpawnGroupEditDlg::OnSelchangedSgcategorytree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	HTREEITEM hSelectedItem  = NULL;
	HTREEITEM hParentItem = NULL;
	this->m_clcNPCList.DeleteAllItems();
	this->m_clcNPCList.SetHotItem(-1);

	HTREEITEM hItem = pNMTreeView->itemOld.hItem;
	if (hItem != NULL)
		this->m_ctcCategoryTree.SetItemState(hItem, 0, TVIS_BOLD);

	hSelectedItem = this->m_ctcCategoryTree.GetSelectedItem();
	this->m_ctcCategoryTree.SetItemState(hSelectedItem, TVIS_BOLD, TVIS_BOLD);
	hParentItem = this->m_ctcCategoryTree.GetParentItem(hSelectedItem);

	if (hParentItem != NULL)
	{
		CString csParent = this->m_ctcCategoryTree.GetItemText(hParentItem);
		CString csSelected = this->m_ctcCategoryTree.GetItemText(hSelectedItem);
		CCategory * pCategory = FindCategory(&m_pApp->m_pScripts->m_olNPCs, csParent);
		if (pCategory == NULL)
			return;
		CSubsection * pSubsection = FindSubsection(pCategory, csSelected);
		if (pSubsection == NULL)
			return;
		if (!pSubsection->m_ItemList.IsEmpty())
		{
			POSITION pos = pSubsection->m_ItemList.GetHeadPosition();
			int iCount = 0;
			while (pos != NULL)
			{
				CSObject * pObject = (CSObject *) pSubsection->m_ItemList.GetNext(pos);
				if ( pObject )
				{
					this->m_clcNPCList.InsertItem(iCount, pObject->m_csDescription, 0);
					this->m_clcNPCList.SetItemText(iCount, 1,pObject->m_csValue);
					this->m_clcNPCList.SetItemData(iCount, (DWORD)pObject);
					iCount++;
				}
			}
		}
	}

	iNameSort = 1;
	iIDSort = 1;
	this->m_clcNPCList.SortItems(CompareFunc, 0);
	iNameSort = -1;
	
	*pResult = 0;
}

int CALLBACK CSpawnGroupEditDlg::CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CSObject * pItem1 = (CSObject *)lParam1;
	if (pItem1 == NULL)
		return 0;
	CSObject * pItem2 = (CSObject *)lParam2;
	if (pItem2 == NULL)
		return 0;
	if (lParamSort == 0)
	{
		if (iNameSort == 0)
			return (lParam1 < lParam2 ? 1 : -1);
		if (iNameSort == 1)
			return (pItem1->m_csDescription < pItem2->m_csDescription ? -1 : 1);
		if (iNameSort == -1)
			return (pItem1->m_csDescription < pItem2->m_csDescription ? 1 : -1);
	}
	else
	{
		if (iIDSort == 0)
			return (lParam1 < lParam2 ? 1 : -1);
		if (iIDSort == 1)
			return (pItem1->m_csDescription < pItem2->m_csDescription ? -1 : 1);
		if (iIDSort == -1)
			return (pItem1->m_csDescription < pItem2->m_csDescription ? 1 : -1);
	}
	return 0;
}

void CSpawnGroupEditDlg::OnSgaddcategory() 
{
	CAddValueDialog dlg;
	CCategory * pCategory;
	if (dlg.DoModal() == IDOK)
	{
		if (m_ccbCategory.FindStringExact(0, dlg.m_csValue) != CB_ERR)
		{
			AfxMessageBox("Category already exists", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		else
			pCategory = FindCategory(&m_pApp->m_pScripts->m_olNPCs, dlg.m_csValue);
		int iIndex = m_ccbCategory.AddString(dlg.m_csValue);
		m_ccbCategory.SetItemData(iIndex, (DWORD) pCategory);
		m_ccbCategory.SetCurSel(iIndex);
		this->OnSelchangeSgcategory();
	}
	return;	
}

void CSpawnGroupEditDlg::OnSgaddsubsection() 
{
	CAddValueDialog dlg;
	CSubsection * pSubsection;
	if (dlg.DoModal() == IDOK)
	{
		if (m_ccbSubsection.FindStringExact(0, dlg.m_csValue) != CB_ERR)
		{
			AfxMessageBox("Subsection already exists", MB_OK | MB_ICONEXCLAMATION);
			return;
		}
		else
		{
			CCategory * pCategory;
			int iSel = m_ccbCategory.GetCurSel();
			if (iSel == -1)
				return;
			pCategory = (CCategory *) m_ccbCategory.GetItemData(iSel);
			pSubsection = FindSubsection(pCategory, dlg.m_csValue);
		}
		int iIndex = m_ccbSubsection.AddString(dlg.m_csValue);
		m_ccbSubsection.SetItemData(iIndex, (DWORD) pSubsection);
		m_ccbSubsection.SetCurSel(iIndex);
	}
	return;
}

void CSpawnGroupEditDlg::OnSgadd() 
{
	int iSel = this->m_clcNPCList.GetNextItem(-1, LVNI_SELECTED);
	if (iSel != -1)
	{
		CSObject * pObject = (CSObject *) this->m_clcNPCList.GetItemData(iSel);
		if ( pObject )
		{
			CString csCreature = this->m_clcNPCList.GetItemText(iSel, 0);
			CString csWeight;
			this->m_ceWeight.GetWindowText(csWeight);
			if ((csWeight == "") || (csWeight == "0"))
				csWeight = "1";
			int iCount = this->m_clcSpawnGroupList.GetItemCount();
			this->m_clcSpawnGroupList.InsertItem(iCount, csCreature);
			this->m_clcSpawnGroupList.SetItemText(iCount, 1, csWeight);
			this->m_clcSpawnGroupList.SetItemData(iCount, ahextoi(pObject->m_csValue));
		}
	}
}

void CSpawnGroupEditDlg::OnSgremove() 
{
	int iSel = this->m_clcSpawnGroupList.GetNextItem(-1, LVNI_SELECTED);
	if (iSel != -1)
		this->m_clcSpawnGroupList.DeleteItem(iSel);
}

void CSpawnGroupEditDlg::OnOK() 
{
	CWaitCursor hourglass;
	RetrieveFields();
	if ( m_pApp->m_pScripts->SaveTables() )
	{
		m_pApp->m_pScripts->RemoveNPCCategories();
		CFileReadProgress dlg;
		dlg.Create(IDD_FILEREAD_PROGRESS);
		m_pApp->m_pScripts->CategorizeNPCs();
		CDialog::OnOK();
	}
}

void CSpawnGroupEditDlg::OnClickSgnpclist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int iSelIndex = this->m_clcNPCList.GetNextItem(-1, LVNI_SELECTED);
	this->m_clcNPCList.SetHotItem(iSelIndex);
	*pResult = 0;
}

void CSpawnGroupEditDlg::OnDblclkSgnpclist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	this->OnSgadd();		
	*pResult = 0;
}

void CSpawnGroupEditDlg::OnKeydownSpawngrouplist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_KEYDOWN* pLVKeyDow = (LV_KEYDOWN*)pNMHDR;
	// Looking for the delete key
	if (pLVKeyDow->wVKey == VK_DELETE)
		this->OnSgremove();
	*pResult = 0;
}

void CSpawnGroupEditDlg::OnCancel() 
{
	m_pApp->m_pScripts->ReloadTables();
	m_pApp->m_pScripts->RemoveNPCCategories();
	CFileReadProgress dlg;
	dlg.Create(IDD_FILEREAD_PROGRESS);
	m_pApp->m_pScripts->CategorizeNPCs();
	CDialog::OnCancel();
}

void CSpawnGroupEditDlg::OnSelchangeSgname() 
{
	RetrieveFields();
	int iSel = this->m_ccbSpawnGroupEntry.GetCurSel();
	if ( iSel == -1 )
		return;
	if ( m_pSpawn )
		delete m_pSpawn;
	CTObject * pBase = (CTObject *) m_ccbSpawnGroupEntry.GetItemDataPtr(iSel);
	m_pSpawn = new CTSpawnDef;
	m_pSpawn->m_csFilename = pBase->m_csFilename;
	m_pSpawn->m_csValue = pBase->m_csValue;
	m_pSpawn->Load();
	PopulateFields();
}

void CSpawnGroupEditDlg::OnNewspawngroup() 
{
	CAddValueDialog dlg;
	dlg.m_csTitle = "New Spawn Group";
	if ( dlg.DoModal() == IDOK )
	{
		CTSpawnDef * pSpawn = new (CTSpawnDef);
		if ( !pSpawn )
			return;
		pSpawn->m_csName = dlg.m_csValue;
		pSpawn->m_iIndex = m_pApp->m_pScripts->m_tables.m_spawns.Add(pSpawn) + 0x8000;
		CString csID;
		csID.Format("%04x", pSpawn->m_iIndex);
		pSpawn->m_csDescription = dlg.m_csValue;
		pSpawn->m_csCategory = "Spawn Groups";
		int index = this->m_ccbSpawnGroupEntry.AddString(csID);
		this->m_ccbSpawnGroupEntry.SetItemDataPtr(index, (void *) pSpawn);
		this->m_ccbSpawnGroupEntry.SetCurSel(index);
		OnSelchangeSgname();
	}
}

void CSpawnGroupEditDlg::PopulateFields()
{
	if ( m_pSpawn )
	{
		// Populate the dialog with the current parameters
		this->m_clcSpawnGroupList.DeleteAllItems();
		this->m_ceName.SetWindowText(m_pSpawn->m_csName);
		this->m_ccbCategory.SelectString(-1, m_pSpawn->m_csCategory);
		this->OnSelchangeSgcategory();
		this->m_ccbSubsection.SelectString(-1, m_pSpawn->m_csSubsection);
		this->m_ceDescription.SetWindowText(m_pSpawn->m_csDescription);
		this->m_ceWeight.SetWindowText("1");
		CString csTitle;
		csTitle.Format("Spawn Group Editor [%04x]", m_pSpawn->m_iIndex);
		this->SetWindowText(csTitle);
		int index = 0;
		for ( int i = 0; i < m_pSpawn->m_Members.GetSize(); i++ )
		{
			CSpawnRec * pRec = (CSpawnRec *) m_pSpawn->m_Members.GetAt(i);
			if ( !pRec )
				continue;
			int iDef = m_pApp->m_pScripts->m_aDefs.Find(pRec->m_csID);
			if ( iDef != -1)
			{
				this->m_clcSpawnGroupList.InsertItem(index, pRec->m_csID);
				this->m_clcSpawnGroupList.SetItemData(index, 0);
			}
			else
			{
				DWORD dwID = ahextoi(pRec->m_csID);
				CString csID;
				csID.Format("%04x", dwID);
				int iIndex = m_pApp->m_pScripts->m_aNPCs.Find(csID);
				if ( iIndex != -1 )
				{
					CNPC * pNPC = (CNPC *) m_pApp->m_pScripts->m_aNPCs.GetAt(iIndex);
					if ( pNPC )
						this->m_clcSpawnGroupList.InsertItem(index, pNPC->m_csDescription);
					else
						this->m_clcSpawnGroupList.InsertItem(index, pRec->m_csID);
					this->m_clcSpawnGroupList.SetItemData(index, dwID);
				}
			}
			CString csWeight;
			csWeight.Format("%ld", pRec->m_iWeight);
			this->m_clcSpawnGroupList.SetItemText(index, 1, csWeight);
			index++;
		}
	}
	else
	{
		// Clear them all out.
		this->m_ccbCategory.SetWindowText("");
		this->m_ccbSubsection.SetWindowText("");
		this->m_ceDescription.SetWindowText("");
		this->m_ceWeight.SetWindowText("1");
		this->m_clcSpawnGroupList.DeleteAllItems();
		this->SetWindowText("Spawn Group Editor");
	}	
}

void CSpawnGroupEditDlg::RetrieveFields()
{
	if ( m_pSpawn )
	{
		// Save the current data
		this->m_ccbCategory.GetWindowText(m_pSpawn->m_csCategory);
		this->m_ccbCategory.GetWindowText(m_csCategory);
		this->m_ceName.GetWindowText(m_pSpawn->m_csName);
		this->m_ccbSubsection.GetWindowText(m_pSpawn->m_csSubsection);
		this->m_ccbSubsection.GetWindowText(m_csSubsection);
		this->m_ceDescription.GetWindowText(m_pSpawn->m_csDescription);
		for (int i = 0; i < m_pSpawn->m_Members.GetSize(); i++)
		{
			CSpawnRec * pRec = (CSpawnRec *) m_pSpawn->m_Members.GetAt(i);
			if ( pRec )
				delete pRec;
		}
		m_pSpawn->m_Members.RemoveAll();
		m_pSpawn->m_iTotalWeight = 0;
		for (i = 0; i < m_clcSpawnGroupList.GetItemCount(); i++)
		{
			CSpawnRec * pRec = new (CSpawnRec);
			int iData = m_clcSpawnGroupList.GetItemData(i);
			if ( !iData )
				pRec->m_csID = m_clcSpawnGroupList.GetItemText(i, 0);
			else
				pRec->m_csID.Format("%05x", iData);
			pRec->m_iWeight = atoi(m_clcSpawnGroupList.GetItemText(i, 1));
			if ( !pRec->m_iWeight )
			{
				delete pRec;
				continue;
			}
			m_pSpawn->m_Members.Add(pRec);
			m_pSpawn->m_iTotalWeight += pRec->m_iWeight;
		}
	}
}

void CSpawnGroupEditDlg::OnDblclkSpawngrouplist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int iSel = this->m_clcSpawnGroupList.GetNextItem(-1, LVNI_SELECTED);
	if ( iSel == -1 )
		return;
	
	CEditValueDlg dlg;
	dlg.m_csTextValue = this->m_clcSpawnGroupList.GetItemText(iSel, 1);
	dlg.m_csCaption.Format("Spawn Weight");
	if ( dlg.DoModal() != IDOK )
		return;
	this->m_clcSpawnGroupList.SetItemText(iSel, 1, dlg.m_csTextValue);

	*pResult = 0;
}

CSpawnGroupEditDlg::~CSpawnGroupEditDlg()
{
	if ( m_pSpawn )
		delete m_pSpawn;
}