/*
 $Id: StaticToolDlg.cpp,v 1.34 2002/12/04 22:24:36 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// StaticToolDlg.cpp : implementation file
//

#include "stdafx.h"
#include "drewsky.h"
#include "StaticToolDlg.h"
#include "FileReadProgress.h"
#include "SettingsTab.h"
#include "TypeSelectionDlg.h"
#include "AttrSelectionDlg.h"
#include "StaticRegionColorDlg.h"
#include "StaticRegionEditDlg.h"
#include "UOMul.h"
#include "StaticArtCriteriaDlg.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CStaticToolDlg dialog


CStaticToolDlg::CStaticToolDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CStaticToolDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CStaticToolDlg)
	//}}AFX_DATA_INIT
	
	m_dwAttrIncludeMask = Main->GetRegistryDword("StaticAttrInclude", ATTR_STATIC);
	m_dwAttrExcludeMask = Main->GetRegistryDword("StaticAttrExclude", ATTR_MOVE_ALWAYS | ATTR_NEWBIE | ATTR_MAGIC | ATTR_DECAY | ATTR_CAN_DECAY);
	CString csTypeInclude = Main->GetRegistryString("StaticTypeInclude");
	CString csTypeExclude = Main->GetRegistryString("StaticTypeExclude");
	CString csArtInclude = Main->GetRegistryString("StaticArtInclude");
	CString csArtExclude = Main->GetRegistryString("StaticArtExclude");
	CString csTemp = csTypeInclude;
	while ( csTemp != "" )
	{
		CString csType = csTemp.SpanExcluding(" ");
		this->m_dwTypeInclude.Add(atoi(csType));
		csTemp = csTemp.Mid(csType.GetLength());
		csTemp.TrimLeft();
	}
	csTemp = csTypeExclude;
	while ( csTemp != "" )
	{
		CString csType = csTemp.SpanExcluding(" ");
		this->m_dwTypeExclude.Add(atoi(csType));
		csTemp = csTemp.Mid(csType.GetLength());
		csTemp.TrimLeft();
	}
	csTemp = csArtInclude;
	while ( csTemp != "" )
	{
		CString csType = csTemp.SpanExcluding(" ");
		this->m_wArtInclude.Add(atoi(csType));
		csTemp = csTemp.Mid(csType.GetLength());
		csTemp.TrimLeft();
	}
	csTemp = csArtExclude;
	while ( csTemp != "" )
	{
		CString csType = csTemp.SpanExcluding(" ");
		this->m_wArtExclude.Add(atoi(csType));
		csTemp = csTemp.Mid(csType.GetLength());
		csTemp.TrimLeft();
	}
	m_pParent = NULL;
	m_iDrawRectIndex = -1;
	m_bExcludeRegions = false;
	m_bIncludeRegions = false;
	m_bDisplayExcludeRegions = Main->GetRegistryDword("StaticDisplayExcludeRegions", 1) ? true : false;
	m_bDisplayIncludeRegions = Main->GetRegistryDword("StaticDisplayIncludeRegions", 1) ? true : false;
	m_bPreviewDynamics = Main->GetRegistryDword("StaticPreviewDynamics", 0) ? true : false;
	m_bPreviewStatics = Main->GetRegistryDword("StaticPreviewStatics", 0) ? true : false;
	m_bPreviewTerrain = Main->GetRegistryDword("StaticPreviewTerrain", 0) ? true : false;

	m_bCleanNeeded = false;
	m_bSaveNeeded = false;
}


void CStaticToolDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CStaticToolDlg)
	DDX_Control(pDX, IDC_MAPSCROLL, m_csMapScroll);
	DDX_Control(pDX, IDC_ZOOMOUT, m_csZoomOut);
	DDX_Control(pDX, IDC_ZOOMIN, m_csZoomIn);
	DDX_Control(pDX, IDC_STATIC_GROUP3, m_cbGroup3);
	DDX_Control(pDX, IDC_STATIC_GROUP2, m_cbGroup2);
	DDX_Control(pDX, IDC_STATIC_GROUP1, m_cbGroup1);
	DDX_Control(pDX, IDC_STATIC_LABEL2, m_csLabel2);
	DDX_Control(pDX, IDC_STATIC_LABEL1, m_csLabel1);
	DDX_Control(pDX, IDC_STATIC_TAGGED, m_csTagged);
	DDX_Control(pDX, IDC_STATIC_SCANNED, m_csScanned);
	DDX_Control(pDX, IDC_STATIC_REMOVE, m_cbRemove);
	DDX_Control(pDX, IDC_STATIC_PROGRESS, m_cpProgress);
	DDX_Control(pDX, IDC_STATIC_LIST, m_clcList);
	DDX_Control(pDX, IDC_STATIC_EXISTING, m_cbExisting);
	DDX_Control(pDX, IDC_STATIC_DYNAMIC, m_cbDynamic);
	DDX_Control(pDX, IDC_STATIC_CRITERIA, m_ceCriteria);
	DDX_Control(pDX, IDC_STATIC_ADD, m_cbAdd);
	DDX_Control(pDX, IDC_STATIC_MAP2, m_uoMap);
	DDX_Control(pDX, IDC_STATIC_PREVIEW, m_Preview);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CStaticToolDlg, CDialog)
	//{{AFX_MSG_MAP(CStaticToolDlg)
	ON_COMMAND(ID_CRITERIA_ARTIDS_EXCLUDE, OnCriteriaArtidsExclude)
	ON_COMMAND(ID_CRITERIA_ATTRIBUTES_EXCLUDE, OnCriteriaAttributesExclude)
	ON_COMMAND(ID_CRITERIA_ARTIDS_INCLUDE, OnCriteriaArtidsInclude)
	ON_COMMAND(ID_CRITERIA_ATTRIBUTES_INCLUDE, OnCriteriaAttributesInclude)
	ON_COMMAND(ID_CRITERIA_REGIONS_EXCLUDE, OnCriteriaRegionsExclude)
	ON_COMMAND(ID_CRITERIA_REGIONS_ADD, OnCriteriaRegionsAdd)
	ON_COMMAND(ID_CRITERIA_TYPES_EXCLUDE, OnCriteriaTypesExclude)
	ON_COMMAND(ID_CRITERIA_TYPES_INCLUDE, OnCriteriaTypesInclude)
	ON_COMMAND(ID_FILE_APPLYPATCHFILE, OnFileApplypatchfile)
	ON_COMMAND(ID_FILE_LOADPATCHFILE, OnFileLoadpatchfile)
	ON_COMMAND(ID_FILE_SAVEPATCHFILE, OnFileSavepatchfile)
	ON_COMMAND(ID_FILE_SELECTWORLDFILE, OnFileSelectworldfile)
	ON_COMMAND(ID_OPTIONS_PREVIEWWINDOW_SHOWEXISTINGSTATICS, OnOptionsPreviewwindowShowexistingstatics)
	ON_COMMAND(ID_OPTIONS_PREVIEWWINDOW_SHOWTAGGEDDYNAMICS, OnOptionsPreviewwindowShowtaggeddynamics)
	ON_COMMAND(ID_OPTIONS_PREVIEWWINDOW_SHOWTERRAIN, OnOptionsPreviewwindowShowterrain)
	ON_WM_SIZE()
	ON_COMMAND(ID_FILE_GO, OnFileGo)
	ON_BN_CLICKED(IDC_ZOOMIN, OnZoomin)
	ON_BN_CLICKED(IDC_ZOOMOUT, OnZoomout)
	ON_WM_LBUTTONDOWN()
	ON_WM_CLOSE()
	ON_UPDATE_COMMAND_UI(ID_CRITERIA_REGIONS_ADD, OnUpdateCriteriaRegionsAdd)
	ON_UPDATE_COMMAND_UI(ID_CRITERIA_REGIONS_EXCLUDE, OnUpdateCriteriaRegionsExclude)
	ON_BN_CLICKED(IDC_STATIC_DYNAMIC, OnStaticDynamic)
	ON_BN_CLICKED(IDC_STATIC_EXISTING, OnStaticExisting)
	ON_BN_CLICKED(IDC_STATIC_REMOVE, OnStaticRemove)
	ON_BN_CLICKED(IDC_STATIC_ADD, OnStaticAdd)
	ON_COMMAND(ID_CRITERIA_REGIONS_MODIFY, OnCriteriaRegionsModify)
	ON_COMMAND(ID_OPTIONS_MAPWINDOW_SETREGIONCOLORS, OnOptionsMapwindowSetregioncolors)
	ON_COMMAND(ID_OPTIONS_MAPWINDOW_SHOWEXCLUDEREGIONS, OnOptionsMapwindowShowexcluderegions)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_MAPWINDOW_SHOWEXCLUDEREGIONS, OnUpdateOptionsMapwindowShowexcluderegions)
	ON_COMMAND(ID_OPTIONS_MAPWINDOW_SHOWINCLUDEREGIONS, OnOptionsMapwindowShowincluderegions)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_MAPWINDOW_SHOWINCLUDEREGIONS, OnUpdateOptionsMapwindowShowincluderegions)
	ON_COMMAND(ID_FILE_CLEANWORLDFILE, OnFileCleanworldfile)
	ON_COMMAND(ID_FILE_EXIT, OnFileExit)
	ON_COMMAND(ID_FILE_CREATEIMPORTFILE, OnFileCreateimportfile)
	ON_COMMAND(ID_STATICTOOL_HELP, OnStatictoolHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStaticToolDlg message handlers

void CStaticToolDlg::OnCriteriaArtidsExclude() 
{
	CStaticArtCriteriaDlg dlg;
	dlg.m_csTitle = "Static Art IDs to exclude";
	dlg.m_aSelections.Copy(m_wArtExclude);
	if ( dlg.DoModal() == IDOK )
	{
		m_wArtExclude.RemoveAll();
		m_wArtExclude.Copy(dlg.m_aSelections);
		UpdateCriteriaWindow();
	}
	CString csTemp;
	for ( int i = 0; i < m_wArtExclude.GetSize(); i++ )
	{
		CString csThis;
		if ( i == m_wArtExclude.GetUpperBound() )
			csThis.Format("%ld", m_wArtExclude.GetAt(i));
		else
			csThis.Format("%ld ", m_wArtExclude.GetAt(i));
		csTemp += csThis;
	}
	Main->PutRegistryString("StaticArtExclude", csTemp);
}

void CStaticToolDlg::OnCriteriaAttributesExclude() 
{
	CAttrSelectionDlg dlg;
	dlg.m_csCaption = "Excluded Attributes";
	dlg.m_dwAttr = m_dwAttrExcludeMask;
	if ( dlg.DoModal() == IDOK )
		m_dwAttrExcludeMask = dlg.m_dwAttr;
	Main->PutRegistryDword("StaticAttrExclude", m_dwAttrExcludeMask);
	UpdateCriteriaWindow();
}

void CStaticToolDlg::OnCriteriaArtidsInclude() 
{
	CStaticArtCriteriaDlg dlg;
	dlg.m_csTitle = "Static Art IDs to include";
	dlg.m_aSelections.Copy(m_wArtInclude);
	if ( dlg.DoModal() == IDOK )
	{
		m_wArtInclude.RemoveAll();
		m_wArtInclude.Copy(dlg.m_aSelections);
		UpdateCriteriaWindow();
	}
	CString csTemp;
	for ( int i = 0; i < m_wArtInclude.GetSize(); i++ )
	{
		CString csThis;
		if ( i == m_wArtInclude.GetUpperBound() )
			csThis.Format("%ld", m_wArtInclude.GetAt(i));
		else
			csThis.Format("%ld ", m_wArtInclude.GetAt(i));
		csTemp += csThis;
	}
	Main->PutRegistryString("StaticArtInclude", csTemp);
}

void CStaticToolDlg::OnCriteriaAttributesInclude() 
{
	CAttrSelectionDlg dlg;
	dlg.m_csCaption = "Included Attributes";
	dlg.m_dwAttr = m_dwAttrIncludeMask;
	if ( dlg.DoModal() == IDOK )
		m_dwAttrIncludeMask = dlg.m_dwAttr;
	Main->PutRegistryDword("StaticAttrInclude", m_dwAttrIncludeMask);
	UpdateCriteriaWindow();
}

void CStaticToolDlg::OnCriteriaRegionsExclude() 
{
	m_bExcludeRegions = ! m_bExcludeRegions;
	CMenu * pMenu = this->GetMenu();
	if ( pMenu )
	{
		pMenu->CheckMenuItem( ID_CRITERIA_REGIONS_EXCLUDE, m_bExcludeRegions ? MF_CHECKED : MF_UNCHECKED );
		pMenu->CheckMenuItem( ID_CRITERIA_REGIONS_ADD, MF_UNCHECKED );
	}
	m_bIncludeRegions = false;
}

void CStaticToolDlg::OnCriteriaRegionsAdd() 
{
	m_bIncludeRegions = ! m_bIncludeRegions;
	CMenu * pMenu = this->GetMenu();
	if ( pMenu )
	{
		pMenu->CheckMenuItem( ID_CRITERIA_REGIONS_ADD, m_bIncludeRegions ? MF_CHECKED : MF_UNCHECKED );
		pMenu->CheckMenuItem( ID_CRITERIA_REGIONS_EXCLUDE, MF_UNCHECKED );
	}
	m_bExcludeRegions = false;
}

void CStaticToolDlg::OnCriteriaTypesExclude() 
{
	CTypeSelectionDlg dlg;
	dlg.m_aTypes.Copy(this->m_dwTypeExclude);
	dlg.m_csCaption = "Excluded Item Types";
	if ( dlg.DoModal() == IDOK )
	{
		m_dwTypeExclude.RemoveAll();
		m_dwTypeExclude.Copy(dlg.m_aTypes);
	}
	CString csTemp;
	for ( int i = 0; i < m_dwTypeExclude.GetSize(); i++ )
	{
		CString csThis;
		if ( i == m_dwTypeExclude.GetUpperBound() )
			csThis.Format("%ld", m_dwTypeExclude.GetAt(i));
		else
			csThis.Format("%ld ", m_dwTypeExclude.GetAt(i));
		csTemp += csThis;
	}
	Main->PutRegistryString("StaticTypeExclude", csTemp);
	UpdateCriteriaWindow();
}

void CStaticToolDlg::OnCriteriaTypesInclude() 
{
	CTypeSelectionDlg dlg;
	dlg.m_aTypes.Copy(this->m_dwTypeInclude);
	dlg.m_csCaption = "Included Item Types";
	if ( dlg.DoModal() == IDOK )
	{
		m_dwTypeInclude.RemoveAll();
		m_dwTypeInclude.Copy(dlg.m_aTypes);
	}
	CString csTemp;
	for ( int i = 0; i < m_dwTypeInclude.GetSize(); i++ )
	{
		CString csThis;
		if ( i == m_dwTypeInclude.GetUpperBound() )
			csThis.Format("%ld", m_dwTypeInclude.GetAt(i));
		else
			csThis.Format("%ld ", m_dwTypeInclude.GetAt(i));
		csTemp += csThis;
	}
	Main->PutRegistryString("StaticTypeInclude", csTemp);
	UpdateCriteriaWindow();
}

void CStaticToolDlg::OnFileApplypatchfile() 
{
	if ( m_bSaveNeeded )
	{
		switch( AfxMessageBox("Do you wish to save your patch data first?", MB_YESNOCANCEL) )
		{
		case IDYES:
			OnFileSavepatchfile();
			break;
		case IDNO:
			break;
		case IDCANCEL:
			return;
		}
	}
	CWaitCursor hourglass;
	if ( !m_patch.Apply() )
	{
		CString csMessage;
		csMessage.Format("Error (%s) applying patch!", m_patch.GetErrorString() );
		AfxMessageBox(csMessage, MB_OK);
	}
	m_bCleanNeeded = true;
}

void CStaticToolDlg::OnFileLoadpatchfile() 
{
	if ( !m_patch.IsEmpty() )
	{
		CString csMessage;
		csMessage.Format("Patch data has already been read.  Do you wish to merge the data together?");
		int idResponse = AfxMessageBox(csMessage, MB_YESNOCANCEL);
		switch ( idResponse )
		{
		case IDCANCEL:
			return;
		case IDYES:
			m_bSaveNeeded = true;
			break;
		case IDNO:
			m_patch.Clean();
			break;
		default:
			return;
		}
	}
#ifndef _DEBUG
	char szCwd[MAX_PATH];
	memset(&szCwd[0], 0x00, sizeof(szCwd));
	getcwd(szCwd, sizeof(szCwd));
	CFileDialog dlg(TRUE, ".pat", NULL, 0, "Patch Files (*.pat)|*.pat||", NULL);
	if ( dlg.DoModal() == IDOK )
	{
		if ( !m_patch.Read(dlg.GetFileName()) )
#else
		if ( !m_patch.Read("c:\\sphere\\save\\test.pat") )
#endif
		{
			CString csMessage;
			csMessage.Format("Error (%s) loading patch file", m_patch.GetErrorString() );
			AfxMessageBox(csMessage, MB_OK);
		}
		// Put the patch objects into our lists
		DWORD dwDataSize = m_patch.GetDataBlockSize(MUL_Statics0);
		if ( dwDataSize != 0 )
		{
			BYTE * pData = new BYTE [dwDataSize];
			m_patch.CopyDataBlock(MUL_Statics0, pData, dwDataSize);
			BYTE * pPtr = pData;
			while ( pPtr < pData + dwDataSize )
			{
				struct UOStaticPatchObject * pPatch = (struct UOStaticPatchObject *) pPtr;
				CUOStaticItem * pStatic = new CUOStaticItem;
				memcpy(pStatic->GetDataPtr(), pPatch, sizeof(struct UOStaticPatchObject));
				if ( pPatch->bFlags & 0x01 )
				{
					// This is a static that we are removing
					if ( m_pStaticItemList.IsEmpty() )
						m_pStaticItemList.AddHead((CObject *) pStatic);
					else
						m_pStaticItemList.AddTail((CObject *) pStatic);
				}
				else
				{
					// This is a dynamic object that we are adding
					if ( m_pPatchItemList.IsEmpty() )
						m_pPatchItemList.AddHead((CObject *) pStatic);
					else
						m_pPatchItemList.AddTail((CObject *)  pStatic);
				}
				pPtr += sizeof(struct UOStaticPatchObject);
			}
			delete [] pData;
		}
#ifndef _DEBUG
	}
	chdir(szCwd);
#endif
	UpdateCriteriaWindow();	
}

void CStaticToolDlg::OnFileSavepatchfile() 
{
#ifndef _DEBUG
	char szCwd[MAX_PATH];
	memset(&szCwd[0], 0x00, sizeof(szCwd));
	getcwd(szCwd, sizeof(szCwd));
	CFileDialog dlg(FALSE, ".pat", NULL, OFN_OVERWRITEPROMPT, "Patch Files (*.pat)|*.pat||", NULL);
	if ( dlg.DoModal() == IDOK )
	{
#endif
		CWaitCursor hourglass;
		CFileReadProgress progress;
		progress.Create(IDD_FILEREAD_PROGRESS);
		progress.SetWindowText("Saving patch file");
		progress.SetRange32(0, m_pStaticItemList.GetCount());
		progress.SetPos(0);
		progress.m_csMessage.SetWindowText("Handling changed existing static objects");
		int i = 0;
		// move the data to the patch object

		// First the existing static stuff
		POSITION pos = m_pStaticItemList.GetHeadPosition();
		while ( pos )
		{
			CUOStaticItem * pStatic = (CUOStaticItem *) m_pStaticItemList.GetNext(pos);
			if ( pStatic )
			{
				struct UOStaticPatchObject patch;
				patch.dwBlockIndex = pStatic->m_dwBlockIndex;
				patch.bFlags = pStatic->m_bFlag;
				memcpy(&patch.blockData, &(pStatic->m_blockData), sizeof(tUOStaticRec));
				m_patch.Insert(MUL_Statics0, sizeof(UOStaticPatchObject), (BYTE *) &patch);
			}
			progress.SetPos(i++);
		}
		progress.SetRange32(0, m_pWorldItemList.GetCount());
		progress.SetPos(0);
		progress.m_csMessage.SetWindowText("Handling new static objects");
		i = 0;
		// New statics
		pos = m_pWorldItemList.GetHeadPosition();
		while ( pos )
		{
			CWorldItem * pItem = (CWorldItem *) m_pWorldItemList.GetNext(pos);
			progress.SetPos(i++);
			if ( pItem && pItem->m_bStaticFlag )
			{
				struct UOStaticPatchObject patch;
				patch.dwBlockIndex = pItem->GetMapBlock();
				patch.bFlags = 0x04;
				patch.blockData.bX = pItem->GetBlockX();
				patch.blockData.bY = pItem->GetBlockY();
				patch.blockData.bZ = (BYTE) pItem->m_pZ;
				patch.blockData.wArtID = pItem->m_wArtID ? pItem->m_wArtID : (WORD) DefLookup(pItem->m_csBaseID);
				if ( patch.blockData.wArtID == 0 )
					continue;
				patch.blockData.wColor = pItem->m_wColor;
				m_patch.Insert(MUL_Statics0, sizeof(UOStaticPatchObject), (BYTE *) &patch);
			}
		}
		progress.DestroyWindow();
		// Write the patch file
#ifndef _DEBUG
		m_patch.Write(dlg.GetFileName());
	}
	chdir(szCwd);
#else
	m_patch.Write("c:\\sphere\\save\\test.pat");
#endif
	m_bSaveNeeded = false;
}

void CStaticToolDlg::OnFileSelectworldfile() 
{
	CString csOldWF = m_csWorldFile;
#ifndef _DEBUG
	char szCwd[MAX_PATH];
	memset(&szCwd[0], 0x00, sizeof(szCwd));
	getcwd(szCwd, sizeof(szCwd));
	CFileDialog dlg(TRUE, NULL, NULL, 0, "Script Files (*.scp)|*.scp||");
	if ( dlg.DoModal() == IDOK )
		m_csWorldFile = dlg.GetPathName();
	chdir(szCwd);
#else
	m_csWorldFile = "c:\\sphere_test\\save\\sphereworld.scp";
#endif
	if ( m_csWorldFile != csOldWF )
		LoadWorldfile();
	UpdateCriteriaWindow();
	FillItemListControl();
}

void CStaticToolDlg::OnOptionsPreviewwindowShowexistingstatics() 
{
	m_bPreviewStatics = !m_bPreviewStatics;
	CMenu * pMenu = this->GetMenu();
	if ( pMenu )
		pMenu->CheckMenuItem(ID_OPTIONS_PREVIEWWINDOW_SHOWEXISTINGSTATICS , m_bPreviewStatics ? MF_CHECKED : MF_UNCHECKED);
	Main->PutRegistryDword("StaticPreviewStatics", (DWORD) m_bPreviewStatics);
	SetPreviewFlags();
}

void CStaticToolDlg::OnOptionsPreviewwindowShowtaggeddynamics() 
{
	m_bPreviewDynamics = !m_bPreviewDynamics;
	CMenu * pMenu = this->GetMenu();
	if ( pMenu )
		pMenu->CheckMenuItem(ID_OPTIONS_PREVIEWWINDOW_SHOWTAGGEDDYNAMICS , m_bPreviewDynamics ? MF_CHECKED : MF_UNCHECKED);
	Main->PutRegistryDword("StaticPreviewDynamics", (DWORD) m_bPreviewDynamics);
	SetPreviewFlags();
}

void CStaticToolDlg::OnOptionsPreviewwindowShowterrain() 
{
	m_bPreviewTerrain = !m_bPreviewTerrain;
	CMenu * pMenu = this->GetMenu();
	if ( pMenu )
		pMenu->CheckMenuItem(ID_OPTIONS_PREVIEWWINDOW_SHOWTERRAIN, m_bPreviewTerrain ? MF_CHECKED : MF_UNCHECKED);
	Main->PutRegistryDword("StaticPreviewTerrain", (DWORD) m_bPreviewTerrain);
	SetPreviewFlags();
}

void CStaticToolDlg::UpdateCriteriaWindow()
{
	CString csCriteria, csTemp;
	if ( m_csWorldFile != "" )
	{
		csTemp.Format("Using worldfile %s%c%c", m_csWorldFile, 0x0d, 0x0a);
		csCriteria += csTemp;
	}
	if ( m_dwAttrExcludeMask != 0 )
	{
		csTemp.Format("%c%cExcluding items with the following attributes: %c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		csCriteria += csTemp;
		for ( int i = 0; i < 16; i++ )
		{
			if ( m_dwAttrExcludeMask & ( 1 << i ) )
			{
				csTemp.Format("\t %s%c%c", AttributeStrings[i], 0x0d, 0x0a);
				csCriteria += csTemp;
			}
		}
	}
	if ( m_dwAttrIncludeMask != 0 )
	{
		csTemp.Format("%c%cIncluding items with the following attributes: %c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		csCriteria += csTemp;
		for ( int i = 0; i < 16; i++ )
		{
			if ( m_dwAttrIncludeMask & ( 1 << i ) )
			{
				csTemp.Format("\t %s%c%c", AttributeStrings[i], 0x0d, 0x0a);
				csCriteria += csTemp;
			}
		}
	}
	if ( m_dwTypeExclude.GetSize() > 0 )
	{
		csTemp.Format("%c%cExcluding items with the following types: %c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		csCriteria += csTemp;
		for ( int i = 0; i < m_dwTypeExclude.GetSize(); i++ )
		{
			csTemp.Format("\t %s%c%c", ItemTypeName[m_dwTypeExclude.GetAt(i)], 0x0d, 0x0a);
			csCriteria += csTemp;
		}
	}
	if ( m_dwTypeInclude.GetSize() > 0 )
	{
		csTemp.Format("%c%cIncluding items with the following types: %c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		csCriteria += csTemp;
		for ( int i = 0; i < m_dwTypeInclude.GetSize(); i++ )
		{
			csTemp.Format("\t %s%c%c", ItemTypeName[m_dwTypeInclude.GetAt(i)], 0x0d, 0x0a);
			csCriteria += csTemp;
		}
	}
	if ( m_wArtExclude.GetSize() > 0 )
	{
		csTemp.Format("%c%cExcluding items with the following art IDs: %c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		csCriteria += csTemp;
		for ( int i = 0; i < m_wArtExclude.GetSize(); i++ )
		{
			csTemp.Format("\t %04x%c%c", m_wArtExclude.GetAt(i), 0x0d, 0x0a);
			csCriteria += csTemp;
		}
	}
	if ( m_wArtInclude.GetSize() > 0 )
	{
		csTemp.Format("%c%cIncluding items with the following art IDs: %c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		csCriteria += csTemp;
		for ( int i = 0; i < m_wArtInclude.GetSize(); i++ )
		{
			csTemp.Format("\t %04x%c%c", m_wArtInclude.GetAt(i), 0x0d, 0x0a);
			csCriteria += csTemp;
		}
	}
	if ( m_aRectInclude.GetSize() > 0 )
	{
		csTemp.Format("%c%cIncluding items in the following map rectangles: %c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		csCriteria += csTemp;
		for ( int i = 0; i < m_aRectInclude.GetSize(); i++ )
		{
			CRect * pRect = (CRect *) m_aRectInclude.GetAt(i);
			if ( !pRect )
				continue;
			csTemp.Format("\t %ld, %ld, %ld, %ld%c%c", pRect->left, pRect->top, pRect->right, pRect->bottom, 0x0d, 0x0a);
			csCriteria += csTemp;
		}
	}
	if ( m_aRectExclude.GetSize() > 0 )
	{
		csTemp.Format("%c%cExcluding items in the following map rectangles: %c%c", 0x0d, 0x0a, 0x0d, 0x0a);
		csCriteria += csTemp;
		for ( int i = 0; i < m_aRectExclude.GetSize(); i++ )
		{
			CRect * pRect = (CRect *) m_aRectExclude.GetAt(i);
			if ( !pRect )
				continue;
			csTemp.Format("\t %ld, %ld, %ld, %ld%c%c", pRect->left, pRect->top, pRect->right, pRect->bottom, 0x0d, 0x0a);
			csCriteria += csTemp;
		}
	}
	this->m_ceCriteria.SetWindowText(csCriteria);
	this->m_ceCriteria.SetSel(-1, 0);
}

void CStaticToolDlg::OnSize(UINT nType, int cx, int cy) 
{
	// Make sure we aren't trying to be narrower or shorter than the original
	if ( cx < m_crDefault.Width() )
		cx = m_crDefault.Width();
	if ( cy < m_crDefault.Height() )
		cy = m_crDefault.Height();

	Main->PutRegistryDword("StaticToolWidth", cx);
	Main->PutRegistryDword("StaticToolHeight", cy);

	int dx, dy;
	dx = 0; dy = 0;
	CRect crNew;
	GetClientRect(crNew);
	//ScreenToClient(crNew);
	dx = crNew.Width() - m_crDefault.Width();
	dy = crNew.Height() - m_crDefault.Height();
	double scaleX, scaleY;
	scaleX = crNew.Width() * 1.0 / m_crDefault.Width();
	scaleY = crNew.Height() * 1.0 / m_crDefault.Height();

	int left, top, width, height, right;
	CRect crAnchor, crAnchor2;
	// Resize the map window -- it will have the same top/left and same scale as default.
	width = int (m_crMap.Width() * scaleX);
	height = int (m_crMap.Height() * scaleY);
	m_uoMap.MoveWindow(m_crMap.left, m_crMap.top, width, height);
	m_uoMap.GetWindowRect(&crAnchor);
	ScreenToClient(crAnchor);
	left = m_crMapScroll.left - m_crMap.right + crAnchor.right;
	m_csMapScroll.MoveWindow(left, m_crMapScroll.top, m_crMapScroll.Width(), m_crMapScroll.Height());
	m_csZoomOut.MoveWindow(left, m_crZoomOut.top, m_crZoomOut.Width(), m_crZoomOut.Height());
	left = m_crZoomIn.left - m_crMap.right + crAnchor.right;
	m_csZoomIn.MoveWindow(left, m_crZoomIn.top, m_crZoomIn.Width(), m_crZoomIn.Height());
	// Move/resize the preview window -- top will be the same.
	left = crAnchor.right + (m_crPreview.left - m_crMap.right);
	right = m_crDefault.right - m_crPreview.right;
	width = crNew.right - right - left;
	height = int (m_crPreview.Height() * scaleY);
	m_Preview.MoveWindow(left, m_crPreview.top, width, height);
	m_Preview.GetWindowRect(&crAnchor2);
	ScreenToClient(crAnchor2);
	// Move/resize the criteria window objects -- relative to the map and preview windows.
	width = crAnchor2.left - (m_crPreview.left - m_crGroup1.right) - m_crGroup1.left;
	height = crNew.Height() - crAnchor.Height() - m_crGroup2.Height() - (m_crDefault.Height() - m_crGroup2.Height() - m_crMap.Height() - m_crGroup1.Height());
	m_cbGroup1.MoveWindow(m_crGroup1.left, crAnchor.bottom + (m_crGroup1.top - m_crMap.bottom), width, height);
	m_cbGroup1.GetWindowRect(&crAnchor);
	ScreenToClient(crAnchor);
	width = ( crAnchor.right - (m_crGroup1.right - m_crCriteria.right) ) - m_crCriteria.left;
	top = crAnchor.top + (m_crCriteria.top - m_crGroup1.top);
	height = ( crAnchor.bottom - (m_crGroup1.bottom - m_crCriteria.bottom) ) - top;
	m_ceCriteria.MoveWindow(m_crCriteria.left, top, width, height);
	// Move/resize the Analysis window objects -- relative to the criteria window
	width = crAnchor.Width();
	height = m_crGroup2.Height();	// doesn't need to change.
	m_cbGroup2.MoveWindow(m_crGroup2.left, crAnchor.bottom + (m_crGroup2.top - m_crGroup1.bottom), width, height);
	m_cbGroup2.GetWindowRect(&crAnchor);
	ScreenToClient(crAnchor);
	// Progress dialog
	width = crAnchor.right - m_crProgress.left - (m_crGroup2.right - m_crProgress.right);
	height = m_crProgress.Height();
	top = crAnchor.top + (m_crProgress.top - m_crGroup2.top);
	m_cpProgress.MoveWindow(m_crProgress.left, top, width, height);
	// Text fields
	top = crAnchor.top + (m_crLabel1.top - m_crGroup2.top);
	m_csLabel1.MoveWindow(m_crLabel1.left, top, m_crLabel1.Width(), m_crLabel1.Height());
	width = ( ( crAnchor.Width() - (m_crGroup2.right - m_crTagged.right) - (m_crLabel1.left - m_crGroup2.left) 
		- (m_crScanned.left - m_crLabel1.right) - (m_crLabel2.left - m_crScanned.right) - (m_crTagged.left - m_crLabel2.right) )
		- m_crLabel1.Width() - m_crLabel2.Width() ) / 2;
	m_csScanned.MoveWindow(m_crScanned.left, top, width, m_crScanned.Height());
	m_csScanned.GetWindowRect(&crAnchor2);
	ScreenToClient(crAnchor2);
	m_csLabel2.MoveWindow(crAnchor2.right + (m_crLabel2.left - m_crScanned.right), top, m_crLabel2.Width(), m_crLabel2.Height());
	m_csLabel2.GetWindowRect(&crAnchor2);
	ScreenToClient(crAnchor2);
	m_csTagged.MoveWindow(crAnchor2.right + (m_crTagged.left - m_crLabel2.right), top, width, m_crTagged.Height());
	// Move/resize the StaticObjects group
	m_Preview.GetWindowRect(&crAnchor);
	m_cbGroup1.GetWindowRect(&crAnchor2);
	ScreenToClient(crAnchor);
	ScreenToClient(crAnchor2);
	width = crAnchor.Width();
	top = crAnchor2.top;
	left = crAnchor.left;
	m_cbGroup2.GetWindowRect(&crAnchor2);
	ScreenToClient(crAnchor2);
	height = crAnchor2.bottom - top;
	m_cbGroup3.MoveWindow(left, top, width, height);
	m_cbExisting.MoveWindow(left + (m_crExisting.left - m_crGroup3.left), top + (m_crExisting.top - m_crGroup3.top), m_crExisting.Width(), m_crExisting.Height());
	m_cbDynamic.MoveWindow(left + (m_crDynamic.left - m_crGroup3.left), top + (m_crDynamic.top - m_crGroup3.top), m_crDynamic.Width(), m_crDynamic.Height());
	height = height - (m_crList.top - m_crGroup3.top) - (m_crGroup3.bottom - m_crList.bottom);
	m_clcList.MoveWindow(left + (m_crList.left - m_crGroup3.left), top + (m_crList.top - m_crGroup3.top), width - (m_crGroup3.Width() - m_crList.Width()), height);
	m_clcList.GetWindowRect(&crAnchor2);
	ScreenToClient(crAnchor2);
	top = crAnchor2.bottom + (m_crAdd.top - m_crList.bottom);
	m_cbAdd.MoveWindow(left + (m_crAdd.left - m_crGroup3.left), top, m_crAdd.Width(), m_crAdd.Height());
	m_cbRemove.MoveWindow(left + (m_crRemove.left - m_crGroup3.left), top, m_crRemove.Width(), m_crRemove.Height());

	
	if ( this->IsWindowVisible() )
	{
		CDialog::OnSize(nType, cx, cy);
		this->Invalidate();
		FillItemListControl();
	}
}

BOOL CStaticToolDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Get the CRects for the controls on this dialog
	GetClientRect(&m_crDefault);

	m_cbAdd.GetWindowRect(&m_crAdd);
	m_cbDynamic.GetWindowRect(&m_crDynamic);
	m_cbExisting.GetWindowRect(&m_crExisting);
	m_cbGroup1.GetWindowRect(&m_crGroup1);
	m_cbGroup2.GetWindowRect(&m_crGroup2);
	m_cbGroup3.GetWindowRect(&m_crGroup3);
	m_cbRemove.GetWindowRect(&m_crRemove);
	m_ceCriteria.GetWindowRect(&m_crCriteria);
	m_clcList.GetWindowRect(&m_crList);
	m_cpProgress.GetWindowRect(&m_crProgress);
	m_csLabel1.GetWindowRect(&m_crLabel1);
	m_csLabel2.GetWindowRect(&m_crLabel2);
	m_uoMap.GetWindowRect(&m_crMap);
	m_Preview.GetWindowRect(&m_crPreview);
	m_csScanned.GetWindowRect(&m_crScanned);
	m_csTagged.GetWindowRect(&m_crTagged);
	m_csMapScroll.GetWindowRect(&m_crMapScroll);
	m_csZoomIn.GetWindowRect(&m_crZoomIn);
	m_csZoomOut.GetWindowRect(&m_crZoomOut);

	ScreenToClient(m_crAdd);
	ScreenToClient(m_crCriteria);
	ScreenToClient(m_crDynamic);
	ScreenToClient(m_crExisting);
	ScreenToClient(m_crList);
	ScreenToClient(m_crMap);
	ScreenToClient(m_crPreview);
	ScreenToClient(m_crProgress);
	ScreenToClient(m_crRemove);
	ScreenToClient(m_crScanned);
	ScreenToClient(m_crTagged);
	ScreenToClient(m_crLabel1);
	ScreenToClient(m_crLabel2);
	ScreenToClient(m_crGroup1);
	ScreenToClient(m_crGroup2);
	ScreenToClient(m_crGroup3);
	ScreenToClient(m_crMapScroll);
	ScreenToClient(m_crZoomIn);
	ScreenToClient(m_crZoomOut);

	m_clcList.SetItemCount(65536);
	m_clcList.InsertColumn(0, "Item name", LVCFMT_LEFT,100, 0);
	m_clcList.InsertColumn(1, "Art ID", LVCFMT_LEFT, 50, 1);
	m_clcList.InsertColumn(2, "Map Block", LVCFMT_LEFT, 80, 2);
	m_clcList.InsertColumn(3, "Block Location", LVCFMT_LEFT, 70, 3);
	m_clcList.InsertColumn(4, "Color", LVCFMT_LEFT, 50, 4);
	m_clcList.InsertColumn(5, "Status", LVCFMT_LEFT, 90, 5);
	m_clcList.InsertColumn(6, "Serial", LVCFMT_LEFT, 90, 6);

	m_csTagged.SetWindowText("0");
	m_csScanned.SetWindowText("0");

	UpdateCriteriaWindow();

	// Set the client paths
	CString csRegKey, csValue;
	csRegKey.Format("Mul%ld", VERFILE_MAP0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_uoMap.SetCustomMulPath(VERFILE_MAP0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_MAP2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_uoMap.SetCustomMulPath(VERFILE_MAP2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STAIDX0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_uoMap.SetCustomMulPath(VERFILE_STAIDX0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STATICS0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_uoMap.SetCustomMulPath(VERFILE_STATICS0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STAIDX2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_uoMap.SetCustomMulPath(VERFILE_STAIDX2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STATICS2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_uoMap.SetCustomMulPath(VERFILE_STATICS2, csValue);

	csRegKey.Format("Mul%ld", VERFILE_MAP0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_Preview.SetCustomMulFile(VERFILE_MAP0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_MAP2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_Preview.SetCustomMulFile(VERFILE_MAP2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STAIDX0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_Preview.SetCustomMulFile(VERFILE_STAIDX0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STATICS0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_Preview.SetCustomMulFile(VERFILE_STATICS0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STAIDX2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_Preview.SetCustomMulFile(VERFILE_STAIDX2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STATICS2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_Preview.SetCustomMulFile(VERFILE_STATICS2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_ART);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_Preview.SetCustomMulFile(VERFILE_ART, csValue);
	csRegKey.Format("Mul%ld", VERFILE_ART);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_Preview.SetCustomMulFile(VERFILE_ARTIDX, csValue);

	CMenu * pMenu = this->GetMenu();
	if ( pMenu )
	{
		pMenu->CheckMenuItem( ID_OPTIONS_MAPWINDOW_SHOWEXCLUDEREGIONS, m_bDisplayExcludeRegions ? MF_CHECKED : MF_UNCHECKED );
		pMenu->CheckMenuItem( ID_OPTIONS_MAPWINDOW_SHOWINCLUDEREGIONS, m_bDisplayIncludeRegions ? MF_CHECKED : MF_UNCHECKED );
		pMenu->CheckMenuItem(ID_OPTIONS_PREVIEWWINDOW_SHOWEXISTINGSTATICS , m_bPreviewStatics ? MF_CHECKED : MF_UNCHECKED);
		pMenu->CheckMenuItem(ID_OPTIONS_PREVIEWWINDOW_SHOWTAGGEDDYNAMICS , m_bPreviewDynamics ? MF_CHECKED : MF_UNCHECKED);
		pMenu->CheckMenuItem(ID_OPTIONS_PREVIEWWINDOW_SHOWTERRAIN, m_bPreviewTerrain ? MF_CHECKED : MF_UNCHECKED);
	}

	int cx, cy;
	cx = (int) Main->GetRegistryDword("StaticToolWidth", 0);
	cy = (int) Main->GetRegistryDword("StaticToolHeight", 0);
	if ( cx > 0 && cy > 0 )
	{
		int mx, my;
		mx = GetSystemMetrics(SM_CXMAXIMIZED);
		my = GetSystemMetrics(SM_CYMAXIMIZED);
		if ( cx > mx )
			cx = mx;
		if ( cy > my )
			cy = my;
		// We want to keep the bottom right at the same point.
		CRect rect;
		this->GetWindowRect(&rect);
		this->MoveWindow(rect.right - cx, rect.bottom - cy, cx, cy);
	}
	SetPreviewFlags();
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CStaticToolDlg::OnFileGo() 
{
	CWaitCursor hourglass;
	if ( m_csWorldFile == "" )
	{
		AfxMessageBox("No worldfile has been selected.  Select your worldfile from the File menu", MB_OK);
		return;
	}
	bool bLoadWorld = false;
	/*
	if ( m_pWorldItemList.GetCount() != 0 )
	{
		if ( AfxMessageBox("Do you wish to reload the currently selected worldfile?", MB_YESNO) == IDYES )
		{
			CleanupWorldItems();
			bLoadWorld = true;
		}
	}
	else
		bLoadWorld = true;
	*/
	if ( m_pStaticItemList.GetCount() != 0 )
	{
		if ( AfxMessageBox("Do you wish to undo any converted static information?", MB_YESNO) == IDYES )
			CleanupStatics();
	}
	if ( bLoadWorld )
		LoadWorldfile();
	else
	{
		// Just go through and untag each worlditem
		POSITION pos = m_pWorldItemList.GetHeadPosition();
		while ( pos )
		{
			CWorldItem * pItem = (CWorldItem *) m_pWorldItemList.GetNext(pos);
			if ( pItem )
				pItem->m_bStaticFlag = 0;
		}
	}
	AnalyzeWorld();
	m_bSaveNeeded = true;
	UpdatePreviewItems();
}

bool CStaticToolDlg::TestItem(CWorldItem *pItem)
{
	// Ignore items in containers
	if ( pItem->m_dwContainer != 0 )
		return false;
	// Check the coordinates of the object
	for ( int i = 0; i < m_aRectExclude.GetSize(); i++ )
	{
		CRect * pRect = (CRect *) m_aRectExclude.GetAt(i);
		if ( !pRect )
			continue;
		CPoint point(pItem->m_pX, pItem->m_pY);
		if ( pRect->PtInRect(point) )
			return false;
	}
	bool bInclude = false;
	if ( m_aRectInclude.GetSize() == 0 )
		bInclude = true;
	else
	{
		for ( i = 0; i < m_aRectInclude.GetSize(); i++ )
		{
			CRect * pRect = (CRect *) m_aRectInclude.GetAt(i);
			if ( !pRect )
				continue;
			CPoint point(pItem->m_pX, pItem->m_pY);
			if ( pRect->PtInRect(point) )
				bInclude = true;
		}
	}
	if ( !bInclude )
		return false;
	// Check item types
	for ( i = 0; i < m_dwTypeExclude.GetSize(); i++ )
	{
		DWORD dwType = m_dwTypeExclude.GetAt(i);
		if (dwType && dwType == pItem->m_dwItemType)
			return false;
	}
	for ( i = 0; i < m_dwTypeInclude.GetSize(); i++ )
	{
		DWORD dwType = m_dwTypeInclude.GetAt(i);
		if (dwType && dwType == pItem->m_dwItemType)
			return true;
	}
	// Check item attributes
	if ( pItem->m_dwAttr & m_dwAttrExcludeMask )
		return false;
	if ( pItem->m_dwAttr & m_dwAttrIncludeMask )
		return true;
	// Check artwork ids
	for ( i = 0; i < m_wArtExclude.GetSize(); i++ )
	{
		WORD wArtID = m_wArtExclude.GetAt(i);
		if ( wArtID )
		{
			if ( pItem->m_wArtID )
			{
				if ( pItem->m_wArtID == wArtID )
					return false;
			}
			else
			{
				if ( (WORD) DefLookup(pItem->m_csBaseID) == wArtID )
					return false;
			}
		}
	}
	for ( i = 0; i < m_wArtInclude.GetSize(); i++ )
	{
		WORD wArtID = m_wArtInclude.GetAt(i);
		if ( wArtID )
		{
			if ( pItem->m_wArtID )
			{
				if ( pItem->m_wArtID == wArtID )
					return true;
			}
			else
			{
				if ( (WORD) DefLookup(pItem->m_csBaseID) == wArtID )
					return true;
			}
		}
	}
	return false;
}

void CStaticToolDlg::CleanupWorldItems()
{
	POSITION pos = m_pWorldItemList.GetHeadPosition();
	while ( pos )
	{
		CWorldItem * pItem = (CWorldItem *) m_pWorldItemList.GetNext(pos);
		if ( pItem )
			delete pItem;
		else
		{
			// We've got a problem here
		}
	}
	m_pWorldItemList.RemoveAll();
	if ( this->GetSafeHwnd() != NULL )
	{
		this->SetWindowText("Axis Static Tool");
		this->m_csTagged.SetWindowText("0");
		this->m_csScanned.SetWindowText("0");
	}
}

CStaticToolDlg::~CStaticToolDlg()
{
	CleanupWorldItems();
	CleanupStatics();
	CleanupPatches();
	m_wArtExclude.RemoveAll();
	m_wArtInclude.RemoveAll();
	m_dwTypeExclude.RemoveAll();
	m_dwTypeExclude.RemoveAll();
	CleanupRects();
}

void CStaticToolDlg::LoadWorldfile()
{
	CString csTemp;
	CStdioFile csfWorld;
	CleanupWorldItems();
	if ( !csfWorld.Open(m_csWorldFile, CFile::modeRead | CFile::shareDenyNone) )
	{
		csTemp.Format("Unable to open worldfile %s.  %s", m_csWorldFile, strerror(GetLastError()));
		AfxMessageBox(csTemp, MB_OK);
		return;
	}
	CFileReadProgress progress;
	progress.Create(IDD_FILEREAD_PROGRESS);
	progress.SetWindowText("Loading Worldfile");
	progress.SetRange32(0, csfWorld.GetLength());
	progress.SetPos(0);
	csTemp.Format("Loading worldfile %s", m_csWorldFile);
	progress.m_csMessage.SetWindowText(csTemp);
	int i = 0;

	BOOL bStatus = TRUE;
	while ( bStatus )
	{
		CString csLine;
		DWORD dwOffset = csfWorld.GetPosition();
		bStatus = csfWorld.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( i++ % 50 == 0 )
		{
			progress.SetPos((DWORD) csfWorld.GetPosition());
		}
		if ( csLine.Find("[WORLDITEM") != -1 )
		{
			CWorldItem * pItem = new CWorldItem;
			pItem->m_csBaseID = csLine.Mid(11).SpanExcluding("]");
			pItem->m_csFilename = csfWorld.GetFilePath();
			pItem->m_dwFileOffset = dwOffset;
			pItem->m_csValue = pItem->m_csBaseID;
			pItem->m_bType = TYPE_WORLDITEM;
			if ( !pItem->Read(csfWorld) )
			{
				delete pItem;
			}
			else
			{
				if ( m_pWorldItemList.IsEmpty() )
					m_pWorldItemList.AddHead((CObject *) pItem);
				else
					m_pWorldItemList.AddTail((CObject *) pItem);
			}
		}
	}
	csTemp.Format("Axis Static Tool - %ld dynamic items", m_pWorldItemList.GetCount());
	this->SetWindowText(csTemp);
	csfWorld.Close();

}

void CStaticToolDlg::AnalyzeWorld()
{
	int scanned, tagged, i;
	scanned = tagged = i = 0;
	m_cpProgress.SetRange32(0, m_pWorldItemList.GetCount());
	m_cpProgress.SetPos(0);

	POSITION pos = m_pWorldItemList.GetHeadPosition();
	while ( pos )
	{
		CWorldItem * pItem = (CWorldItem *) m_pWorldItemList.GetNext(pos);
		scanned++;
		if ( !pItem )
			continue;
		if ( i++ % 100 == 0 )
		{
			CString csTemp;
			csTemp.Format("%ld", scanned);
			m_csScanned.SetWindowText(csTemp);
			csTemp.Format("%ld", tagged);
			m_csTagged.SetWindowText(csTemp);
			m_cpProgress.SetPos(scanned);
		}
		if ( TestItem(pItem) )
		{
			tagged++;
			pItem->m_bStaticFlag = 1;
		}
	}
	FillItemListControl();
	m_cpProgress.SetPos(0);
}

void CStaticToolDlg::UpdateObjectListing()
{
	this->m_clcList.ShowWindow(SW_HIDE);
	bool bPreviewFilter = true;
	if ( this->m_cbDynamic.GetCheck() )
	{
		// Add all of the dynamic objects to the list
	}
	if ( this->m_cbExisting.GetCheck() )
	{
		// Add all of the static objects to the list
	}
	this->m_clcList.ShowWindow(SW_SHOW);
}

void CStaticToolDlg::LoadStatics()
{
	/*
	CFile cfIdx, cfMul;
	CString csFilename;
	csFilename.Format("%s\\staidx0.mul", csUOPath);
	if ( !cfIdx.Open(csFilename, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone) )
	{
		CString csMessage;
		csMessage.Format("Unable to open file %s. %s", csFilename, strerror(GetLastError()) );
		AfxMessageBox(csMessage, MB_OK);
		Main->m_log.Add(csMessage);
		return;
	}
	csFilename.Format("%s\\statics0.mul", csUOPath);
	if ( !cfMul.Open(csFilename, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone) )
	{
		CString csMessage;
		csMessage.Format("Unable to open file %s. %s", csFilename, strerror(GetLastError()) );
		AfxMessageBox(csMessage, MB_OK);
		Main->m_log.Add(csMessage);
		cfIdx.Close();
		return;
	}
	CFileReadProgress progress;
	progress.Create(IDD_FILEREAD_PROGRESS);
	progress.m_csMessage.SetWindowText("Loading existing statics");
	progress.SetRange32(0, cfMul.GetLength());
	progress.SetPos(0);
	BOOL bStatus = TRUE;
	DWORD dwBlock = 0;
	DWORD dwItem = 0;
	while ( bStatus )
	{
		// Read the index block
		tUOStaticIdx idx;
		memset(&idx, 0x00, sizeof(idx));
		if ( cfIdx.Read(&idx, sizeof(idx)) != sizeof(idx) )
		{
			bStatus = FALSE;
			break;
		}
		if ( idx.dwStart != 0xFFFFFFFF )
		{
			cfMul.Seek(idx.dwStart, CFile::begin);
			for ( UINT iCount = 0; iCount < idx.dwLength / 7 ; iCount++ )
			{
				if ( dwItem++ % 5000 == 0 )
					progress.SetPos(cfMul.GetPosition());
				CUOStaticItem * pItem = new CUOStaticItem;
				pItem->m_dwBlockIndex = dwBlock;
				pItem->Read(cfMul);
				if ( m_pStaticItemList.IsEmpty() )
					m_pStaticItemList.AddHead((CObject *) pItem);
				else
					m_pStaticItemList.AddTail((CObject *) pItem);
			}
		}
		dwBlock++;
	}
	progress.DestroyWindow();
	cfIdx.Close();
	cfMul.Close();
	*/
}

void CStaticToolDlg::CleanupStatics()
{
	POSITION pos = m_pStaticItemList.GetHeadPosition();
	while ( pos )
	{
		CUOStaticItem * pItem = (CUOStaticItem *) m_pStaticItemList.GetNext(pos);
		if ( pItem )
			delete pItem;
	}
	m_pStaticItemList.RemoveAll();
}

void CStaticToolDlg::CleanupPatches()
{
	POSITION pos = m_pPatchItemList.GetHeadPosition();
	while ( pos )
	{
		CUOStaticItem * pItem = (CUOStaticItem *) m_pPatchItemList.GetNext(pos);
		if ( pItem )
			delete pItem;
	}
	m_pPatchItemList.RemoveAll();
}

void CStaticToolDlg::OnZoomin() 
{
	m_uoMap.ZoomIn();
	FillItemListControl();
}

void CStaticToolDlg::OnZoomout() 
{
	m_uoMap.ZoomOut();
	FillItemListControl();
}

void CStaticToolDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// Which window are we in?
	CRect rect, map;
	ClientToScreen(&point);
	m_csMapScroll.GetWindowRect(&rect);
	m_uoMap.GetWindowRect(&map);
	if ( rect.PtInRect(point) )
	{
		// The scroll button was clicked
		m_csMapScroll.ScreenToClient(&point);
		// Determine which quadrant this was in
		int xq, yq, iq;
		xq = point.x / ( rect.Width() / 3 );
		yq = point.y / ( rect.Height() / 3 );
		iq = (yq * 3) + xq;
		int xScroll = m_uoMap.CtrlToMapX(map.Width()) - m_uoMap.CtrlToMapX(0);
		int yScroll = m_uoMap.CtrlToMapY(map.Height()) - m_uoMap.CtrlToMapY(0);
		int dScroll = (int) sqrt(xScroll * xScroll + yScroll * yScroll);
		switch ( iq )
		{
		case 0:
			m_uoMap.Scroll(7, dScroll);
			break;
		case 1:
			m_uoMap.Scroll(0, yScroll);
			break;
		case 2:
			m_uoMap.Scroll(1, dScroll);
			break;
		case 3:
			m_uoMap.Scroll(6, xScroll);
			break;
		case 4:
			m_uoMap.SetCenter(3072, 2048);
			break;
		case 5:
			m_uoMap.Scroll(2, xScroll);
			break;
		case 6:
			m_uoMap.Scroll(5, dScroll);
			break;
		case 7:
			m_uoMap.Scroll(4, yScroll);
			break;
		case 8:
			m_uoMap.Scroll(3, dScroll);
			break;
		}
		m_Preview.SetArtIndex(m_uoMap.GetCenterBlock());
		return;
	}
	CDialog::OnLButtonDown(nFlags, point);
}

void CStaticToolDlg::OnClose() 
{
	if ( m_pParent )
	{
		CSettingsTab * parent = (CSettingsTab *) m_pParent;
		parent->m_bDeleteStaticTool = TRUE;
	}
	CDialog::OnClose();
}

BEGIN_EVENTSINK_MAP(CStaticToolDlg, CDialog)
    //{{AFX_EVENTSINK_MAP(CStaticToolDlg)
	ON_EVENT(CStaticToolDlg, IDC_STATIC_MAP2, -605 /* MouseDown */, OnMouseDownUOMap, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	ON_EVENT(CStaticToolDlg, IDC_STATIC_MAP2, -607 /* MouseUp */, OnMouseUpUOMap, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	ON_EVENT(CStaticToolDlg, IDC_STATIC_MAP2, -606 /* MouseMove */, OnMouseMoveUOMap, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CStaticToolDlg::OnMouseDownUOMap(short Button, short Shift, long x, long y) 
{
	CPoint pt(x, y);
	m_ptDown.x = m_uoMap.CtrlToMapX((short)pt.x);
	m_ptDown.y = m_uoMap.CtrlToMapY((short)pt.y);
	m_uoMap.Invalidate();	
}

void CStaticToolDlg::OnMouseUpUOMap(short Button, short Shift, long x, long y) 
{
	bool bRefreshList = false;
	CPoint pt(x, y);
	m_ptUp.x = m_uoMap.CtrlToMapX((short)pt.x);
	m_ptUp.y = m_uoMap.CtrlToMapY((short)pt.y);
	CRect rect;
	int l, r, t, b;
	if ( m_ptDown.x < m_ptUp.x )
	{
		l = m_ptDown.x;
		r = m_ptUp.x;
	}
	else
	{
		r = m_ptDown.x;
		l = m_ptUp.x;
	}
	if ( m_ptDown.y < m_ptUp.y )
	{
		t = m_ptDown.y;
		b = m_ptUp.y;
	}
	else
	{
		b = m_ptDown.y;
		t = m_ptUp.y;
	}
	rect.SetRect(l, t, r, b);
	if ( m_iDrawRectIndex != -1 )
		m_uoMap.RemoveDrawRectAt(m_iDrawRectIndex);
	m_iDrawRectIndex = m_uoMap.AddDrawRect((short) rect.left, (short) rect.top, (short)rect.Width(), (short)rect.Height(), 2, 0x00FFFFFF);
	if ( m_bIncludeRegions )
	{
		CRect * pRect = new CRect;
		pRect->CopyRect(&rect);
		m_aRectInclude.Add((void *)pRect);
	}
	else if ( m_bExcludeRegions )
	{
		CRect * pRect = new CRect;
		pRect->CopyRect(&rect);
		m_aRectExclude.Add((void *)pRect);
	}
	if ( m_bIncludeRegions || m_bExcludeRegions )
		UpdateMapRegions();
	else
	{
		// Zoom in to this level and re-center
		bRefreshList = true;
		m_uoMap.RemoveDrawRectAt(m_iDrawRectIndex);
		m_iDrawRectIndex = -1;
		m_uoMap.SetCenter((short)rect.CenterPoint().x, (short)rect.CenterPoint().y);
		m_Preview.SetArtIndex(m_uoMap.GetCenterBlock());
		// Determine which zoom level is most appropriate
		CRect clientRect;
		m_uoMap.GetClientRect(&clientRect);
		int wc, wm;
		wc = clientRect.Width();
		wm = rect.Height();
		if ( wm > 8 )
		{
			short scale = 0;
			if ( wm / wc > 8 )
				scale = -3;
			else if ( wm / wc > 4 )
				scale = -2;
			else if ( wm / wc > 2 )
				scale = -1;
			else if ( wc / wm > 128 )
				scale = 7;
			else if ( wc / wm > 64 )
				scale = 6;
			else if ( wc / wm > 32 )
				scale = 5;
			else if ( wc / wm > 16 )
				scale = 4;
			else if ( wc / wm > 8 )
				scale = 3;
			else if ( wc / wm > 4 )
				scale = 2;
			else if ( wc / wm > 2 )
				scale = 1;
			m_uoMap.SetZoomLevel(scale);
		}
	}
	m_uoMap.Invalidate();
	if ( bRefreshList )
		FillItemListControl();
}

void CStaticToolDlg::OnMouseMoveUOMap(short Button, short Shift, long x, long y) 
{
	if ( Button == 0 )
		return;
	CPoint pt(x, y);
	m_ptUp.x = m_uoMap.CtrlToMapX((short)pt.x);
	m_ptUp.y = m_uoMap.CtrlToMapY((short)pt.y);
	CRect rect;
	int l, r, t, b;
	if ( m_ptDown.x < m_ptUp.x )
	{
		l = m_ptDown.x;
		r = m_ptUp.x;
	}
	else
	{
		r = m_ptDown.x;
		l = m_ptUp.x;
	}
	if ( m_ptDown.y < m_ptUp.y )
	{
		t = m_ptDown.y;
		b = m_ptUp.y;
	}
	else
	{
		b = m_ptDown.y;
		t = m_ptUp.y;
	}
	rect.SetRect(l, t, r, b);
	if ( m_iDrawRectIndex != -1 )
		m_uoMap.RemoveDrawRectAt(m_iDrawRectIndex);
	m_iDrawRectIndex = m_uoMap.AddDrawRect((short) rect.left, (short) rect.top, (short)rect.Width(), (short)rect.Height(), 2, 0x00FFFFFF);
	m_uoMap.Invalidate();
}

void CStaticToolDlg::OnUpdateCriteriaRegionsAdd(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bIncludeRegions ? 1 : 0 );
}

void CStaticToolDlg::OnUpdateCriteriaRegionsExclude(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bExcludeRegions ? 1 : 0);
}

void CStaticToolDlg::FillItemListControl()
{
	m_clcList.DeleteAllItems();
	//m_clcList.ShowWindow(SW_HIDE);
	// Determine the bounds of the map preview window.
	CPoint tl, br;
	CRect rect;
	m_uoMap.GetClientRect(rect);
	tl = rect.TopLeft();
	br = rect.BottomRight();
	CRect mapRect;
	mapRect.left = m_uoMap.CtrlToMapX((short)tl.x);
	mapRect.top = m_uoMap.CtrlToMapY((short)tl.y);
	mapRect.right = m_uoMap.CtrlToMapX((short)br.x);
	mapRect.bottom = m_uoMap.CtrlToMapY((short)br.y);
	int index = 0;
	if ( m_cbExisting.GetCheck() )
	{
		// Only list statics if we are at zoomlevel n or higher
		if ( m_uoMap.GetZoomLevel() > 0 )
		{
			CFile staIdx, staMul, tileData;
			CString csFileIdx, csFileMul, csFileTile;
			csFileIdx.Format("%sstaidx0.mul", csUOPath);
			csFileMul.Format("%sstatics0.mul", csUOPath);
			csFileTile.Format("%stiledata.mul", csUOPath);
			if ( staIdx.Open(csFileIdx, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone) )
			{
				if ( staMul.Open(csFileMul, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone) )
				{
					if ( tileData.Open(csFileTile, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone) )
					{
						int YTop, YBottom, XLeft, XRight;
						YTop = mapRect.top / 8;
						YBottom = mapRect.bottom / 8;
						XLeft = mapRect.left / 8;
						XRight = mapRect.right / 8;
						for ( int y = YTop; y <= YBottom; y++ )
						{
							for ( int x = XLeft; x <= XRight; x++ )
							{
								// Seek to the appropriate spot in the index file
								DWORD mapBlock = (x * 512) + y;
								tUOStaticIdx idx;
								staIdx.Seek( (mapBlock * sizeof(idx) ), CFile::begin );
								staIdx.Read(&idx, sizeof(idx));
								if ( idx.dwStart != 0xFFFFFFFF )
								{
									// Seek to the beginning of the data block
									staMul.Seek( (idx.dwStart), CFile::begin );
									for ( DWORD i = 0; i < idx.dwLength / sizeof(tUOStaticRec); i++ )
									{
										tUOStaticRec rec;
										staMul.Read(&rec, sizeof(rec));
										// Check to see if we've already got this guy in the static item list.
										POSITION pos = m_pStaticItemList.GetHeadPosition();
										CUOStaticItem * pStatic = NULL;
										while ( pos )
										{
											CUOStaticItem * pTest = (CUOStaticItem *) m_pStaticItemList.GetNext(pos);
											if ( pTest )
											{
												if ( pTest->m_dwBlockIndex == mapBlock )
												{
													if ( memcmp(&(pTest->m_blockData), &rec, sizeof(tUOStaticRec)) == 0 )
													{
														pStatic = pTest;
														break;
													}
												}
											}
										}
										DWORD tdBlock = rec.wArtID / 32;
										DWORD tdIndex = rec.wArtID % 32;
										tileData.Seek( (836 * 512) + (1188 * tdBlock), CFile::begin);
										CTileDataGroup block;
										tileData.Read(&block, sizeof(block));
										char szBuffer[32];
										memset(&szBuffer[0], 0x00, 32);
										memcpy(&szBuffer[0], block.m_aDataBlocks[tdIndex].m_szName, 20);
										CString csName, csBlock, csBlockLocation, csColour, csID, csStatus;
										csName.Format("%s", szBuffer);
										csID.Format("%04x", rec.wArtID);
										index = m_clcList.InsertItem(index, csName);
										csBlock.Format("%ld", mapBlock);
										csBlockLocation.Format("%d, %d, %d", rec.bX, rec.bY, (char)rec.bZ);
										if ( pStatic )
										{
											if ( pStatic->m_bFlag & 0x2 )
												csStatus = "Removed Static";
											else if ( pStatic->m_bFlag & 0x1 )
												csStatus = "Pending Dynamic";
											else
												csStatus = "Static";
										}
										else
											csStatus = "Static";
										csColour.Format("%0x", rec.wColor);
										m_clcList.SetItemText(index, 1, csID);
										m_clcList.SetItemText(index, 2, csBlock);
										m_clcList.SetItemText(index, 3, csBlockLocation);
										m_clcList.SetItemText(index, 4, csColour);
										m_clcList.SetItemText(index, 5, csStatus);
										if ( pStatic )
											m_clcList.SetItemData(index, (DWORD) pStatic);
										index++;
									}
								}
							}
						}
						tileData.Close();
					}
					staMul.Close();
				}
				staIdx.Close();
			}
		}
	}
	if ( m_cbDynamic.GetCheck() )
	{
		POSITION pos = m_pWorldItemList.GetHeadPosition();
		while ( pos )
		{
			CWorldItem * pItem = (CWorldItem *) m_pWorldItemList.GetNext(pos);
			if ( !pItem )
				continue;
			CPoint ptTest;
			ptTest.x = pItem->m_pX;
			ptTest.y = pItem->m_pY;
			if ( ! mapRect.PtInRect(ptTest) )
				continue;
			if( pItem->m_csName == "" )
			{
				// Load the name from the mulfiles.
				DWORD idx = DefLookup(pItem->m_csBaseID);
				if ( idx != 0 )
				{
					DWORD tdBlock = idx / 32;
					DWORD tdIndex = idx % 32;
					CFile tileData;
					CString csFilename;
					csFilename.Format("%stiledata.mul", csUOPath);
					if ( tileData.Open(csFilename, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone) )
					{
						tileData.Seek( (836 * 512) + (1188 * tdBlock), CFile::begin);
						CTileDataGroup block;
						tileData.Read(&block, sizeof(block));
						char szBuffer[32];
						memset(&szBuffer[0], 0x00, 32);
						memcpy(&szBuffer[0], block.m_aDataBlocks[tdIndex].m_szName, 20);
						pItem->m_csName.Format("%s", szBuffer);
						tileData.Close();
					}
				}
			}
			index = m_clcList.InsertItem(index, pItem->m_csName);
			CString csBlock, csBlockLocation, csColour, csSerial;
			csBlock.Format("%ld", pItem->GetMapBlock());
			csBlockLocation.Format("%d, %d, %d", pItem->GetBlockX(), pItem->GetBlockY(), (char)pItem->m_pZ);
			csColour.Format("%0x", pItem->m_wColor);
			csSerial.Format("%08x", pItem->m_dwSerial);
			m_clcList.SetItemData(index, (DWORD) pItem);
			m_clcList.SetItemText(index, 1, pItem->m_csBaseID);
			m_clcList.SetItemText(index, 2, csBlock);
			m_clcList.SetItemText(index, 3, csBlockLocation);
			m_clcList.SetItemText(index, 4, csColour);
			m_clcList.SetItemText(index, 5, pItem->m_bStaticFlag ? "Pending Static" : "Dynamic");
			m_clcList.SetItemText(index, 6, csSerial);
			index++;
		}
		// Add any patches that we need to use.
		pos = m_pPatchItemList.GetHeadPosition();
		while ( pos )
		{
			CUOStaticItem * pPatch = (CUOStaticItem *) m_pPatchItemList.GetNext(pos);
			if ( !pPatch )
				continue;
			CPoint ptTest;
			int bx, by;
			bx = pPatch->m_dwBlockIndex / 512;
			by = pPatch->m_dwBlockIndex % 512;
			ptTest.x = bx * 8 + pPatch->m_blockData.bX;
			ptTest.y = by * 8 + pPatch->m_blockData.bY;
			if ( ! mapRect.PtInRect(ptTest) )
				continue;
			CString csName;
			if ( pPatch->m_blockData.wArtID != 0 )
			{
				DWORD tdBlock = pPatch->m_blockData.wArtID / 32;
				DWORD tdIndex = pPatch->m_blockData.wArtID % 32;
				CFile tileData;
				CString csFilename;
				csFilename.Format("%stiledata.mul", csUOPath);
				if ( tileData.Open(csFilename, CFile::modeRead | CFile::typeBinary | CFile::shareDenyNone) )
				{
					tileData.Seek( (836 * 512) + (1188 * tdBlock), CFile::begin);
					CTileDataGroup block;
					tileData.Read(&block, sizeof(block));
					char szBuffer[32];
					memset(&szBuffer[0], 0x00, 32);
					memcpy(&szBuffer[0], block.m_aDataBlocks[tdIndex].m_szName, 20);
					csName.Format("%s", szBuffer);
					tileData.Close();
				}
			}
			index = m_clcList.InsertItem(index, csName);
			CString csBlock, csBlockLocation, csColour, csSerial, csID;
			csBlock.Format("%ld", pPatch->m_dwBlockIndex);
			csBlockLocation.Format("%d, %d, %d", pPatch->m_blockData.bX, pPatch->m_blockData.bY, (char)pPatch->m_blockData.bZ);
			csColour.Format("%0x", pPatch->m_blockData.wColor);
			csID.Format("%04x", pPatch->m_blockData.wArtID);
			m_clcList.SetItemData(index, (DWORD) pPatch);
			m_clcList.SetItemText(index, 1, csID);
			m_clcList.SetItemText(index, 2, csBlock);
			m_clcList.SetItemText(index, 3, csBlockLocation);
			m_clcList.SetItemText(index, 4, csColour);
			m_clcList.SetItemText(index, 5, "Pending Static");
			m_clcList.SetItemText(index, 6, csSerial);
			index++;
		}
	}
	//m_clcList.ShowWindow(SW_SHOW);
}

void CStaticToolDlg::OnStaticDynamic() 
{
	FillItemListControl();	
}

void CStaticToolDlg::OnStaticExisting() 
{
	FillItemListControl();	
}

void CStaticToolDlg::OnStaticRemove() 
{
	// Change selected objects from Pending Static to Dynamic and Static to Pending Dynamic
	int iSel = m_clcList.GetSelectedCount();
	POSITION pos = m_clcList.GetFirstSelectedItemPosition();
	while ( pos )
	{
		int idx = m_clcList.GetNextSelectedItem(pos);
		if ( idx == -1 )
			break;
		CObject * pData = (CObject *) m_clcList.GetItemData(idx);
		if ( pData == NULL )
		{
			// This must be an existing static
			CUOStaticItem * pStatic = new CUOStaticItem;
			CString csTmp = m_clcList.GetItemText(idx, 2);
			pStatic->m_dwBlockIndex = (DWORD) atoi(csTmp);
			csTmp = m_clcList.GetItemText(idx, 1);
			pStatic->m_blockData.wArtID = (WORD) ahextoi(csTmp);
			csTmp = m_clcList.GetItemText(idx, 3);
			CString csX, csY, csZ;
			csX = csTmp.SpanExcluding(",");
			csY = csTmp.Mid(csTmp.Find(",") + 1).SpanExcluding(",");
			csZ = csTmp.Mid(csTmp.ReverseFind(',') + 1);
			pStatic->m_blockData.bX = (BYTE) atoi(csX);
			pStatic->m_blockData.bY = (BYTE) atoi(csY);
			pStatic->m_blockData.bZ = (BYTE) atoi(csZ);
			m_clcList.SetItemData(idx, (DWORD) pStatic);
			m_clcList.SetItemText(idx, 5, "Pending Dynamic");
			pStatic->m_bFlag |= 0x1;
			if ( m_pStaticItemList.IsEmpty() )
				m_pStaticItemList.AddHead(pStatic);
			else
				m_pStaticItemList.AddTail(pStatic);
		}
		CWorldItem * pDynamic = dynamic_cast <CWorldItem *> (pData);
		if ( pDynamic )
		{
			// This is a dyanmic object
			if ( pDynamic->m_bStaticFlag )
			{
				pDynamic->m_bStaticFlag = false;
				m_clcList.SetItemText(idx, 5, "Dynamic");
			}
		}
		CUOStaticItem * pStatic = dynamic_cast <CUOStaticItem *> (pData);
		if ( pStatic )
		{
			// This is a static that we've set aside to make dyamic
			m_clcList.SetItemText(idx, 5, "Removed Static");
			pStatic->m_bFlag |= 0x2;
		}
	}
	m_bSaveNeeded = true;
	UpdatePreviewItems();
}

void CStaticToolDlg::OnStaticAdd() 
{
	// Change selected objects from Dynamic to Pending Static and Pending Dynamic to Static
	int iSel = m_clcList.GetSelectedCount();
	POSITION pos = m_clcList.GetFirstSelectedItemPosition();
	while ( pos )
	{
		int idx = m_clcList.GetNextSelectedItem(pos);
		if ( idx == -1 )
			break;
		CObject * pData = (CObject *) m_clcList.GetItemData(idx);
		if ( pData == NULL )
		{
			// This must already be a static
		}
		CWorldItem * pDynamic = dynamic_cast <CWorldItem *> (pData);
		if ( pDynamic )
		{
			// This is a dynamic object
			if ( !pDynamic->m_bStaticFlag )
			{
				pDynamic->m_bStaticFlag = true;
				m_clcList.SetItemText(idx, 5, "Pending Static");
			}
		}
		CUOStaticItem * pStatic = dynamic_cast <CUOStaticItem *> (pData);
		if ( pStatic )
		{
			// This is a static that we've tagged as dynamic or removed
			m_clcList.SetItemText(idx, 5, "Static");
			pStatic->m_bFlag = 0;
		}
	}
	m_bSaveNeeded = true;
	UpdatePreviewItems();
}

void CStaticToolDlg::OnCriteriaRegionsModify() 
{
	CStaticRegionEditDlg dlg;
	// Copy our rectangles to the dialog
	CopyRects(m_aRectExclude, dlg.m_aExcludes);
	CopyRects(m_aRectInclude, dlg.m_aIncludes);
	if ( dlg.DoModal() == IDOK )
	{
		CleanupRects();
		CopyRects(dlg.m_aExcludes, m_aRectExclude);
		CopyRects(dlg.m_aIncludes, m_aRectInclude);
	}
	UpdateMapRegions();
}

void CStaticToolDlg::OnOptionsMapwindowSetregioncolors() 
{
	CStaticRegionColorDlg dlg;
	if ( dlg.DoModal() == IDOK )
		UpdateMapRegions();
}

void CStaticToolDlg::OnOptionsMapwindowShowexcluderegions() 
{
	m_bDisplayExcludeRegions = ! m_bDisplayExcludeRegions;	
	CMenu * pMenu = this->GetMenu();
	if ( pMenu )
		pMenu->CheckMenuItem( ID_OPTIONS_MAPWINDOW_SHOWEXCLUDEREGIONS, m_bDisplayExcludeRegions ? MF_CHECKED : MF_UNCHECKED );
	Main->PutRegistryDword("StaticDisplayExcludeRegions", (DWORD) m_bDisplayIncludeRegions);
	UpdateMapRegions();
}

void CStaticToolDlg::OnUpdateOptionsMapwindowShowexcluderegions(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bDisplayExcludeRegions ? 1 : 0 );
}

void CStaticToolDlg::OnOptionsMapwindowShowincluderegions() 
{
	m_bDisplayIncludeRegions = ! m_bDisplayIncludeRegions;
	CMenu * pMenu = this->GetMenu();
	if ( pMenu )
		pMenu->CheckMenuItem( ID_OPTIONS_MAPWINDOW_SHOWINCLUDEREGIONS, m_bDisplayIncludeRegions ? MF_CHECKED : MF_UNCHECKED );
	Main->PutRegistryDword("StaticDisplayIncludeRegions", (DWORD) m_bDisplayIncludeRegions);
	UpdateMapRegions();
}

void CStaticToolDlg::OnUpdateOptionsMapwindowShowincluderegions(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bDisplayIncludeRegions ? 1 : 0 );	
}

void CStaticToolDlg::UpdateMapRegions()
{
	m_uoMap.RemoveDrawRects();
	m_iDrawRectIndex = -1;
	DWORD dwExcludeColor = Main->GetRegistryDword("StaticExcludeRegionColor", 0x00AA00);
	DWORD dwIncludeColor = Main->GetRegistryDword("StaticIncludeRegionColor", 0x0000AA);

	if ( m_bDisplayIncludeRegions )
	{
		for ( int i = 0; i < m_aRectInclude.GetSize(); i++ )
		{
			CRect * pRect = (CRect *) m_aRectInclude.GetAt(i);
			if ( pRect )
				m_uoMap.AddDrawRect((short) pRect->left, (short) pRect->top, (short) pRect->Width(), (short) pRect->Height(), 0, dwIncludeColor);
		}
	}
	if ( m_bDisplayExcludeRegions )
	{
		for ( int i = 0; i < m_aRectExclude.GetSize(); i++ )
		{
			CRect * pRect = (CRect *) m_aRectExclude.GetAt(i);
			if ( pRect )
				m_uoMap.AddDrawRect((short) pRect->left, (short) pRect->top, (short) pRect->Width(), (short) pRect->Height(), 0, dwExcludeColor);
		}
	}
	UpdateCriteriaWindow();
}

void CStaticToolDlg::CleanupRects()
{
	for ( int i = 0; i < m_aRectExclude.GetSize(); i++ )
	{
		CRect * pRect = (CRect *) m_aRectExclude.GetAt(i);
		if ( pRect )
			delete pRect;
	}
	m_aRectExclude.RemoveAll();
	for ( i = 0; i < m_aRectInclude.GetSize(); i++ )
	{
		CRect * pRect = (CRect *) m_aRectInclude.GetAt(i);
		if ( pRect )
			delete pRect;
	}
	m_aRectInclude.RemoveAll();
}

void CStaticToolDlg::CopyRects(CPtrArray &sourceArray, CPtrArray &destArray)
{
	for ( int i = 0; i < sourceArray.GetSize(); i++ )
	{
		CRect * pRect = (CRect *) sourceArray.GetAt(i);
		if ( pRect )
		{
			CRect * pNewRect = new CRect;
			pNewRect->CopyRect(pRect);
			destArray.Add((CObject *) pNewRect);
		}
	}
}

void CStaticToolDlg::SetPreviewFlags()
{
	WORD wFlags = m_Preview.GetDrawFlags() & 0xFF0F;
	wFlags |= (m_bPreviewTerrain ? 0x0010 : 0);
	wFlags |= (m_bPreviewStatics ? 0x0020 : 0);
	wFlags |= (m_bPreviewDynamics ? 0x0040 : 0);
	m_Preview.SetDrawFlags(wFlags);
}

void CStaticToolDlg::UpdatePreviewItems()
{
	m_Preview.RemoveHideStatics();
	m_Preview.RemovePreviewItems();

	POSITION pos = m_pStaticItemList.GetHeadPosition();
	while ( pos )
	{
		CUOStaticItem * pStatic = (CUOStaticItem *) m_pStaticItemList.GetNext(pos);
		if ( pStatic && pStatic->m_bFlag )
			m_Preview.AddHideStatic(pStatic->m_blockData.wArtID, pStatic->m_dwBlockIndex, pStatic->m_blockData.bX, pStatic->m_blockData.bY, pStatic->m_blockData.bZ, pStatic->m_blockData.wColor);
	}
	pos = m_pWorldItemList.GetHeadPosition();
	while ( pos )
	{
		CWorldItem * pItem = (CWorldItem *) m_pWorldItemList.GetNext(pos);
		if ( pItem && pItem->m_bStaticFlag )
			m_Preview.AddPreviewItem(pItem->m_wArtID ? pItem->m_wArtID : (WORD) DefLookup(pItem->m_csBaseID), pItem->m_pX, pItem->m_pY, pItem->m_pZ, pItem->m_wColor);
	}
	pos = m_pPatchItemList.GetHeadPosition();
	while ( pos )
	{
		CUOStaticItem * pPatch = (CUOStaticItem *) m_pPatchItemList.GetNext(pos);
		if ( pPatch )
		{
			int bx, by, x, y;
			bx = pPatch->m_dwBlockIndex / 512;
			by = pPatch->m_dwBlockIndex % 512;
			x = bx * 8 + pPatch->m_blockData.bX;
			y = by * 8 + pPatch->m_blockData.bY;
			m_Preview.AddPreviewItem(pPatch->m_blockData.wArtID, x, y, pPatch->m_blockData.bZ, pPatch->m_blockData.wColor);
		}
	}
	m_Preview.RedrawControl();
}

bool CStaticToolDlg::OnFileCleanworldfile() 
{
	CWaitCursor hourglass;
	CFileReadProgress progress;
	CStdioFile OldWorld, NewWorld;
	progress.Create(IDD_FILEREAD_PROGRESS);
	progress.SetWindowText("Cleaning Worldfile");
	progress.SetPos(0);
	progress.m_csMessage.SetWindowText("Cleaning worldfile");
	if ( ! OldWorld.Open(m_csWorldFile, CFile::modeRead | CFile::typeText | CFile::shareDenyNone ) )
	{
		AfxMessageBox("Unable to open existing worldfile", MB_OK);
		return false;
	}
	CString csNewWorld;
	csNewWorld.Format("%s.clean", m_csWorldFile);
	if ( ! NewWorld.Open(csNewWorld, CFile::modeCreate | CFile::modeWrite | CFile::typeText | CFile::shareDenyWrite ) )
	{
		AfxMessageBox("Unable to open new worldfile", MB_OK);
		OldWorld.Close();
		return false;
	}
	progress.SetRange32(0, (DWORD) OldWorld.GetLength());
	BOOL bStatus = TRUE;
	CString csLine;
	bool bWarnings = false;
	int line = 0;
	while ( bStatus )
	{
		bStatus = OldWorld.ReadString(csLine);
		if ( !bStatus )
			break;
		if ( ( line % 100 ) == 0 )
			progress.SetPos((DWORD) OldWorld.GetPosition());
		if ( csLine.Find("[WORLDITEM") == 0 )
		{
			// This is a worlditem
			// The next line should be a UID.
			CString csSerial;
			bStatus = OldWorld.ReadString(csSerial);
			if ( ! bStatus )
				break;
			if ( csSerial.Find("SERIAL=") == 0 )
			{
				// Yep...is this one of the UIDs we are getting rid of?
				DWORD dwUID = (DWORD) ahextoi(csSerial.Mid(7));
				if ( !TestUID(dwUID) )
				{
					// Keep this one in the worldfile.
					NewWorld.WriteString(csLine);
					NewWorld.WriteString("\n");
					NewWorld.WriteString(csSerial);
					NewWorld.WriteString("\n");
				}
				else
				{
					// Oops!  We were just deleting the [WORLDITEM blah] line but leaving everything else!
					while ( bStatus )
					{
						DWORD dwPos = OldWorld.GetPosition();
						bStatus = OldWorld.ReadString(csLine);
						if ( ! bStatus )
							break;
						if ( ( line % 100 ) == 0 )
							progress.SetPos((DWORD) OldWorld.GetPosition());
						if ( csLine.Find("[") == 0 )
						{
							// We've hit the next block
							OldWorld.Seek(dwPos, CFile::begin);
							break;
						}
						line++;
					}
				}
			}
			else
			{
				// Who has been messing with the worldfile format?
				bWarnings = true;
				Main->m_log.Add("Worlditem at line %ld in worldfile %s does not have expected format!", line - 1, m_csWorldFile);
				// Keep this one in the worldfile.
				NewWorld.WriteString(csLine);
				NewWorld.WriteString("\n");
				NewWorld.WriteString(csSerial);
				NewWorld.WriteString("\n");
			}
			line ++;
		}
		else
		{
			NewWorld.WriteString(csLine);
			NewWorld.WriteString("\n");
		}
		line++;
	}
	OldWorld.Close();
	NewWorld.Close();

	// Create the import file?
	
	// Cleanup
	if ( bWarnings )
		AfxMessageBox("The worldfile clean process reported warnings.  Check your axis.log file for details.", MB_OK);
	m_bCleanNeeded = false;
	return true;	
}

void CStaticToolDlg::OnFileExit() 
{
	CString csMessage;
	// Make sure we've saved everything, if applicable
	if ( m_bSaveNeeded )
	{
		csMessage.Format("Do you wish to save your patch data?");
		switch ( AfxMessageBox(csMessage, MB_YESNOCANCEL) )
		{
		case IDYES:
			OnFileSavepatchfile();
			break;
		case IDNO:
			break;
		case IDCANCEL:
			return;
		}
	}
	if ( m_bCleanNeeded )
	{
		csMessage.Format("You have moved objects from the worldfile to the statics.  Do you wish to remove these objects from the worldfile?\n\nIf you choose No here, you will not be able to clean these objects again.");
		switch (AfxMessageBox(csMessage, MB_YESNOCANCEL) )
		{
		case IDYES:
			if ( !OnFileCleanworldfile() )
				return;
			break;
		case IDNO:
			break;
		case IDCANCEL:
			return;
		}
	}
	CDialog::OnOK();
}

bool CStaticToolDlg::TestUID(DWORD dwUID)
{
	POSITION pos = m_pWorldItemList.GetHeadPosition();
	while ( pos )
	{
		CWorldItem * pItem = (CWorldItem *) m_pWorldItemList.GetNext(pos);
		if ( pItem && pItem->m_dwSerial == dwUID )
			return pItem->m_bStaticFlag ? true : false;
	}
	return false;
}

void CStaticToolDlg::OnFileCreateimportfile() 
{
	CFileReadProgress progress;
	progress.Create(IDD_FILEREAD_PROGRESS);
	// Create an import file for any statics that we converted to dynamics
	progress.SetWindowText("Creating import file");
	progress.SetPos(0);
	progress.m_csMessage.SetWindowText("Creating import file for converted statics");
	progress.SetRange32(0, (DWORD) m_pStaticItemList.GetCount());
	CString csImportFile;
	if ( m_csWorldFile != "" )
		csImportFile.Format("%s.import", m_csWorldFile);
	else
	{
#ifndef _DEBUG
		char szCwd[MAX_PATH];
		memset(&szCwd[0], 0x00, sizeof(szCwd));
		getcwd(szCwd, sizeof(szCwd));
		CFileDialog dlg(FALSE, NULL, NULL, 0, "Script Files (*.scp)|*.scp||");
		if ( dlg.DoModal() == IDOK )
			csImportFile = dlg.GetPathName();
		chdir(szCwd);
#else
		csImportFile = "c:\\sphere\\save\\static_import.scp";
#endif
	}
	CStdioFile ImportFile;
	if ( ImportFile.Open(csImportFile, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite) )
	{
		CString csLine;
		csLine.Format("// %s.import\n", m_csWorldFile);
		ImportFile.WriteString(csLine);
		time_t now = time(NULL);
		csLine.Format("// Created %s by Axis\n// This file contains scripts for objects that were converted from the client statics files.\n\n\n", asctime(localtime(&now)) );
		ImportFile.WriteString(csLine);

		int count = 1;

		POSITION pos = m_pStaticItemList.GetHeadPosition();
		while ( pos != NULL )
		{
			CUOStaticItem * pStatic = (CUOStaticItem *) m_pStaticItemList.GetNext(pos);
			if ( pStatic && (pStatic->m_bFlag & 0x01) )
			{
				int x, y, z;
				z = pStatic->m_blockData.bZ;
				x = ( (pStatic->m_dwBlockIndex / 512) * 8 ) + pStatic->m_blockData.bX;
				y = ( (pStatic->m_dwBlockIndex % 512) * 8 ) + pStatic->m_blockData.bY;
				if ( pStatic->m_blockData.wColor )
					csLine.Format("[WORLDITEM 0%x]\nSERIAL=0%08x\nCOLOR=0%04x\nATTR=08010\nP=%ld,%ld,%ld\n\n", pStatic->m_blockData.wArtID, count++, pStatic->m_blockData.wColor, x, y, z);
				else
					csLine.Format("[WORLDITEM 0%x]\nSERIAL=0%08x\nATTR=08010\nP=%ld,%ld,%ld\n\n", pStatic->m_blockData.wArtID, count++, x, y, z);
				ImportFile.WriteString(csLine);
			}
		}
		ImportFile.Close();
	}
	else
	{
		AfxMessageBox("Unable to open import file for writing");
	}	
}

void CStaticToolDlg::OnStatictoolHelp() 
{
	WinHelp(0x0200CF);	
}
