/*
 $Id: editgraymapdlg.cpp,v 1.6 2002/08/12 22:59:35 pesterle Exp $

 **********************************************************************
 * Copyright (C) Philip A. Esterle 1998-2002
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 **********************************************************************

*/

// EditGraymapDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Area.h"
#include "drewsky.h"
#include "EditGraymapDlg.h"
#include "AddValueDialog.h"
#include <math.h>
#include "RemoteConnection.h"
#include "RemoteConsole.h"
#include "EditValue.h"
#include "EditFlagsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEditGraymapDlg dialog

const char * MapTagDescriptions [] = 
{
	"Region Title",
	"Classification Group",
	"Teleport Point",
	"Map Flags",
	"Guard allegiance",
	"Chance of Weather",
	"Chance of Cold",
	"Name for locals",
	"Region Trigger",
	"Periodic Trigger",
	"Announcement",
};

CEditGraymapDlg::CEditGraymapDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CEditGraymapDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CEditGraymapDlg)
	//}}AFX_DATA_INIT
	m_wFlags = 0;
	m_bSearching = false;
	m_button = 0;
	m_dragRect = 0;
}


void CEditGraymapDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEditGraymapDlg)
	DDX_Control(pDX, IDC_REGIONRECTLIST, m_clbRectList);
	DDX_Control(pDX, IDC_REGIONTREE, m_ctcRegionTree);
	DDX_Control(pDX, IDC_MAPVALS, m_clcValues);
	DDX_Control(pDX, IDC_DRAWRECTS, m_cbDrawRects);
	DDX_Control(pDX, IDC_CURSORCOORDS, m_csCursor);
	DDX_Control(pDX, IDC_MAPCENTER, m_csCoords);
	DDX_Control(pDX, IDC_MAPDISPLAY, m_map);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEditGraymapDlg, CDialog)
	//{{AFX_MSG_MAP(CEditGraymapDlg)
	ON_NOTIFY(TVN_SELCHANGED, IDC_REGIONTREE, OnSelchangedRegiontree)
	ON_NOTIFY(TVN_ENDLABELEDIT, IDC_REGIONTREE, OnEndlabeleditRegiontree)
	ON_NOTIFY(TVN_KEYDOWN, IDC_REGIONTREE, OnKeydownRegiontree)
	ON_BN_CLICKED(IDC_NEWROOM, OnNewroom)
	ON_BN_CLICKED(IDC_NEWAREA, OnNewarea)
	ON_WM_LBUTTONDOWN()
	ON_BN_CLICKED(IDC_SETPOINT, OnSetpoint)
	ON_WM_RBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_BN_CLICKED(IDC_REMOVERECT, OnRemoverect)
	ON_NOTIFY(NM_DBLCLK, IDC_MAPVALS, OnDblclkMapvals)
	ON_BN_CLICKED(IDC_LOCATEAREA, OnLocatearea)
	ON_BN_CLICKED(IDC_LOCATEROOM, OnLocateroom)
	ON_BN_CLICKED(IDC_MOVEUP, OnMoveup)
	ON_BN_CLICKED(IDC_MOVEDOWN, OnMovedown)
	ON_WM_KEYDOWN()
	ON_BN_CLICKED(IDC_ZOOMIN, OnZoomin)
	ON_BN_CLICKED(IDC_ZOOMOUT, OnZoomout)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEditGraymapDlg message handlers

BOOL CEditGraymapDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	Main->m_pScripts->LoadMap();

	// Now we need to build the tree view
	POSITION areaPos = Main->m_pScripts->m_olAreas.GetHeadPosition();
	HTREEITEM hItem = this->m_ctcRegionTree.InsertItem( "Areas", 0, 0, TVI_ROOT, TVI_LAST );
	while ( areaPos != NULL )
	{
		CArea * pArea = (CArea *) Main->m_pScripts->m_olAreas.GetNext( areaPos );
		this->PopulateTreeView( hItem, pArea );
	}
	POSITION roomPos = Main->m_pScripts->m_olRooms.GetHeadPosition();
	hItem = this->m_ctcRegionTree.InsertItem( "Rooms", 0, 0, TVI_ROOT, TVI_LAST );
	while ( roomPos != NULL )
	{
		CRoom * pRoom = (CRoom *) Main->m_pScripts->m_olRooms.GetNext( roomPos );
		this->PopulateTreeView( hItem, pRoom );
	}

	this->m_clcValues.InsertColumn( 0, "Description", LVCFMT_LEFT, 140, 0 );
	this->m_clcValues.InsertColumn( 1, "Value", LVCFMT_LEFT, 150, 1 );

	this->m_clcValues.InsertItem( 0, "Title of the region" );
	this->m_clcValues.InsertItem( 1, "Axis classification group" );
	this->m_clcValues.InsertItem( 2, "Teleport Point" );
	this->m_clcValues.InsertItem( 3, "Map Flags" );
	this->m_clcValues.InsertItem( 4, "Guard owner title" );
	this->m_clcValues.InsertItem( 5, "Chance of weather" );
	this->m_clcValues.InsertItem( 6, "Chance of snow" );
	this->m_clcValues.InsertItem( 7, "Name of locals" );
	this->m_clcValues.InsertItem( 8, "Region Trigger" );
	this->m_clcValues.InsertItem( 9, "Periodic Trigger" );
	this->m_clcValues.InsertItem( 10, "Announcement" );

	CString csRegKey, csValue;
	csRegKey.Format("Mul%ld", VERFILE_MAP0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_map.SetCustomMulPath(VERFILE_MAP0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_MAP2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_map.SetCustomMulPath(VERFILE_MAP2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STAIDX0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_map.SetCustomMulPath(VERFILE_STAIDX0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STATICS0);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_map.SetCustomMulPath(VERFILE_STATICS0, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STAIDX2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_map.SetCustomMulPath(VERFILE_STAIDX2, csValue);
	csRegKey.Format("Mul%ld", VERFILE_STATICS2);
	csValue = Main->GetRegistryString(csRegKey, "");
	if ( csValue != "" )
		m_map.SetCustomMulPath(VERFILE_STATICS2, csValue);


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

CEditGraymapDlg::~CEditGraymapDlg()
{
}

void CEditGraymapDlg::OnSelchangedRegiontree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	HTREEITEM hItem = pNMTreeView->itemOld.hItem;
	if (hItem != NULL)
	{
		this->m_ctcRegionTree.SetItemState(hItem, 0, TVIS_BOLD);
	}
	// Populate the controls with the values of the newly selected item
	hItem = pNMTreeView->itemNew.hItem;
	if (hItem != NULL)
	{
		this->m_ctcRegionTree.SetItemState(hItem, TVIS_BOLD, TVIS_BOLD);
		CArea * pNewArea = (CArea *) this->m_ctcRegionTree.GetItemData(hItem);
		if ( pNewArea != NULL )
		{
			this->DisplayValues( pNewArea );
			this->UpdateData(false);
			this->LoadMap();
		}
	}
	*pResult = 0;
}

void CEditGraymapDlg::OnEndlabeleditRegiontree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pNMHDR;

	if (pTVDispInfo->item.pszText == NULL)
		*pResult = 0;
	else
		*pResult = 1;
	if (pTVDispInfo->item.pszText != NULL)
	{
		if (pTVDispInfo->item.pszText[0] == 0x00)
		{
			*pResult = 0;
			AfxMessageBox("Item cannot be blank", MB_OK | MB_ICONEXCLAMATION);
			return;
		}

		CArea * pArea = (CArea *) this->m_ctcRegionTree.GetItemData(pTVDispInfo->item.hItem);
		pArea->m_csName.Format("%s", pTVDispInfo->item.pszText);
	}
}

void CEditGraymapDlg::OnKeydownRegiontree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_KEYDOWN* pTVKeyDown = (TV_KEYDOWN*)pNMHDR;
	if (pTVKeyDown->wVKey == VK_DELETE)
	{
		// try to figure out which item is selected
		HTREEITEM hItem = this->m_ctcRegionTree.GetSelectedItem();
		if (hItem != NULL)
		{
			CArea * pArea = (CArea *) this->m_ctcRegionTree.GetItemData(hItem);
			if ( pArea->m_bCustom == false )
			{
				AfxMessageBox( "You cannot delete standard areas or rooms", MB_OK | MB_ICONINFORMATION );
				return;
			}
			if (pArea != NULL)
			{
				// Is this a room?
				CRoom * pRoom = dynamic_cast <CRoom*> (pArea);
				if ( pRoom )
				{
					POSITION pos = Main->m_pScripts->m_olRooms.Find( pRoom );
					if ( pos != NULL )
					{
						Main->m_pScripts->m_olRooms.Find( pRoom );
						delete pRoom;
					}
				}
				else
				{
					POSITION pos = Main->m_pScripts->m_olAreas.Find(pArea);
					if (pos != NULL)
					{
						Main->m_pScripts->m_olAreas.RemoveAt(pos);
						delete pArea;
					}
				}
			}
			this->m_ctcRegionTree.DeleteItem(hItem);
		}
		*pResult = 1;
	}
	else
		*pResult = 0;
}

void CEditGraymapDlg::OnNewroom() 
{
	HTREEITEM hParent = NULL;
	HTREEITEM hTestItem = this->m_ctcRegionTree.GetRootItem();
	while ( hTestItem != NULL )
	{
		if ( this->m_ctcRegionTree.GetItemText( hTestItem ) == "Rooms" )
		{
			hParent = hTestItem;
			break;
		}
		hTestItem = this->m_ctcRegionTree.GetNextSiblingItem( hTestItem );
	}
	CAddValueDialog dlg;
	dlg.m_csTitle = "New Room";
	if (dlg.DoModal() == IDOK)
	{
		CRoom * pRoom = new (CRoom);
		pRoom->m_csName = dlg.m_csValue;
		pRoom->m_bCustom = true;
		HTREEITEM hNewItem = this->m_ctcRegionTree.InsertItem(pRoom->m_csName, 0, 0, hParent, TVI_LAST);
		if ( Main->m_pScripts->m_olRooms.IsEmpty() )
			Main->m_pScripts->m_olRooms.AddHead( pRoom );
		else
			Main->m_pScripts->m_olRooms.AddTail( pRoom );
		this->m_ctcRegionTree.SetItemData(hNewItem, (DWORD) pRoom);
		HTREEITEM hOldSel = this->m_ctcRegionTree.GetSelectedItem();
		this->m_ctcRegionTree.SetItemState( hOldSel, 0, TVIS_BOLD );
		this->m_ctcRegionTree.SetItemState( hNewItem, TVIS_BOLD, TVIS_BOLD );
		this->m_ctcRegionTree.SelectItem( hNewItem );
		this->DisplayValues( pRoom );
		this->m_ctcRegionTree.RedrawWindow();
	}
}

void CEditGraymapDlg::OnNewarea() 
{
	CAddValueDialog dlg;
	dlg.m_csTitle = "New Area";
	HTREEITEM hParent = NULL;
	HTREEITEM hTestItem = this->m_ctcRegionTree.GetRootItem();
	while ( hTestItem != NULL )
	{
		if ( this->m_ctcRegionTree.GetItemText( hTestItem ) == "Areas" )
		{
			hParent = hTestItem;
			break;
		}
		hTestItem = this->m_ctcRegionTree.GetNextSiblingItem( hTestItem );
	}
	if (dlg.DoModal() == IDOK)
	{
		CArea * pArea = new (CArea);
		pArea->m_bCustom = true;
		pArea->m_csName = dlg.m_csValue;
		HTREEITEM hNewItem = this->m_ctcRegionTree.InsertItem(pArea->m_csName, 0, 0, hParent, TVI_LAST);
		if ( Main->m_pScripts->m_olAreas.IsEmpty() )
			Main->m_pScripts->m_olAreas.AddHead( pArea );
		else
			Main->m_pScripts->m_olAreas.AddTail( pArea );
		this->m_ctcRegionTree.SetItemData(hNewItem, (DWORD) pArea);
		HTREEITEM hOldSel = this->m_ctcRegionTree.GetSelectedItem();
		this->m_ctcRegionTree.SetItemState( hOldSel, 0, TVIS_BOLD );
		this->m_ctcRegionTree.SetItemState( hNewItem, TVIS_BOLD, TVIS_BOLD );
		this->m_ctcRegionTree.SelectItem( hNewItem );
		this->DisplayValues( pArea );
		this->m_ctcRegionTree.RedrawWindow();
	}
}

void CEditGraymapDlg::OnOK() 
{
	// Save everything
	CWaitCursor hourglass;
	Main->m_pScripts->SaveMap();
	CDialog::OnOK();
}

void CEditGraymapDlg::OnCancel() 
{
	if (AfxMessageBox("If you cancel, all your changes will be lost.\nAre you sure you wish to cancel?", MB_YESNO | MB_ICONQUESTION) == IDYES)
		CDialog::OnCancel();
	else
		return;
}

void CEditGraymapDlg::LoadMap()
{
	// Clear the existing draw objects from the map
	m_map.RemoveDrawRects();

	HTREEITEM hItem = this->m_ctcRegionTree.GetSelectedItem();
	if (hItem == NULL)
		return;
	CArea * pAreaSelf = (CArea *) this->m_ctcRegionTree.GetItemData(hItem);
	CRoom * pRoomSelf = dynamic_cast <CRoom *> (pAreaSelf);

	// Add Areas that overlap me
	POSITION pos = NULL;
	if ( pRoomSelf == NULL )
	{
		pos = Main->m_pScripts->m_olAreas.GetHeadPosition();
		//pos = Main->m_pScripts->m_olAreas.Find( pAreaSelf );
		while ( pos != NULL )
		{
			CArea * pArea = (CArea *) Main->m_pScripts->m_olAreas.GetNext( pos );
			if ( pArea && ( pArea != pAreaSelf ) && pArea->Overlaps( pAreaSelf ) && ( pAreaSelf->m_pOriginal != pArea) )
			{
				for ( int i = 0; i <= pArea->m_caRects.GetUpperBound(); i++ )
				{
					CString csRect = pArea->m_caRects.GetAt(i);
					CString csX1, csY1, csX2, csY2;
					int x1, y1, x2, y2;
					csX1 = csRect.SpanExcluding(",");
					csRect = csRect.Mid(csRect.Find(",") + 1);
					csY1 = csRect.SpanExcluding(",");
					csRect = csRect.Mid(csRect.Find(",") + 1);
					csX2 = csRect.SpanExcluding(",");
					csRect = csRect.Mid(csRect.Find(",") + 1);
					csY2 = csRect.SpanExcluding(",");
					x1 = atoi(csX1);
					y1 = atoi(csY1);
					x2 = atoi(csX2);
					y2 = atoi(csY2);
					m_map.AddDrawRect(x1, y1, x2 - x1, y2 - y1, 1, 0x000000ff);	// Make it red
				}
			}
		}
	}
	// Color my overlapping rooms
	if ( pRoomSelf )
		pos = pos = Main->m_pScripts->m_olRooms.Find( pRoomSelf );
	else
		pos = Main->m_pScripts->m_olRooms.GetHeadPosition();
	while ( pos != NULL && pAreaSelf )
	{
		CRoom * pRoom = (CRoom *) Main->m_pScripts->m_olRooms.GetNext( pos );
		if ( pRoom && pRoom->IsInside( pAreaSelf ) )
		{
			for ( int i = 0; i <= pRoom->m_caRects.GetUpperBound(); i++ )
			{
				CString csRect = pRoom->m_caRects.GetAt(i);
				CString csX1, csY1, csX2, csY2;
				int x1, y1, x2, y2;
				csX1 = csRect.SpanExcluding(",");
				csRect = csRect.Mid(csRect.Find(",") + 1);
				csY1 = csRect.SpanExcluding(",");
				csRect = csRect.Mid(csRect.Find(",") + 1);
				csX2 = csRect.SpanExcluding(",");
				csRect = csRect.Mid(csRect.Find(",") + 1);
				csY2 = csRect.SpanExcluding(",");
				x1 = atoi(csX1);
				y1 = atoi(csY1);
				x2 = atoi(csX2);
				y2 = atoi(csY2);
				m_map.AddDrawRect(x1, y1, x2 - x1, y2 - y1, 1, 0x0000ff00);
			}
		}
	}
	// Color myself
	if ( pAreaSelf )
	{
		for ( int i = 0; i <= pAreaSelf->m_caRects.GetUpperBound(); i++ )
		{
			CString csRect = pAreaSelf->m_caRects.GetAt(i);
			CString csX1, csY1, csX2, csY2;
			int x1, y1, x2, y2;
			csX1 = csRect.SpanExcluding(",");
			csRect = csRect.Mid(csRect.Find(",") + 1);
			csY1 = csRect.SpanExcluding(",");
			csRect = csRect.Mid(csRect.Find(",") + 1);
			csX2 = csRect.SpanExcluding(",");
			csRect = csRect.Mid(csRect.Find(",") + 1);
			csY2 = csRect.SpanExcluding(",");
			x1 = atoi(csX1);
			y1 = atoi(csY1);
			x2 = atoi(csX2);
			y2 = atoi(csY2);
			m_map.AddDrawRect(x1, y1, x2 - x1, y2 - y1, 1, 0x00ff0000);
		}
	}
	CString csCoords;
	this->m_csCoords.GetWindowText(csCoords);
	int x, y;
	CString csX = csCoords.Left(csCoords.Find(","));
	csCoords = csCoords.Mid(csCoords.Find(",") + 1);
	CString csY = csCoords.SpanExcluding(",");
	x = atoi(csX);
	y = atoi(csY);
	m_map.SetCenter((short) x, (short) y);
}


void CEditGraymapDlg::OnSetpoint() 
{
	CString csCoords;
	this->m_csCoords.GetWindowText(csCoords);
	HTREEITEM hItem = this->m_ctcRegionTree.GetSelectedItem();
	if (hItem != NULL)
	{
		CArea * pArea = (CArea *) this->m_ctcRegionTree.GetItemData(hItem);
		pArea = this->SetCustomArea( pArea );
		pArea->m_csPoint = csCoords;
		this->m_clcValues.SetItemText( 2, 1, csCoords );
	}
	return;
}

void CEditGraymapDlg::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CDialog::OnRButtonDown(nFlags, point);
}

void CEditGraymapDlg::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CDialog::OnLButtonDown(nFlags, point);
}

void CEditGraymapDlg::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CDialog::OnLButtonUp(nFlags, point);
}

void CEditGraymapDlg::OnMouseMove(UINT nFlags, CPoint point) 
{
	CDialog::OnMouseMove(nFlags, point);
}

void CEditGraymapDlg::OnRemoverect() 
{
	int iSel = this->m_clbRectList.GetCurSel();
	if (iSel != -1)
	{
		CString csRect;
		this->m_clbRectList.GetText(iSel, csRect);
		HTREEITEM hItem = this->m_ctcRegionTree.GetSelectedItem();
		if (hItem != NULL)
		{
			this->m_clbRectList.DeleteString(iSel);
			CArea * pArea = (CArea *) this->m_ctcRegionTree.GetItemData(hItem);
			pArea = this->SetCustomArea( pArea );
			pArea->m_caRects.RemoveAll();
			for (int i = 0; i < (this->m_clbRectList.GetCount()); i++)
			{
				this->m_clbRectList.GetText( i, csRect );
				pArea->m_caRects.Add(csRect);
			}
			LoadMap();
		}
	}
}

LRESULT CEditGraymapDlg::DefWindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	if (message == WM_USER && lParam == 1)
	{
		// Calculate the X and Y coordinates of the block that we just got sent
		int X, Y;
		X = (wParam / 512) * 8;
		Y = (wParam % 512) * 8;
		CString csCoords;
		csCoords.Format("%ld,%ld", X, Y);
		this->m_csCoords.SetWindowText(csCoords);
		this->LoadMap();;
	}
	return CDialog::DefWindowProc(message, wParam, lParam);
}

void CEditGraymapDlg::SetFlags()
{
	UpdateData(FALSE);
}

void CEditGraymapDlg::GetFlags()
{
	this->m_wFlags = 0;
}

void CEditGraymapDlg::PopulateTreeView(HTREEITEM hItem, CArea *pArea)
{
	HTREEITEM hAreaItem = this->m_ctcRegionTree.InsertItem( pArea->m_csName, 0, 0, hItem, TVI_LAST );
	this->m_ctcRegionTree.SetItemData( hAreaItem, (DWORD) pArea );
	// Now take care of the children
	POSITION pos = pArea->m_colAreas.GetHeadPosition();
	while ( pos != NULL )
	{
		CArea *pChildArea = (CArea *) pArea->m_colAreas.GetNext( pos );
		if ( pChildArea )
			this->PopulateTreeView( hAreaItem, pChildArea );
	}
	// Now take care of the rooms
	POSITION roomPos = pArea->m_colRooms.GetHeadPosition();
	while ( roomPos != NULL )
	{
		CRoom * pRoom = (CRoom *) pArea->m_colRooms.GetNext( roomPos );
		if ( pRoom )
		{
			HTREEITEM hRoomItem = this->m_ctcRegionTree.InsertItem(pRoom->m_csName, 0, 0, hAreaItem, TVI_LAST);
			this->m_ctcRegionTree.SetItemData(hRoomItem, (DWORD) pRoom);
		}
	}
}

void CEditGraymapDlg::DisplayValues(CArea *pArea)
{
	if ( pArea == NULL )
		return;

	CString csText;
	this->m_clcValues.SetItemText( 0, 1, pArea->m_csTitle );
	this->m_clcValues.SetItemText( 1, 1, pArea->m_csGroup );
	this->m_clcValues.SetItemText( 2, 1, pArea->m_csPoint );
	csText.Format("0x%04x", pArea->m_wFlags );
	this->m_clcValues.SetItemText( 3, 1, csText );
	this->m_clcValues.SetItemText( 4, 1, pArea->m_csGuardOwner );
	if ( pArea->m_iRainChance == -1 )
		csText = "";
	else
		csText.Format("%ld", pArea->m_iRainChance );
	this->m_clcValues.SetItemText( 5, 1, csText );
	if ( pArea->m_iSnowChance == -1 )
		csText = "";
	else
		csText.Format("%ld", pArea->m_iSnowChance );
	this->m_clcValues.SetItemText( 6, 1, csText );
	this->m_clcValues.SetItemText( 7, 1, pArea->m_csLocal );
	if ( pArea->m_dwTriggerID == 0xFFFFFFFF )
		csText = "";
	else
		csText.Format("%05x", pArea->m_dwTriggerID );
	this->m_clcValues.SetItemText( 8, 1, csText );
	if ( pArea->m_dwTrigPeriodic == 0xFFFFFFFF )
		csText = "";
	else
		csText.Format("%05x", pArea->m_dwTrigPeriodic );
	this->m_clcValues.SetItemText( 9, 1, csText );
	this->m_clcValues.SetItemText( 10, 1, pArea->m_csAnnouncement );

	this->m_clbRectList.ResetContent();
	for ( int i = 0; i < pArea->m_caRects.GetSize(); i++ )
		this->m_clbRectList.AddString( pArea->m_caRects.GetAt( i ) );

	if ( !m_bSearching )
		if ( pArea->m_csPoint != "" )
			this->m_csCoords.SetWindowText(pArea->m_csPoint);

	return;
}

CArea * CEditGraymapDlg::SetCustomArea(CArea *pArea)
{
	if ( pArea->m_bCustom )
		return pArea;
	// Create a new area....or a room.
	CRoom * pRoomTest = dynamic_cast <CRoom *> (pArea);
	CArea * pNewArea = NULL;
	if ( pRoomTest != NULL )
	{
		CRoom * pNewRoom = new (CRoom);
		pNewArea = ( CArea * ) pNewRoom;
		if ( Main->m_pScripts->m_olRooms.IsEmpty() )
			Main->m_pScripts->m_olRooms.AddHead( pNewRoom );
		else
			Main->m_pScripts->m_olRooms.AddTail( pNewRoom );
	}
	else
	{
		pNewArea = new (CArea);
		if ( Main->m_pScripts->m_olAreas.IsEmpty() )
			Main->m_pScripts->m_olAreas.AddHead( pNewArea );
		else
			Main->m_pScripts->m_olAreas.AddTail( pNewArea );
	}
	pNewArea->Copy(pArea);
	pNewArea->m_bCustom = true;
	pNewArea->m_pOriginal = pArea;
	HTREEITEM hParent = NULL;
	HTREEITEM hTestItem = this->m_ctcRegionTree.GetRootItem();
	CString csTestText = ( pRoomTest == NULL ? "Areas" : "Rooms" );
	while ( hTestItem != NULL )
	{
		if ( this->m_ctcRegionTree.GetItemText( hTestItem ) == csTestText )
		{
			hParent = hTestItem;
			break;
		}
		hTestItem = this->m_ctcRegionTree.GetNextSiblingItem( hTestItem );
	}
	HTREEITEM hItem = this->m_ctcRegionTree.InsertItem( pNewArea->m_csName, 0, 0, hParent, TVI_LAST );
	this->m_ctcRegionTree.SetItemData( hItem, (DWORD) pNewArea );
	HTREEITEM hOldSel = this->m_ctcRegionTree.GetSelectedItem();
	this->m_ctcRegionTree.SetItemState( hOldSel, 0, TVIS_BOLD );
	this->m_ctcRegionTree.SetItemState( hItem, TVIS_BOLD, TVIS_BOLD );
	this->m_ctcRegionTree.SelectItem( hItem );
	this->DisplayValues( pNewArea );
	this->m_ctcRegionTree.RedrawWindow();

	return pNewArea;
}

void CEditGraymapDlg::OnDblclkMapvals(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int iSel = this->m_clcValues.GetNextItem( -1, LVNI_SELECTED );
	if ( iSel == -1 )
		return;
	HTREEITEM hSelItem = this->m_ctcRegionTree.GetSelectedItem();
	if ( hSelItem == NULL )
		return;
	CArea * pArea = (CArea *) this->m_ctcRegionTree.GetItemData( hSelItem );
	CString csVal = this->m_clcValues.GetItemText( iSel, 1 );
	if ( iSel != 3 )
	{
		CEditValueDlg dlg;
		dlg.m_bTextMode = true;
		dlg.m_csCaption.Format("%s", MapTagDescriptions[iSel] );
		dlg.m_csTextValue = csVal;
		if ( dlg.DoModal() == IDOK )
		{
			pArea = this->SetCustomArea( pArea );
			int iVal = atoi( dlg.m_csTextValue );
			if ( iSel == 5 || iSel == 6 || iSel == 8 || iSel == 9 )
				csVal.Format( "%ld", iVal );
			else
				csVal = dlg.m_csTextValue;
			switch (iSel)
			{
			case 0:
				pArea->m_csTitle = csVal; break;
			case 1:
				pArea->m_csGroup = csVal; break;
			case 2:
				pArea->m_csPoint = csVal; break;
			case 4:
				pArea->m_csGuardOwner = csVal; break;
			case 5:
				pArea->m_iRainChance = iVal; break;
			case 6:
				pArea->m_iSnowChance = iVal; break;
			case 7:
				pArea->m_csLocal = csVal; break;
			case 8:
				pArea->m_dwTriggerID = iVal; break;
			case 9:
				pArea->m_dwTrigPeriodic = iVal; break;
			case 10:
				pArea->m_csAnnouncement = csVal; break;
			default:
				break;
			}
			this->m_clcValues.SetItemText( iSel, 1, csVal );
		}
	}
	else
	{
		CEditFlagsDlg dlg;
		dlg.m_csCaption.Format("%s", MapTagDescriptions[iSel] );
		dlg.m_dwFlags = pArea->m_wFlags;
		dlg.m_iFlagTable = 2;
		if ( dlg.DoModal() == IDOK )
		{
			pArea = this->SetCustomArea( pArea );
			pArea->m_wFlags = (WORD) dlg.m_dwFlags;
			csVal.Format("0x%04x", pArea->m_wFlags );
			this->m_clcValues.SetItemText( iSel, 1, csVal );
		}
	}
	return;
	*pResult = 0;
}

void CEditGraymapDlg::OnLocatearea() 
{
	CWaitCursor hourglass;
	m_bSearching = true;

	CString csPoint;
	this->m_csCoords.GetWindowText(csPoint);
	CPoint p;
	CString csTemp = csPoint.SpanExcluding(",");
	p.x = atoi( csTemp );
	csTemp = csPoint.Mid( csPoint.Find(",") + 1 );
	csTemp = csTemp.SpanExcluding(",");
	p.y = atoi( csTemp );

	CArea * pThisArea = NULL;
	POSITION pos = Main->m_pScripts->m_olAreas.GetHeadPosition();
	while ( pos != NULL )
	{
		CArea * pTest = (CArea *) Main->m_pScripts->m_olAreas.GetNext( pos );
		if ( pTest && pTest->IsInside( p ) )
			pThisArea = pTest;
	}
	if ( pThisArea )
	{
		HTREEITEM hParent = NULL;
		HTREEITEM hTestItem = this->m_ctcRegionTree.GetRootItem();
		while ( hTestItem != NULL )
		{
			if ( this->m_ctcRegionTree.GetItemText( hTestItem ) == "Areas" )
			{
				hParent = hTestItem;
				break;
			}
			hTestItem = this->m_ctcRegionTree.GetNextSiblingItem( hTestItem );
		}
		if ( hParent != NULL )
		{
			hTestItem = this->m_ctcRegionTree.GetChildItem( hParent );
			while ( hTestItem != NULL )
			{
				CArea * pTest = (CArea *) this->m_ctcRegionTree.GetItemData( hTestItem );
				if ( pTest == pThisArea )
				{
					this->DisplayValues( pTest );
					break;
				}
				hTestItem = this->m_ctcRegionTree.GetNextSiblingItem( hTestItem );
			}
			HTREEITEM hLastSel = this->m_ctcRegionTree.GetSelectedItem();
			if ( hLastSel )
				this->m_ctcRegionTree.SetItemState( hLastSel, 0, TVIS_BOLD );
			this->m_ctcRegionTree.SelectItem( hTestItem );
			if ( hTestItem )
				this->m_ctcRegionTree.SetItemState( hTestItem, TVIS_BOLD, TVIS_BOLD );
			this->m_ctcRegionTree.RedrawWindow();
			this->LoadMap();
		}
	}
	m_bSearching = false;
}

void CEditGraymapDlg::OnLocateroom() 
{
	CWaitCursor hourglass;
	m_bSearching = true;
	CString csPoint;
	this->m_csCoords.GetWindowText(csPoint);
	CPoint p;
	CString csTemp = csPoint.SpanExcluding(",");
	p.x = atoi( csTemp );
	csTemp = csPoint.Mid( csPoint.Find(",") + 1 );
	csTemp = csTemp.SpanExcluding(",");
	p.y = atoi( csTemp );

	CRoom * pThisRoom = NULL;
	POSITION pos = Main->m_pScripts->m_olRooms.GetHeadPosition();
	while ( pos != NULL )
	{
		CRoom * pTest = (CRoom *) Main->m_pScripts->m_olRooms.GetNext( pos );
		if ( pTest && pTest->IsInside( p ) )
			pThisRoom = pTest;
	}
	if ( pThisRoom )
	{
		HTREEITEM hParent = NULL;
		HTREEITEM hTestItem = this->m_ctcRegionTree.GetRootItem();
		while ( hTestItem != NULL )
		{
			if ( this->m_ctcRegionTree.GetItemText( hTestItem ) == "Rooms" )
			{
				hParent = hTestItem;
				break;
			}
			hTestItem = this->m_ctcRegionTree.GetNextSiblingItem( hTestItem );
		}
		if ( hParent != NULL )
		{
			hTestItem = this->m_ctcRegionTree.GetChildItem( hParent );
			while ( hTestItem != NULL )
			{
				CRoom * pTest = (CRoom *) this->m_ctcRegionTree.GetItemData( hTestItem );
				if ( pTest == pThisRoom )
				{
					this->DisplayValues( pTest );
					break;
				}
				hTestItem = this->m_ctcRegionTree.GetNextSiblingItem( hTestItem );
			}
			HTREEITEM hLastSel = this->m_ctcRegionTree.GetSelectedItem();
			if ( hLastSel )
				this->m_ctcRegionTree.SetItemState( hLastSel, 0, TVIS_BOLD );
			this->m_ctcRegionTree.SelectItem( hTestItem );
			if ( hTestItem )
				this->m_ctcRegionTree.SetItemState( hTestItem, TVIS_BOLD, TVIS_BOLD );
			this->m_ctcRegionTree.RedrawWindow();
			this->LoadMap();
		}
	}	
	m_bSearching = false;
}

void CEditGraymapDlg::OnMoveup() 
{
	
}

void CEditGraymapDlg::OnMovedown() 
{
	
}

void CEditGraymapDlg::OnZoomin() 
{
	m_map.ZoomIn();	
}

void CEditGraymapDlg::OnZoomout() 
{
	m_map.ZoomOut();	
}

BEGIN_EVENTSINK_MAP(CEditGraymapDlg, CDialog)
    //{{AFX_EVENTSINK_MAP(CEditGraymapDlg)
	ON_EVENT(CEditGraymapDlg, IDC_MAPDISPLAY, -605 /* MouseDown */, OnMouseDownMapdisplay, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	ON_EVENT(CEditGraymapDlg, IDC_MAPDISPLAY, -606 /* MouseMove */, OnMouseMoveMapdisplay, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	ON_EVENT(CEditGraymapDlg, IDC_MAPDISPLAY, -607 /* MouseUp */, OnMouseUpMapdisplay, VTS_I2 VTS_I2 VTS_I4 VTS_I4)
	//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CEditGraymapDlg::OnMouseDownMapdisplay(short Button, short Shift, long x, long y) 
{
	short X, Y;
	X = (short) x;
	Y = (short) y;
	m_map.CtrlToMap(&X, &Y);
	m_button = Button;
	m_pTopLeft.x = X;
	m_pTopLeft.y = Y;

	switch (Button)
	{
	case 0:
		break;
	case 1:
		break;
	case 2:
		// Right button
		break;
	}
}

void CEditGraymapDlg::OnMouseMoveMapdisplay(short Button, short Shift, long x, long y) 
{
	short X, Y;
	X = (short) x;
	Y = (short) y;
	m_map.CtrlToMap(&X, &Y);
	m_pBottomRight.x = X;
	m_pBottomRight.y = Y;
	CString csCoords;
	csCoords.Format("%ld, %ld", X, Y);
	m_csCursor.SetWindowText(csCoords);
	if ( Button )
	{
		if ( m_dragRect != -1 )
			m_map.RemoveDrawRectAt(m_dragRect);
		m_dragRect = m_map.AddDrawRect((short)m_pTopLeft.x, (short)m_pTopLeft.y, (short)(m_pBottomRight.x - m_pTopLeft.x), (short)(m_pBottomRight.y - m_pTopLeft.y), 2, 0x00FFFFFF);
	}
}

void CEditGraymapDlg::OnMouseUpMapdisplay(short Button, short Shift, long x, long y) 
{
	short X, Y;
	X = (short) x;
	Y = (short) y;
	m_map.CtrlToMap(&X, &Y);
	m_pBottomRight.x = X;
	m_pBottomRight.y = Y;
	if ( m_dragRect != -1 )
		m_map.RemoveDrawRectAt(m_dragRect);
	m_dragRect = -1;

	switch (m_button)
	{
	case 0:
		break;
	case 1:
		// Left button
		{
			// Are we drawing rects or no?
			if ( m_cbDrawRects.GetCheck() )
			{
				if ( m_pTopLeft != m_pBottomRight )
				{
					HTREEITEM hItem = this->m_ctcRegionTree.GetSelectedItem();
					if (hItem != NULL)
					{
						CRect rect;
						rect.SetRect(m_pTopLeft, m_pBottomRight);
						CArea * pArea = (CArea *) this->m_ctcRegionTree.GetItemData(hItem);
						pArea = this->SetCustomArea( pArea );
						CString csNewRect;
						csNewRect.Format("%ld,%ld,%ld,%ld", rect.left, rect.top, rect.right, rect.bottom);
						pArea->m_caRects.Add(csNewRect);
						// Add this new rect to the Rectangles window
						this->m_clbRectList.AddString(csNewRect);
						// Refresh the map
						LoadMap();
					}
				}
			}
			else
			{
				// Not drawing rects
			}
		}
		break;
	case 2:
		// Right button
		{
			// If the top left and bottom right points are the same, we're re-centering without zooming
			if ( m_pTopLeft == m_pBottomRight )
			{
				CString csCoords;
				csCoords.Format("%ld, %ld", X, Y);
				m_csCoords.SetWindowText(csCoords);
				m_map.SetCenter(X, Y);
			}
			else
			{
				// Recenter and zoom
				short xc, yc;
				xc = (m_pBottomRight.x + m_pTopLeft.x) / 2;
				yc = (m_pBottomRight.y + m_pTopLeft.y) / 2;
				m_map.SetCenter(xc, yc);
				// Determine which zoom level is most appropriate
				CRect clientRect;
				m_map.GetClientRect(&clientRect);
				int wc, wm;
				wc = clientRect.Width();
				wm = m_pBottomRight.y - m_pTopLeft.y;
				if ( wm > 8 )
				{
					short scale = 0;
					if ( wm / wc > 8 )
						scale = -3;
					else if ( wm / wc > 4 )
						scale = -2;
					else if ( wm / wc > 2 )
						scale = -1;
					else if ( wc / wm > 128 )
						scale = 7;
					else if ( wc / wm > 64 )
						scale = 6;
					else if ( wc / wm > 32 )
						scale = 5;
					else if ( wc / wm > 16 )
						scale = 4;
					else if ( wc / wm > 8 )
						scale = 3;
					else if ( wc / wm > 4 )
						scale = 2;
					else if ( wc / wm > 2 )
						scale = 1;
					m_map.SetZoomLevel(scale);
				}
			}
		}
		break;
	}
	m_button = Button;	
}
